"use strict";
/*!
 * Copyright 2014 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */Object.defineProperty(exports,"__esModule",{value:!0}),exports.Subscription=exports.AckResponses=exports.AckError=void 0;const events_1=require("events"),extend=require("extend"),snakeCase=require("lodash.snakecase"),iam_1=require("./iam"),snapshot_1=require("./snapshot"),subscriber_1=require("./subscriber"),util_1=require("./util");var subscriber_2=require("./subscriber");Object.defineProperty(exports,"AckError",{enumerable:!0,get:function(){return subscriber_2.AckError}}),Object.defineProperty(exports,"AckResponses",{enumerable:!0,get:function(){return subscriber_2.AckResponses}});class Subscription extends events_1.EventEmitter{constructor(e,t,s){super(),s=s||{},this.pubsub=e,this.request=e.request.bind(e),this.name=Subscription.formatName_(this.projectId,t),this.topic=s.topic,this.iam=new iam_1.IAM(e,this.name),this._subscriber=new subscriber_1.Subscriber(this,s),this._subscriber.on("error",(e=>this.emit("error",e))).on("debug",(e=>this.emit("debug",e))).on("message",(e=>this.emit("message",e))).on("close",(()=>this.emit("close"))),this._listen()}get isOpen(){return!(!this._subscriber||!this._subscriber.isOpen)}get projectId(){return this.pubsub&&this.pubsub.projectId||"{{projectId}}"}close(e){this._subscriber.close().then((()=>e()),e)}create(e,t){if(!this.topic)throw new Error("Subscriptions can only be created when accessed through Topics");const s=this.name.split("/").pop(),i="object"==typeof e?e:{};t="function"==typeof e?e:t,this.pubsub.createSubscription(this.topic,s,i,((e,s,i)=>{e?t(e,null,i):(Object.assign(this,s),t(null,this,i))}))}createSnapshot(e,t,s){if("string"!=typeof e)throw new Error("A name is required to create a snapshot.");const i="object"==typeof t?t:{};s="function"==typeof t?t:s;const o=this.snapshot(e),n={name:o.name,subscription:this.name};this.request({client:"SubscriberClient",method:"createSnapshot",reqOpts:n,gaxOpts:i},((e,t)=>{e?s(e,null,t):(o.metadata=t,s(null,o,t))}))}delete(e,t){const s="object"==typeof e?e:{};t="function"==typeof e?e:t;const i={subscription:this.name};this.isOpen&&this._subscriber.close(),this.request({client:"SubscriberClient",method:"deleteSubscription",reqOpts:i,gaxOpts:s},t)}detached(e){this.getMetadata(((t,s)=>{t?e(t):e(null,s.detached)}))}exists(e){this.getMetadata((t=>{t?5!==t.code?e(t):e(null,!1):e(null,!0)}))}get(e,t){const s="object"==typeof e?e:{};t="function"==typeof e?e:t;const i=!!s.autoCreate&&this.topic;delete s.autoCreate,this.getMetadata(s,((e,o)=>{e?5===e.code&&i?this.create({gaxOpts:s},t):t(e,null,o):t(null,this,o)}))}getMetadata(e,t){const s="object"==typeof e?e:{};t="function"==typeof e?e:t;const i={subscription:this.name};this.request({client:"SubscriberClient",method:"getSubscription",reqOpts:i,gaxOpts:s},((e,s)=>{e||(this.metadata=s),t(e,s)}))}modifyPushConfig(e,t,s){const i="object"==typeof t?t:{};s="function"==typeof t?t:s;const o={subscription:this.name,pushConfig:e};this.request({client:"SubscriberClient",method:"modifyPushConfig",reqOpts:o,gaxOpts:i},s)}open(){this._subscriber.isOpen||this._subscriber.open()}seek(e,t,s){const i="object"==typeof t?t:{};s="function"==typeof t?t:s;const o={subscription:this.name};if("string"==typeof e)o.snapshot=snapshot_1.Snapshot.formatName_(this.pubsub.projectId,e);else{if("[object Date]"!==Object.prototype.toString.call(e))throw new Error("Either a snapshot name or Date is needed to seek to.");{const t=e.getTime();o.time={seconds:Math.floor(t/1e3),nanos:1e3*Math.floor(t%1e3)}}}this.request({client:"SubscriberClient",method:"seek",reqOpts:o,gaxOpts:i},s)}setMetadata(e,t,s){const i="object"==typeof t?t:{};s="function"==typeof t?t:s;const o=Subscription.formatMetadata_(e),n=Object.keys(o).map(snakeCase);o.name=this.name;const r={subscription:o,updateMask:{paths:n}};this.request({client:"SubscriberClient",method:"updateSubscription",reqOpts:r,gaxOpts:i},s)}setOptions(e){this._subscriber.setOptions(e)}snapshot(e){return this.pubsub.snapshot.call(this,e)}_listen(){this.on("newListener",(e=>{this.isOpen||"message"!==e||this._subscriber.open()})),this.on("removeListener",(()=>{this.isOpen&&0===this.listenerCount("message")&&this._subscriber.close()}))}
/*!
     * Formats Subscription metadata.
     *
     * @private
     */static formatMetadata_(e){const t=extend(!0,{},e);return"number"==typeof e.messageRetentionDuration&&(t.messageRetentionDuration={seconds:e.messageRetentionDuration,nanos:0}),e.pushEndpoint&&(t.pushConfig={pushEndpoint:e.pushEndpoint},delete t.pushEndpoint),e.oidcToken&&(t.pushConfig={...t.pushConfig,oidcToken:e.oidcToken},delete t.oidcToken),t}
/*!
     * Format the name of a subscription. A subscription's full name is in the
     * format of projects/{projectId}/subscriptions/{subName}.
     *
     * @private
     */static formatName_(e,t){return t.indexOf("/")>-1?t:"projects/"+e+"/subscriptions/"+t}}exports.Subscription=Subscription,
/*! Developer Documentation
 *
 * All async methods (except for streams) will return a Promise in the event
 * that a callback is omitted.
 */
(0,util_1.promisifySome)(Subscription,Subscription.prototype,["close","create","createSnapshot","delete","detached","exists","get","getMetadata","modifyPushConfig","seek","setMetadata"]);