"use strict";
/*!
 * Copyright 2018 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */Object.defineProperty(exports,"__esModule",{value:!0}),exports.MessageStream=exports.ChannelError=exports.StatusError=void 0;const promisify_1=require("@google-cloud/promisify"),google_gax_1=require("google-gax"),isStreamEnded=require("is-stream-ended"),stream_1=require("stream"),pull_retry_1=require("./pull-retry"),default_options_1=require("./default-options"),temporal_1=require("./temporal"),exponential_retry_1=require("./exponential-retry"),debug_1=require("./debug"),KEEP_ALIVE_INTERVAL=3e4,PULL_TIMEOUT=require("./v1/subscriber_client_config.json").interfaces["google.pubsub.v1.Subscriber"].methods.StreamingPull.timeout_millis,DEFAULT_OPTIONS={highWaterMark:0,maxStreams:default_options_1.defaultOptions.subscription.maxStreams,timeout:3e5,retryMinBackoff:temporal_1.Duration.from({millis:100}),retryMaxBackoff:temporal_1.Duration.from({seconds:60})};class StatusError extends Error{constructor(e){super(e.details),this.code=e.code,this.details=e.details,this.metadata=e.metadata}}exports.StatusError=StatusError;class ChannelError extends Error{constructor(e){super(`Failed to connect to channel. Reason: ${process.env.DEBUG_GRPC?e.stack:e.message}`),this.code=e.message.includes("deadline")?google_gax_1.grpc.status.DEADLINE_EXCEEDED:google_gax_1.grpc.status.UNKNOWN,this.details=e.message,this.metadata=new google_gax_1.grpc.Metadata}}exports.ChannelError=ChannelError;class MessageStream extends stream_1.PassThrough{constructor(e,t={}){super({objectMode:!0,highWaterMark:(t=Object.assign({},DEFAULT_OPTIONS,t)).highWaterMark}),this._options=t,this._retrier=new exponential_retry_1.ExponentialRetry(t.retryMinBackoff,t.retryMaxBackoff),this._streams=[];for(let e=0;e<t.maxStreams;e++)this._streams.push({});this._subscriber=e}async start(){await this._fillStreamPool(),this._keepAliveHandle=setInterval((()=>this._keepAlive()),3e4),this._keepAliveHandle.unref()}setStreamAckDeadline(e){const t={streamAckDeadlineSeconds:e.totalOf("second")};for(const e of this._streams)e.stream&&e.stream.write(t)}_destroy(e,t){this._keepAliveHandle&&clearInterval(this._keepAliveHandle),this._retrier.close();for(let e=0;e<this._streams.length;e++){this._streams[e].stream&&this._removeStream(e)}t(e)}_replaceStream(e,t){this._removeStream(e),this._setHighWaterMark(t);const s=this._streams[e];s.stream=t,s.receivedStatus=!1,t.on("error",(t=>this._onError(e,t))).once("status",(t=>this._onStatus(e,t))).on("data",(t=>this._onData(e,t)))}_onData(e,t){const s=this._streams[e];this._retrier.reset(s),this.emit("data",t)}async _fillStreamPool(){if(this.destroyed)return;let e;try{e=await this._getClient()}catch(e){const t=e;this.destroy(t)}const t=[];for(let s=0;s<this._streams.length;s++)t.push(this._fillOne(s,e));await Promise.all(t);try{await this._waitForClientReady(e)}catch(e){const t=e;this.destroy(t)}}async _fillOne(e,t){if(this.destroyed)return;if(this._streams[e].stream)return;if(!t)try{t=await this._getClient()}catch(e){const t=e;return void this.destroy(t)}const s=Date.now()+PULL_TIMEOUT,r={subscription:this._subscriber.name,streamAckDeadlineSeconds:this._subscriber.ackDeadline,maxOutstandingMessages:this._subscriber.useLegacyFlowControl?0:this._subscriber.maxMessages,maxOutstandingBytes:this._subscriber.useLegacyFlowControl?0:this._subscriber.maxBytes},a={headers:{"x-goog-request-params":"subscription="+this._subscriber.name}},i=t.streamingPull({deadline:s,otherArgs:a});this._replaceStream(e,i),i.write(r)}async _getClient(){const e=await this._subscriber.getClient();return e.initialize(),e.subscriberStub}_keepAlive(){this._streams.forEach((e=>{!e.receivedStatus&&e.stream&&e.stream.write({})}))}_activeStreams(){return this._streams.reduce(((e,t)=>(t.stream?1:0)+e),0)}_onEnd(e,t){this._removeStream(e),pull_retry_1.PullRetry.retry(t)?(this.emit("debug",new debug_1.DebugMessage(`Subscriber stream ${e} has ended with status ${t.code}; will be retried.`)),pull_retry_1.PullRetry.resetFailures(t)&&this._retrier.reset(this._streams[e]),this._retrier.retryLater(this._streams[e],(()=>{this._fillOne(e)}))):0===this._activeStreams()&&(this.emit("debug",new debug_1.DebugMessage(`Subscriber stream ${e} has ended with status ${t.code}; will not be retried.`)),this.destroy(new StatusError(t)))}async _onError(e,t){await(0,promisify_1.promisify)(process.nextTick)();const s=t.code,r=this._streams[e],a=!r.stream||r.stream&&!r.receivedStatus;"number"==typeof s&&a||this.emit("error",t)}_onStatus(e,t){if(this.destroyed)return;const s=this._streams[e];s.receivedStatus=!0,s.stream&&(isStreamEnded(s.stream)?this._onEnd(e,t):(s.stream.once("end",(()=>this._onEnd(e,t))),s.stream.push(null)))}_removeStream(e){const t=this._streams[e];t.stream&&(t.stream.unpipe(this),t.stream.cancel(),t.stream=void 0,t.receivedStatus=void 0)}_setHighWaterMark(e){e._readableState.highWaterMark=this._options.highWaterMark}async _waitForClientReady(e){const t=Date.now()+this._options.timeout;try{await(0,promisify_1.promisify)(e.waitForReady).call(e,t)}catch(e){throw new ChannelError(e)}}}exports.MessageStream=MessageStream;