"use strict";
/*!
 * Copyright 2018 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */Object.defineProperty(exports,"__esModule",{value:!0}),exports.Subscriber=exports.Message=exports.AckError=exports.AckResponses=void 0;const precise_date_1=require("@google-cloud/precise-date"),projectify_1=require("@google-cloud/projectify"),promisify_1=require("@google-cloud/promisify"),events_1=require("events"),api_1=require("@opentelemetry/api"),semantic_conventions_1=require("@opentelemetry/semantic-conventions"),histogram_1=require("./histogram"),lease_manager_1=require("./lease-manager"),message_queues_1=require("./message-queues"),message_stream_1=require("./message-stream"),default_options_1=require("./default-options"),opentelemetry_tracing_1=require("./opentelemetry-tracing"),temporal_1=require("./temporal");exports.AckResponses={PermissionDenied:"PERMISSION_DENIED",FailedPrecondition:"FAILED_PRECONDITION",Success:"SUCCESS",Invalid:"INVALID",Other:"OTHER"};class AckError extends Error{constructor(e,s){let t=`${e}`;s&&(t+=` : ${s}`),super(t),this.errorCode=e}}exports.AckError=AckError;class Message{constructor(e,{ackId:s,message:t,deliveryAttempt:i}){this.ackId=s,this.attributes=t.attributes||{},this.data=t.data,this.deliveryAttempt=Number(i||0),this.id=t.messageId,this.orderingKey=t.orderingKey,this.publishTime=new precise_date_1.PreciseDate(t.publishTime),this.received=Date.now(),this._handled=!1,this._length=this.data.length,this._subscriber=e}get length(){return this._length}ackFailed(e){this._ackFailed=e}ack(){this._handled||(this._handled=!0,this._subscriber.ack(this))}async ackWithResponse(){if(!this._subscriber.isExactlyOnceDelivery)return this.ack(),exports.AckResponses.Success;if(this._ackFailed)throw this._ackFailed;if(this._handled)return exports.AckResponses.Invalid;this._handled=!0;try{return await this._subscriber.ackWithResponse(this)}catch(e){throw this.ackFailed(e),e}}modAck(e){this._handled||this._subscriber.modAck(this,e)}async modAckWithResponse(e){if(!this._subscriber.isExactlyOnceDelivery)return this.modAck(e),exports.AckResponses.Success;if(this._ackFailed)throw this._ackFailed;if(this._handled)return exports.AckResponses.Invalid;try{return await this._subscriber.modAckWithResponse(this,e)}catch(e){throw this.ackFailed(e),e}}nack(){this._handled||(this._handled=!0,this._subscriber.nack(this))}async nackWithResponse(){if(!this._subscriber.isExactlyOnceDelivery)return this.nack(),exports.AckResponses.Success;if(this._ackFailed)throw this._ackFailed;if(this._handled)return exports.AckResponses.Invalid;this._handled=!0;try{return await this._subscriber.nackWithResponse(this)}catch(e){throw this.ackFailed(e),e}}}exports.Message=Message;const minAckDeadlineForExactlyOnceDelivery=temporal_1.Duration.from({seconds:60});class Subscriber extends events_1.EventEmitter{constructor(e,s={}){super(),this.ackDeadline=default_options_1.defaultOptions.subscription.ackDeadline,this.maxMessages=default_options_1.defaultOptions.subscription.maxOutstandingMessages,this.maxBytes=default_options_1.defaultOptions.subscription.maxOutstandingBytes,this.useLegacyFlowControl=!1,this.isOpen=!1,this._useOpentelemetry=!1,this._histogram=new histogram_1.Histogram({min:10,max:600}),this._latencies=new histogram_1.Histogram,this._subscription=e,this.setOptions(s)}updateAckDeadline(e){let s=this.ackDeadline;e&&(this._histogram.add(e),s=this._histogram.percentile(99));const[t,i]=this.getMinMaxDeadlines();t&&(s=Math.max(s,t.totalOf("second"))),i&&(s=Math.min(s,i.totalOf("second"))),this.ackDeadline=s}getMinMaxDeadlines(){var e,s;const t=this.isExactlyOnceDelivery?minAckDeadlineForExactlyOnceDelivery:default_options_1.defaultOptions.subscription.minAckDeadline,i=default_options_1.defaultOptions.subscription.maxAckDeadline;return[null!==(e=this._options.minAckDeadline)&&void 0!==e?e:t,null!==(s=this._options.maxAckDeadline)&&void 0!==s?s:i]}get isExactlyOnceDelivery(){return!!this.subscriptionProperties&&!!this.subscriptionProperties.exactlyOnceDeliveryEnabled}setSubscriptionProperties(e){const s=this.isExactlyOnceDelivery;if(this.subscriptionProperties=e,s!==this.isExactlyOnceDelivery){this.updateAckDeadline();const e=this.isExactlyOnceDelivery?60:10;this._stream.setStreamAckDeadline(temporal_1.Duration.from({seconds:e}))}}get modAckLatency(){const e=this._latencies.percentile(99);let s=0;return this._modAcks&&(s=this._modAcks.maxMilliseconds),1e3*e+s}get name(){if(!this._name){const{name:e,projectId:s}=this._subscription;this._name=(0,projectify_1.replaceProjectIdToken)(e,s)}return this._name}async ack(e){const s=(Date.now()-e.received)/1e3;this.updateAckDeadline(s);this._acks.add(e).catch((()=>{})),await this._acks.onFlush(),this._inventory.remove(e)}async ackWithResponse(e){const s=(Date.now()-e.received)/1e3;return this.updateAckDeadline(s),await this._acks.add(e),this._inventory.remove(e),exports.AckResponses.Success}async close(){this.isOpen&&(this.isOpen=!1,this._stream.destroy(),this._inventory.clear(),await this._waitForFlush(),this.emit("close"),this._acks.close(),this._modAcks.close())}async getClient(){const e=this._subscription.pubsub,[s]=await(0,promisify_1.promisify)(e.getClient_).call(e,{client:"SubscriberClient"});return s}async modAck(e,s){const t=Date.now();this._modAcks.add(e,s).catch((()=>{})),await this._modAcks.onFlush();const i=(Date.now()-t)/1e3;this._latencies.add(i)}async modAckWithResponse(e,s){const t=Date.now();await this._modAcks.add(e,s);const i=(Date.now()-t)/1e3;return this._latencies.add(i),exports.AckResponses.Success}async nack(e){await this.modAck(e,0),this._inventory.remove(e)}async nackWithResponse(e){return await this.modAckWithResponse(e,0)}open(){const{batching:e,flowControl:s,streamingOptions:t}=this._options;this._acks=new message_queues_1.AckQueue(this,e),this._modAcks=new message_queues_1.ModAckQueue(this,e),this._inventory=new lease_manager_1.LeaseManager(this,s),this._stream=new message_stream_1.MessageStream(this,t),this._stream.on("error",(e=>this.emit("error",e))).on("debug",(e=>this.emit("debug",e))).on("data",(e=>this._onData(e))).once("close",(()=>this.close())),this._inventory.on("full",(()=>this._stream.pause())).on("free",(()=>this._stream.resume())),this._stream.start().catch((e=>{this.emit("error",e),this.close()})),this.isOpen=!0}setOptions(e){this._options=e,this._useOpentelemetry=e.enableOpenTelemetryTracing||!1;const s=e.ackDeadline;if(void 0!==s&&(this.ackDeadline=s,e.minAckDeadline=temporal_1.Duration.from({seconds:s}),e.maxAckDeadline=temporal_1.Duration.from({seconds:s})),this.useLegacyFlowControl=e.useLegacyFlowControl||!1,e.flowControl){this.maxMessages=e.flowControl.maxMessages||default_options_1.defaultOptions.subscription.maxOutstandingMessages,this.maxBytes=e.flowControl.maxBytes||default_options_1.defaultOptions.subscription.maxOutstandingBytes,e.streamingOptions||(e.streamingOptions={});const{maxStreams:s=default_options_1.defaultOptions.subscription.maxStreams}=e.streamingOptions;e.streamingOptions.maxStreams=Math.min(s,this.maxMessages)}}_constructSpan(e){if(!this._useOpentelemetry||!e.attributes||!e.attributes.googclient_OpenTelemetrySpanContext)return;const s=e.attributes.googclient_OpenTelemetrySpanContext,t=s?JSON.parse(s):void 0,i={ackId:e.ackId,deliveryAttempt:e.deliveryAttempt,[semantic_conventions_1.SemanticAttributes.MESSAGING_SYSTEM]:"pubsub",[semantic_conventions_1.SemanticAttributes.MESSAGING_OPERATION]:"process",[semantic_conventions_1.SemanticAttributes.MESSAGING_DESTINATION]:this.name,[semantic_conventions_1.SemanticAttributes.MESSAGING_DESTINATION_KIND]:"topic",[semantic_conventions_1.SemanticAttributes.MESSAGING_MESSAGE_ID]:e.id,[semantic_conventions_1.SemanticAttributes.MESSAGING_PROTOCOL]:"pubsub",[semantic_conventions_1.SemanticAttributes.MESSAGING_MESSAGE_PAYLOAD_SIZE_BYTES]:e.data.length,"messaging.pubsub.received_at":e.received,"messaging.pubsub.acknowlege_id":e.ackId,"messaging.pubsub.delivery_attempt":e.deliveryAttempt},n=`${this.name} process`;return t?(0,opentelemetry_tracing_1.createSpan)(n.trim(),api_1.SpanKind.CONSUMER,i,t):void 0}_onData(e){e.subscriptionProperties&&this.setSubscriptionProperties(e.subscriptionProperties);const{receivedMessages:s}=e;for(const e of s){const s=new Message(this,e),t=this._constructSpan(s);this.isOpen?this.isExactlyOnceDelivery?s.modAckWithResponse(this.ackDeadline).then((()=>{this._inventory.add(s)})).catch((()=>{this._discardMessage(s)})):(s.modAck(this.ackDeadline),this._inventory.add(s)):s.nack(),t&&t.end()}}_discardMessage(e){}async _waitForFlush(){const e=[];this._acks.numPendingRequests&&(e.push(this._acks.onFlush()),this._acks.flush()),this._modAcks.numPendingRequests&&(e.push(this._modAcks.onFlush()),this._modAcks.flush()),this._acks.numInFlightRequests&&e.push(this._acks.onDrain()),this._modAcks.numInFlightRequests&&e.push(this._modAcks.onDrain()),await Promise.all(e)}}exports.Subscriber=Subscriber;