#!/usr/bin/env node
"use strict";
/**
 * @license
 * Copyright 2020 gRPC authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */Object.defineProperty(exports,"__esModule",{value:!0});const fs=require("fs"),path=require("path"),Protobuf=require("protobufjs"),yargs=require("yargs"),camelCase=require("lodash.camelcase"),util_1=require("../src/util"),templateStr="%s",useNameFmter=({outputTemplate:e,inputTemplate:t})=>{if(e===t)throw new Error("inputTemplate and outputTemplate must differ");return{outputName:t=>e.replace("%s",t),inputName:e=>t.replace("%s",e)}};class TextFormatter{constructor(){this.indentText="  ",this.indentValue=0,this.textParts=[]}indent(){this.indentValue+=1}unindent(){this.indentValue-=1}writeLine(e){for(let e=0;e<this.indentValue;e+=1)this.textParts.push(this.indentText);this.textParts.push(e),this.textParts.push("\n")}getFullText(){return this.textParts.join("")}}function compareName(e,t){return e.name<t.name?-1:e.name>t.name?1:0}function isNamespaceBase(e){return Array.isArray(e.nestedArray)}function stripLeadingPeriod(e){return e.startsWith(".")?e.substring(1):e}function getImportPath(e){return e.parent instanceof Protobuf.Type?getImportPath(e.parent):stripLeadingPeriod(e.fullName).replace(/\./g,"/")}function getPath(e){return stripLeadingPeriod(e.fullName).replace(/\./g,"/")+".ts"}function getPathToRoot(e){const t=stripLeadingPeriod(e.fullName).split(".").length-1;if(0===t)return"./";let n="";for(let e=0;e<t;e++)n+="../";return n}function getRelativeImportPath(e,t){return getPathToRoot(e)+getImportPath(t)}function getTypeInterfaceName(e){return e.fullName.replace(/\./g,"_")}function getImportLine(e,t,n){const r=void 0===t?"./"+getImportPath(e):getRelativeImportPath(t,e),{outputName:i,inputName:o}=useNameFmter(n),a=getTypeInterfaceName(e);let s;if(e.parent instanceof Protobuf.Type)if(e instanceof Protobuf.Type||e instanceof Protobuf.Enum)s=`${o(a)}, ${i(a)}`;else{if(!(e instanceof Protobuf.Service))throw new Error("Invalid object passed to getImportLine");s=`${a}Client, ${a}Definition`}else if(e instanceof Protobuf.Type||e instanceof Protobuf.Enum)s=`${o(e.name)} as ${o(a)}, ${i(e.name)} as ${i(a)}`;else{if(!(e instanceof Protobuf.Service))throw new Error("Invalid object passed to getImportLine");s=`${e.name}Client as ${a}Client, ${e.name}Definition as ${a}Definition`}return`import type { ${s} } from '${r}';`}function getChildMessagesAndEnums(e){const t=[];for(const n of e.nestedArray)(n instanceof Protobuf.Type||n instanceof Protobuf.Enum)&&t.push(n),isNamespaceBase(n)&&t.push(...getChildMessagesAndEnums(n));return t}function formatComment(e,t,n){if(t||(null==n?void 0:n.deprecated)){if(e.writeLine("/**"),t)for(const n of t.split("\n"))e.writeLine(` * ${n.replace(/\*\//g,"* /")}`);(null==n?void 0:n.deprecated)&&e.writeLine(" * @deprecated"),e.writeLine(" */")}}const typeBrandHint="This field is a type brand and is not populated at runtime. Instances of this type should be created using type assertions.\nhttps://github.com/grpc/grpc-node/pull/2281";function formatTypeBrand(e,t){formatComment(e,typeBrandHint),e.writeLine(`__type: '${t.fullName}'`)}function getTypeNamePermissive(e,t,n,r,i){const{inputName:o}=useNameFmter(i);switch(e){case"double":case"float":return"number | string";case"int32":case"uint32":case"sint32":case"fixed32":case"sfixed32":return"number";case"int64":case"uint64":case"sint64":case"fixed64":case"sfixed64":return"number | string | Long";case"bool":return"boolean";case"string":return"string";case"bytes":return"Buffer | Uint8Array | string";default:if(null===t)throw new Error("Found field with no usable type");const e=getTypeInterfaceName(t);return t instanceof Protobuf.Type?n||r?o(e):`${o(e)} | null`:o(e)}}function getFieldTypePermissive(e,t){const n=getTypeNamePermissive(e.type,e.resolvedType,e.repeated,e.map,t);if(e instanceof Protobuf.MapField){return`{[key: ${"string"===e.keyType?"string":"number"}]: ${n}}`}return n}function generatePermissiveMessageInterface(e,t,n,r){const{inputName:i}=useNameFmter(n);if(n.includeComments&&formatComment(e,t.comment,t.options),".google.protobuf.Any"===t.fullName)return e.writeLine(`export type ${i("Any")} = AnyExtension | {`),e.writeLine("  type_url: string;"),e.writeLine("  value: Buffer | Uint8Array | string;"),void e.writeLine("}");e.writeLine(`export interface ${i(null!=r?r:t.name)} {`),e.indent();for(const r of t.fieldsArray){const t=r.repeated?"[]":"",i=getFieldTypePermissive(r,n);n.includeComments&&formatComment(e,r.comment,r.options),e.writeLine(`'${r.name}'?: (${i})${t};`)}for(const r of t.oneofsArray){const t=r.fieldsArray.map((e=>`"${e.name}"`)).join("|");n.includeComments&&formatComment(e,r.comment,r.options),e.writeLine(`'${r.name}'?: ${t};`)}n.inputBranded&&formatTypeBrand(e,t),e.unindent(),e.writeLine("}")}function getTypeNameRestricted(e,t,n,r,i){const{outputName:o}=useNameFmter(i);switch(e){case"double":case"float":return i.json?"number | string":"number";case"int32":case"uint32":case"sint32":case"fixed32":case"sfixed32":return"number";case"int64":case"uint64":case"sint64":case"fixed64":case"sfixed64":return i.longs===Number?"number":i.longs===String?"string":"Long";case"bool":return"boolean";case"string":return"string";case"bytes":return i.bytes===Array?"Uint8Array":i.bytes===String?"string":"Buffer";default:if(null===t)throw new Error("Found field with no usable type");const e=getTypeInterfaceName(t);return t instanceof Protobuf.Type?!i.defaults||n||r?`${o(e)}`:`${o(e)} | null`:o(e)}}function getFieldTypeRestricted(e,t){const n=getTypeNameRestricted(e.type,e.resolvedType,e.repeated,e.map,t);if(e instanceof Protobuf.MapField){return`{[key: ${"string"===e.keyType?"string":"number"}]: ${n}}`}return n}function generateRestrictedMessageInterface(e,t,n,r){var i,o,a;const{outputName:s}=useNameFmter(n);if(n.includeComments&&formatComment(e,t.comment,t.options),".google.protobuf.Any"===t.fullName&&n.json){let t=n.defaults?"":"?";return e.writeLine(`export type ${s("Any")} = AnyExtension | {`),e.writeLine(`  type_url${t}: string;`),e.writeLine(`  value${t}: ${getTypeNameRestricted("bytes",null,!1,!1,n)};`),void e.writeLine("}")}e.writeLine(`export interface ${s(null!=r?r:t.name)} {`),e.indent();for(const r of t.fieldsArray){let t;t=!r.partOf&&(r.repeated?null!==(i=n.defaults||n.arrays)&&void 0!==i&&i:r.map?null!==(o=n.defaults||n.objects)&&void 0!==o&&o:null!==(a=n.defaults)&&void 0!==a&&a);const s=t?"":"?",u=r.repeated?"[]":"",c=getFieldTypeRestricted(r,n);n.includeComments&&formatComment(e,r.comment,r.options),e.writeLine(`'${r.name}'${s}: (${c})${u};`)}if(n.oneofs)for(const r of t.oneofsArray){const t=r.fieldsArray.map((e=>`"${e.name}"`)).join("|");n.includeComments&&formatComment(e,r.comment,r.options),e.writeLine(`'${r.name}': ${t};`)}n.outputBranded&&formatTypeBrand(e,t),e.unindent(),e.writeLine("}")}function generateMessageInterfaces(e,t,n){var r,i;let o=!1,a=new Set;const s=getChildMessagesAndEnums(t);e.writeLine(`// Original file: ${null===(i=null!==(r=t.filename)&&void 0!==r?r:"null")||void 0===i?void 0:i.replace(/\\/g,"/")}`),e.writeLine("");const u=e=>["int64","uint64","sint64","fixed64","sfixed64"].includes(e.type);t.fieldsArray.sort(((e,t)=>e.id-t.id));for(const r of t.fieldsArray){if(r.resolvedType&&s.indexOf(r.resolvedType)<0){const i=r.resolvedType;if(a.has(i.fullName))continue;a.add(i.fullName),e.writeLine(getImportLine(i,t,n))}u(r)&&(o=!0)}for(const r of s)if(r instanceof Protobuf.Type)for(const i of r.fieldsArray){if(i.resolvedType&&s.indexOf(i.resolvedType)<0){const r=i.resolvedType;if(a.has(r.fullName))continue;a.add(r.fullName),e.writeLine(getImportLine(r,t,n))}u(i)&&(o=!0)}o&&e.writeLine("import type { Long } from '@grpc/proto-loader';"),".google.protobuf.Any"===t.fullName&&e.writeLine("import type { AnyExtension } from '@grpc/proto-loader';"),e.writeLine("");for(const t of s.sort(compareName)){const r=getTypeInterfaceName(t);t instanceof Protobuf.Type?(generatePermissiveMessageInterface(e,t,n,r),e.writeLine(""),generateRestrictedMessageInterface(e,t,n,r)):generateEnumInterface(e,t,n,r),e.writeLine("")}generatePermissiveMessageInterface(e,t,n),e.writeLine(""),generateRestrictedMessageInterface(e,t,n)}function generateEnumInterface(e,t,n,r){var i,o,a;const{inputName:s,outputName:u}=useNameFmter(n),c=null!=r?r:t.name;e.writeLine(`// Original file: ${null===(o=null!==(i=t.filename)&&void 0!==i?i:"null")||void 0===o?void 0:o.replace(/\\/g,"/")}`),e.writeLine(""),n.includeComments&&formatComment(e,t.comment,t.options),e.writeLine(`export const ${c} = {`),e.indent();for(const r of Object.keys(t.values))n.includeComments&&formatComment(e,t.comments[r],(null!==(a=t.valuesOptions)&&void 0!==a?a:{})[r]),e.writeLine(`${r}: ${n.enums==String?`'${r}'`:t.values[r]},`);e.unindent(),e.writeLine("} as const;"),e.writeLine(""),n.includeComments&&formatComment(e,t.comment,t.options),e.writeLine(`export type ${s(c)} =`),e.indent();for(const r of Object.keys(t.values))n.includeComments&&formatComment(e,t.comments[r]),e.writeLine(`| '${r}'`),e.writeLine(`| ${t.values[r]}`);e.unindent(),e.writeLine(""),n.includeComments&&formatComment(e,t.comment,t.options),e.writeLine(`export type ${u(c)} = typeof ${c}[keyof typeof ${c}]`)}const CLIENT_RESERVED_METHOD_NAMES=new Set(["close","getChannel","waitForReady","makeUnaryRequest","makeClientStreamRequest","makeServerStreamRequest","makeBidiStreamRequest","resolveCallInterceptors","checkOptionalUnaryResponseArguments","checkMetadataAndOptions"]);function generateServiceClientInterface(e,t,n){const{outputName:r,inputName:i}=useNameFmter(n);n.includeComments&&formatComment(e,t.comment,t.options),e.writeLine(`export interface ${t.name}Client extends grpc.Client {`),e.indent();for(const o of Object.keys(t.methods).sort()){const a=t.methods[o];for(const t of new Set([o,camelCase(o)])){if(CLIENT_RESERVED_METHOD_NAMES.has(t))continue;n.includeComments&&formatComment(e,a.comment,a.options);const o=i(getTypeInterfaceName(a.resolvedRequestType)),s=r(getTypeInterfaceName(a.resolvedResponseType)),u=`grpc.requestCallback<${s}>`;if(a.requestStream)if(a.responseStream){const n=`grpc.ClientDuplexStream<${o}, ${s}>`;e.writeLine(`${t}(metadata: grpc.Metadata, options?: grpc.CallOptions): ${n};`),e.writeLine(`${t}(options?: grpc.CallOptions): ${n};`)}else{const n=`grpc.ClientWritableStream<${o}>`;e.writeLine(`${t}(metadata: grpc.Metadata, options: grpc.CallOptions, callback: ${u}): ${n};`),e.writeLine(`${t}(metadata: grpc.Metadata, callback: ${u}): ${n};`),e.writeLine(`${t}(options: grpc.CallOptions, callback: ${u}): ${n};`),e.writeLine(`${t}(callback: ${u}): ${n};`)}else if(a.responseStream){const n=`grpc.ClientReadableStream<${s}>`;e.writeLine(`${t}(argument: ${o}, metadata: grpc.Metadata, options?: grpc.CallOptions): ${n};`),e.writeLine(`${t}(argument: ${o}, options?: grpc.CallOptions): ${n};`)}else{const n="grpc.ClientUnaryCall";e.writeLine(`${t}(argument: ${o}, metadata: grpc.Metadata, options: grpc.CallOptions, callback: ${u}): ${n};`),e.writeLine(`${t}(argument: ${o}, metadata: grpc.Metadata, callback: ${u}): ${n};`),e.writeLine(`${t}(argument: ${o}, options: grpc.CallOptions, callback: ${u}): ${n};`),e.writeLine(`${t}(argument: ${o}, callback: ${u}): ${n};`)}}e.writeLine("")}e.unindent(),e.writeLine("}")}function generateServiceHandlerInterface(e,t,n){const{inputName:r,outputName:i}=useNameFmter(n);n.includeComments&&formatComment(e,t.comment,t.options),e.writeLine(`export interface ${t.name}Handlers extends grpc.UntypedServiceImplementation {`),e.indent();for(const o of Object.keys(t.methods).sort()){const a=t.methods[o];n.includeComments&&formatComment(e,a.comment,t.options);const s=i(getTypeInterfaceName(a.resolvedRequestType)),u=r(getTypeInterfaceName(a.resolvedResponseType));a.requestStream?a.responseStream?e.writeLine(`${o}: grpc.handleBidiStreamingCall<${s}, ${u}>;`):e.writeLine(`${o}: grpc.handleClientStreamingCall<${s}, ${u}>;`):a.responseStream?e.writeLine(`${o}: grpc.handleServerStreamingCall<${s}, ${u}>;`):e.writeLine(`${o}: grpc.handleUnaryCall<${s}, ${u}>;`),e.writeLine("")}e.unindent(),e.writeLine("}")}function generateServiceDefinitionInterface(e,t,n){const{inputName:r,outputName:i}=useNameFmter(n);n.grpcLib?e.writeLine(`export interface ${t.name}Definition extends grpc.ServiceDefinition {`):e.writeLine(`export interface ${t.name}Definition {`),e.indent();for(const n of Object.keys(t.methods).sort()){const o=t.methods[n],a=getTypeInterfaceName(o.resolvedRequestType),s=getTypeInterfaceName(o.resolvedResponseType);e.writeLine(`${n}: MethodDefinition<${r(a)}, ${r(s)}, ${i(a)}, ${i(s)}>`)}e.unindent(),e.writeLine("}")}function generateServiceInterfaces(e,t,n){var r,i;if(e.writeLine(`// Original file: ${null===(i=null!==(r=t.filename)&&void 0!==r?r:"null")||void 0===i?void 0:i.replace(/\\/g,"/")}`),e.writeLine(""),n.grpcLib){const r=n.grpcLib.startsWith(".")?getPathToRoot(t)+n.grpcLib:n.grpcLib;e.writeLine(`import type * as grpc from '${r}'`)}e.writeLine("import type { MethodDefinition } from '@grpc/proto-loader'");const o=new Set;for(const e of t.methodsArray)o.add(e.resolvedRequestType),o.add(e.resolvedResponseType);for(const r of Array.from(o.values()).sort(compareName))e.writeLine(getImportLine(r,t,n));e.writeLine(""),n.grpcLib&&(generateServiceClientInterface(e,t,n),e.writeLine(""),generateServiceHandlerInterface(e,t,n),e.writeLine("")),generateServiceDefinitionInterface(e,t,n)}function containsDefinition(e,t){for(const n of t.nestedArray.sort(compareName)){if(n instanceof e)return!0;if(isNamespaceBase(n)&&!(n instanceof Protobuf.Type)&&!(n instanceof Protobuf.Enum)&&containsDefinition(e,n))return!0}return!1}function generateDefinitionImports(e,t,n){const r=[];containsDefinition(Protobuf.Enum,t)&&r.push("EnumTypeDefinition"),containsDefinition(Protobuf.Type,t)&&r.push("MessageTypeDefinition"),r.length&&e.writeLine(`import type { ${r.join(", ")} } from '@grpc/proto-loader';`)}function generateServiceImports(e,t,n){for(const r of t.nestedArray.sort(compareName))r instanceof Protobuf.Service?e.writeLine(getImportLine(r,void 0,n)):!isNamespaceBase(r)||r instanceof Protobuf.Type||r instanceof Protobuf.Enum||generateServiceImports(e,r,n)}function generateSingleLoadedDefinitionType(e,t,n){if(t instanceof Protobuf.Service){n.includeComments&&formatComment(e,t.comment,t.options);const r=getTypeInterfaceName(t);e.writeLine(`${t.name}: SubtypeConstructor<typeof grpc.Client, ${r}Client> & { service: ${r}Definition }`)}else t instanceof Protobuf.Enum?e.writeLine(`${t.name}: EnumTypeDefinition`):t instanceof Protobuf.Type?e.writeLine(`${t.name}: MessageTypeDefinition`):isNamespaceBase(t)&&generateLoadedDefinitionTypes(e,t,n)}function generateLoadedDefinitionTypes(e,t,n){e.writeLine(`${t.name}: {`),e.indent();for(const r of t.nestedArray.sort(compareName))generateSingleLoadedDefinitionType(e,r,n);e.unindent(),e.writeLine("}")}function generateRootFile(e,t,n){if(n.grpcLib){e.writeLine(`import type * as grpc from '${n.grpcLib}';`),generateDefinitionImports(e,t,n),e.writeLine(""),generateServiceImports(e,t,n),e.writeLine(""),e.writeLine("type SubtypeConstructor<Constructor extends new (...args: any) => any, Subtype> = {"),e.writeLine("  new(...args: ConstructorParameters<Constructor>): Subtype;"),e.writeLine("};"),e.writeLine(""),e.writeLine("export interface ProtoGrpcType {"),e.indent();for(const r of t.nestedArray)generateSingleLoadedDefinitionType(e,r,n);e.unindent(),e.writeLine("}"),e.writeLine("")}}async function writeFile(e,t){return await fs.promises.mkdir(path.dirname(e),{recursive:!0}),fs.promises.writeFile(e,t)}function generateFilesForNamespace(e,t){const n=[];for(const r of e.nestedArray){const e=new TextFormatter;r instanceof Protobuf.Type?(generateMessageInterfaces(e,r,t),t.verbose&&console.log(`Writing ${t.outDir}/${getPath(r)} from file ${r.filename}`),n.push(writeFile(`${t.outDir}/${getPath(r)}`,e.getFullText()))):r instanceof Protobuf.Enum?(generateEnumInterface(e,r,t),t.verbose&&console.log(`Writing ${t.outDir}/${getPath(r)} from file ${r.filename}`),n.push(writeFile(`${t.outDir}/${getPath(r)}`,e.getFullText()))):r instanceof Protobuf.Service?(generateServiceInterfaces(e,r,t),t.verbose&&console.log(`Writing ${t.outDir}/${getPath(r)} from file ${r.filename}`),n.push(writeFile(`${t.outDir}/${getPath(r)}`,e.getFullText()))):isNamespaceBase(r)&&n.push(...generateFilesForNamespace(r,t))}return n}function writeFilesForRoot(e,t,n){const r=[],i=new TextFormatter;return n.grpcLib&&(generateRootFile(i,e,n),n.verbose&&console.log(`Writing ${n.outDir}/${t}`),r.push(writeFile(`${n.outDir}/${t}`,i.getFullText()))),r.push(...generateFilesForNamespace(e,n)),r}async function writeAllFiles(e,t){await fs.promises.mkdir(t.outDir,{recursive:!0});const n=new Map;for(const t of e){const e=path.basename(t).replace(/\.proto$/,".ts");n.has(e)?n.get(e).push(t):n.set(e,[t])}for(const[e,r]of n.entries()){writeFilesForRoot(await(0,util_1.loadProtosWithOptions)(r,t),e,t)}}async function runScript(){const e={boolean:!0,default:!1},t=await yargs.parserConfiguration({"parse-positional-numbers":!1}).option("keepCase",e).option("longs",{string:!0,default:"Long"}).option("enums",{string:!0,default:"number"}).option("bytes",{string:!0,default:"Buffer"}).option("defaults",e).option("arrays",e).option("objects",e).option("oneofs",e).option("json",e).boolean("verbose").option("includeComments",e).option("includeDirs",{normalize:!0,array:!0,alias:"I"}).option("outDir",{alias:"O",normalize:!0}).option("grpcLib",{string:!0}).option("inputTemplate",{string:!0,default:"%s"}).option("outputTemplate",{string:!0,default:"%s__Output"}).option("inputBranded",e).option("outputBranded",e).coerce("longs",(e=>{switch(e){case"String":return String;case"Number":return Number;default:return}})).coerce("enums",(e=>"String"===e?String:void 0)).coerce("bytes",(e=>{switch(e){case"Array":return Array;case"String":return String;default:return}})).alias({verbose:"v"}).describe({keepCase:"Preserve the case of field names",longs:"The type that should be used to output 64 bit integer values. Can be String, Number",enums:"The type that should be used to output enum fields. Can be String",bytes:"The type that should be used to output bytes fields. Can be String, Array",defaults:"Output default values for omitted fields",arrays:"Output default values for omitted repeated fields even if --defaults is not set",objects:"Output default values for omitted message fields even if --defaults is not set",oneofs:"Output virtual oneof fields set to the present field's name",json:"Represent Infinity and NaN as strings in float fields. Also decode google.protobuf.Any automatically",includeComments:"Generate doc comments from comments in the original files",includeDirs:"Directories to search for included files",outDir:"Directory in which to output files",grpcLib:"The gRPC implementation library that these types will be used with. If not provided, some types will not be generated",inputTemplate:'Template for mapping input or "permissive" type names',outputTemplate:'Template for mapping output or "restricted" type names',inputBranded:'Output property for branded type for  "permissive" types with fullName of the Message as its value',outputBranded:'Output property for branded type for  "restricted" types with fullName of the Message as its value'}).demandOption(["outDir"]).demand(1).usage("$0 [options] filenames...").epilogue("WARNING: This tool is in alpha. The CLI and generated code are subject to change").argv;t.verbose&&console.log("Parsed arguments:",t),(0,util_1.addCommonProtos)(),writeAllFiles(t._,Object.assign(Object.assign({},t),{alternateCommentMode:!0})).then((()=>{t.verbose&&console.log("Success")}),(e=>{console.error(e),process.exit(1)}))}require.main===module&&runScript();