/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{ArithmeticException,DateTimeParseException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil,MAX_SAFE_INTEGER,MIN_SAFE_INTEGER}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalAmount}from"./temporal/TemporalAmount";import{TemporalUnit}from"./temporal/TemporalUnit";import{LocalTime}from"./LocalTime";export class Duration extends TemporalAmount{constructor(t,n){super(),this._seconds=MathUtil.safeToInt(t),this._nanos=MathUtil.safeToInt(n)}static ofDays(t){return Duration._create(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_DAY),0)}static ofHours(t){return Duration._create(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_HOUR),0)}static ofMinutes(t){return Duration._create(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_MINUTE),0)}static ofSeconds(t,n=0){const i=MathUtil.safeAdd(t,MathUtil.floorDiv(n,LocalTime.NANOS_PER_SECOND)),e=MathUtil.floorMod(n,LocalTime.NANOS_PER_SECOND);return Duration._create(i,e)}static ofMillis(t){let n=MathUtil.intDiv(t,1e3),i=MathUtil.intMod(t,1e3);return i<0&&(i+=1e3,n--),Duration._create(n,1e6*i)}static ofNanos(t){let n=MathUtil.intDiv(t,LocalTime.NANOS_PER_SECOND),i=MathUtil.intMod(t,LocalTime.NANOS_PER_SECOND);return i<0&&(i+=LocalTime.NANOS_PER_SECOND,n--),this._create(n,i)}static of(t,n){return Duration.ZERO.plus(t,n)}static from(t){requireNonNull(t,"amount"),requireInstance(t,TemporalAmount);let n=Duration.ZERO;return t.units().forEach((i=>{n=n.plus(t.get(i),i)})),n}static between(t,n){requireNonNull(t,"startInclusive"),requireNonNull(n,"endExclusive");let i=t.until(n,ChronoUnit.SECONDS),e=0;if(t.isSupported(ChronoField.NANO_OF_SECOND)&&n.isSupported(ChronoField.NANO_OF_SECOND))try{const s=t.getLong(ChronoField.NANO_OF_SECOND);if(e=n.getLong(ChronoField.NANO_OF_SECOND)-s,i>0&&e<0)e+=LocalTime.NANOS_PER_SECOND;else if(i<0&&e>0)e-=LocalTime.NANOS_PER_SECOND;else if(0===i&&0!==e){const e=n.with(ChronoField.NANO_OF_SECOND,s);i=t.until(e,ChronoUnit.SECONDS)}}catch(t){}return this.ofSeconds(i,e)}static parse(t){requireNonNull(t,"text");const n=new RegExp("([-+]?)P(?:([-+]?[0-9]+)D)?(T(?:([-+]?[0-9]+)H)?(?:([-+]?[0-9]+)M)?(?:([-+]?[0-9]+)(?:[.,]([0-9]{0,9}))?S)?)?","i").exec(t);if(null!==n&&"T"===n[3]==!1){const i="-"===n[1],e=n[2],s=n[4],o=n[5],r=n[6],a=n[7];if(null!=e||null!=s||null!=o||null!=r){const n=Duration._parseNumber(t,e,LocalTime.SECONDS_PER_DAY,"days"),u=Duration._parseNumber(t,s,LocalTime.SECONDS_PER_HOUR,"hours"),l=Duration._parseNumber(t,o,LocalTime.SECONDS_PER_MINUTE,"minutes"),h=Duration._parseNumber(t,r,1,"seconds"),N=null!=r&&"-"===r.charAt(0),c=Duration._parseFraction(t,a,N?-1:1);try{return Duration._create(i,n,u,l,h,c)}catch(n){throw new DateTimeParseException("Text cannot be parsed to a Duration: overflow",t,0,n)}}}throw new DateTimeParseException("Text cannot be parsed to a Duration",t,0)}static _parseNumber(t,n,i,e){if(null==n)return 0;try{return"+"===n[0]&&(n=n.substring(1)),MathUtil.safeMultiply(parseFloat(n),i)}catch(n){throw new DateTimeParseException(`Text cannot be parsed to a Duration: ${e}`,t,0,n)}}static _parseFraction(t,n,i){return null==n||0===n.length?0:(n=`${n}000000000`.substring(0,9),parseFloat(n)*i)}static _create(){return arguments.length<=2?Duration._createSecondsNanos(arguments[0],arguments[1]):Duration._createNegateDaysHoursMinutesSecondsNanos(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5])}static _createNegateDaysHoursMinutesSecondsNanos(t,n,i,e,s,o){const r=MathUtil.safeAdd(n,MathUtil.safeAdd(i,MathUtil.safeAdd(e,s)));return t?Duration.ofSeconds(r,o).negated():Duration.ofSeconds(r,o)}static _createSecondsNanos(t=0,n=0){return 0===t&&0===n?Duration.ZERO:new Duration(t,n)}get(t){if(t===ChronoUnit.SECONDS)return this._seconds;if(t===ChronoUnit.NANOS)return this._nanos;throw new UnsupportedTemporalTypeException(`Unsupported unit: ${t}`)}units(){return[ChronoUnit.SECONDS,ChronoUnit.NANOS]}isZero(){return 0===this._seconds&&0===this._nanos}isNegative(){return this._seconds<0}seconds(){return this._seconds}nano(){return this._nanos}withSeconds(t){return Duration._create(t,this._nanos)}withNanos(t){return ChronoField.NANO_OF_SECOND.checkValidIntValue(t),Duration._create(this._seconds,t)}plusDuration(t){return requireNonNull(t,"duration"),this.plus(t.seconds(),t.nano())}plus(t,n){return 1===arguments.length?this.plusDuration(t):2===arguments.length&&n instanceof TemporalUnit?this.plusAmountUnit(t,n):this.plusSecondsNanos(t,n)}plusAmountUnit(t,n){if(requireNonNull(t,"amountToAdd"),requireNonNull(n,"unit"),n===ChronoUnit.DAYS)return this.plusSecondsNanos(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_DAY),0);if(n.isDurationEstimated())throw new UnsupportedTemporalTypeException("Unit must not have an estimated duration");if(0===t)return this;if(n instanceof ChronoUnit){switch(n){case ChronoUnit.NANOS:return this.plusNanos(t);case ChronoUnit.MICROS:return this.plusSecondsNanos(1e3*MathUtil.intDiv(t,1e9),1e3*MathUtil.intMod(t,1e9));case ChronoUnit.MILLIS:return this.plusMillis(t);case ChronoUnit.SECONDS:return this.plusSeconds(t)}return this.plusSecondsNanos(MathUtil.safeMultiply(n.duration().seconds(),t),0)}const i=n.duration().multipliedBy(t);return this.plusSecondsNanos(i.seconds(),i.nano())}plusDays(t){return this.plusSecondsNanos(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_DAY),0)}plusHours(t){return this.plusSecondsNanos(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_HOUR),0)}plusMinutes(t){return this.plusSecondsNanos(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_MINUTE),0)}plusSeconds(t){return this.plusSecondsNanos(t,0)}plusMillis(t){return this.plusSecondsNanos(MathUtil.intDiv(t,1e3),1e6*MathUtil.intMod(t,1e3))}plusNanos(t){return this.plusSecondsNanos(0,t)}plusSecondsNanos(t,n){if(requireNonNull(t,"secondsToAdd"),requireNonNull(n,"nanosToAdd"),0===t&&0===n)return this;let i=MathUtil.safeAdd(this._seconds,t);i=MathUtil.safeAdd(i,MathUtil.intDiv(n,LocalTime.NANOS_PER_SECOND)),n=MathUtil.intMod(n,LocalTime.NANOS_PER_SECOND);const e=MathUtil.safeAdd(this._nanos,n);return Duration.ofSeconds(i,e)}minus(t,n){return 1===arguments.length?this.minusDuration(t):this.minusAmountUnit(t,n)}minusDuration(t){requireNonNull(t,"duration");const n=t.seconds(),i=t.nano();return n===MIN_SAFE_INTEGER?this.plus(MAX_SAFE_INTEGER,-i):this.plus(-n,-i)}minusAmountUnit(t,n){return requireNonNull(t,"amountToSubtract"),requireNonNull(n,"unit"),t===MIN_SAFE_INTEGER?this.plusAmountUnit(MAX_SAFE_INTEGER,n):this.plusAmountUnit(-t,n)}minusDays(t){return t===MIN_SAFE_INTEGER?this.plusDays(MAX_SAFE_INTEGER):this.plusDays(-t)}minusHours(t){return t===MIN_SAFE_INTEGER?this.plusHours(MAX_SAFE_INTEGER):this.plusHours(-t)}minusMinutes(t){return t===MIN_SAFE_INTEGER?this.plusMinutes(MAX_SAFE_INTEGER):this.plusMinutes(-t)}minusSeconds(t){return t===MIN_SAFE_INTEGER?this.plusSeconds(MAX_SAFE_INTEGER):this.plusSeconds(-t)}minusMillis(t){return t===MIN_SAFE_INTEGER?this.plusMillis(MAX_SAFE_INTEGER):this.plusMillis(-t)}minusNanos(t){return t===MIN_SAFE_INTEGER?this.plusNanos(MAX_SAFE_INTEGER):this.plusNanos(-t)}multipliedBy(t){if(0===t)return Duration.ZERO;if(1===t)return this;let n=MathUtil.safeMultiply(this._seconds,t),i=MathUtil.safeMultiply(this._nanos,t);return n+=MathUtil.intDiv(i,LocalTime.NANOS_PER_SECOND),i=MathUtil.intMod(i,LocalTime.NANOS_PER_SECOND),Duration.ofSeconds(n,i)}dividedBy(t){if(0===t)throw new ArithmeticException("Cannot divide by zero");if(1===t)return this;const n=MathUtil.intDiv(this._seconds,t),i=MathUtil.roundDown((this._seconds/t-n)*LocalTime.NANOS_PER_SECOND);let e=MathUtil.intDiv(this._nanos,t);return e=i+e,Duration.ofSeconds(n,e)}negated(){return this.multipliedBy(-1)}abs(){return this.isNegative()?this.negated():this}addTo(t){return requireNonNull(t,"temporal"),0!==this._seconds&&(t=t.plus(this._seconds,ChronoUnit.SECONDS)),0!==this._nanos&&(t=t.plus(this._nanos,ChronoUnit.NANOS)),t}subtractFrom(t){return requireNonNull(t,"temporal"),0!==this._seconds&&(t=t.minus(this._seconds,ChronoUnit.SECONDS)),0!==this._nanos&&(t=t.minus(this._nanos,ChronoUnit.NANOS)),t}toDays(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_DAY)}toHours(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_HOUR)}toMinutes(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_MINUTE)}toMillis(){let t=Math.round(MathUtil.safeMultiply(this._seconds,1e3));return t=MathUtil.safeAdd(t,MathUtil.intDiv(this._nanos,1e6)),t}toNanos(){let t=MathUtil.safeMultiply(this._seconds,LocalTime.NANOS_PER_SECOND);return t=MathUtil.safeAdd(t,this._nanos),t}compareTo(t){requireNonNull(t,"otherDuration"),requireInstance(t,Duration,"otherDuration");const n=MathUtil.compareNumbers(this._seconds,t.seconds());return 0!==n?n:this._nanos-t.nano()}equals(t){return this===t||t instanceof Duration&&(this.seconds()===t.seconds()&&this.nano()===t.nano())}toString(){if(this===Duration.ZERO)return"PT0S";const t=MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_HOUR),n=MathUtil.intDiv(MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_HOUR),LocalTime.SECONDS_PER_MINUTE),i=MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_MINUTE);let e="PT";if(0!==t&&(e+=`${t}H`),0!==n&&(e+=`${n}M`),0===i&&0===this._nanos&&e.length>2)return e;if(i<0&&this._nanos>0?e+=-1===i?"-0":i+1:e+=i,this._nanos>0){let t;for(e+=".",t=i<0?""+(2*LocalTime.NANOS_PER_SECOND-this._nanos):`${LocalTime.NANOS_PER_SECOND+this._nanos}`,t=t.slice(1,t.length),e+=t;"0"===e.charAt(e.length-1);)e=e.slice(0,e.length-1)}return e+="S",e}toJSON(){return this.toString()}}export function _init(){Duration.ZERO=new Duration(0,0)}