/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{LocalTime}from"./LocalTime";import{ZonedDateTime}from"./ZonedDateTime";import{MathUtil}from"./MathUtil";import{OffsetDateTime}from"./OffsetDateTime";import{Temporal}from"./temporal/Temporal";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalUnit}from"./temporal/TemporalUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{DateTimeFormatter}from"./format/DateTimeFormatter";const NANOS_PER_MILLI=1e6;export class Instant extends Temporal{static now(t=Clock.systemUTC()){return t.instant()}static ofEpochSecond(t,n=0){const e=t+MathUtil.floorDiv(n,LocalTime.NANOS_PER_SECOND),o=MathUtil.floorMod(n,LocalTime.NANOS_PER_SECOND);return Instant._create(e,o)}static ofEpochMilli(t){const n=MathUtil.floorDiv(t,1e3),e=MathUtil.floorMod(t,1e3);return Instant._create(n,1e6*e)}static ofEpochMicro(t){const n=MathUtil.floorDiv(t,1e6),e=MathUtil.floorMod(t,1e6);return Instant._create(n,1e3*e)}static from(t){try{const n=t.getLong(ChronoField.INSTANT_SECONDS),e=t.get(ChronoField.NANO_OF_SECOND);return Instant.ofEpochSecond(n,e)}catch(n){throw new DateTimeException(`Unable to obtain Instant from TemporalAccessor: ${t}, type ${typeof t}`,n)}}static parse(t){return DateTimeFormatter.ISO_INSTANT.parse(t,Instant.FROM)}static _create(t,n){return 0===t&&0===n?Instant.EPOCH:new Instant(t,n)}static _validate(t,n){if(t<Instant.MIN_SECONDS||t>Instant.MAX_SECONDS)throw new DateTimeException("Instant exceeds minimum or maximum instant");if(n<0||n>LocalTime.NANOS_PER_SECOND)throw new DateTimeException("Instant exceeds minimum or maximum instant")}constructor(t,n){super(),Instant._validate(t,n),this._seconds=MathUtil.safeToInt(t),this._nanos=MathUtil.safeToInt(n)}isSupported(t){return t instanceof ChronoField?t===ChronoField.INSTANT_SECONDS||t===ChronoField.NANO_OF_SECOND||t===ChronoField.MICRO_OF_SECOND||t===ChronoField.MILLI_OF_SECOND:t instanceof ChronoUnit?t.isTimeBased()||t===ChronoUnit.DAYS:null!=t&&t.isSupportedBy(this)}range(t){return super.range(t)}get(t){return this.getLong(t)}getLong(t){if(t instanceof ChronoField){switch(t){case ChronoField.NANO_OF_SECOND:return this._nanos;case ChronoField.MICRO_OF_SECOND:return MathUtil.intDiv(this._nanos,1e3);case ChronoField.MILLI_OF_SECOND:return MathUtil.intDiv(this._nanos,1e6);case ChronoField.INSTANT_SECONDS:return this._seconds}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.getFrom(this)}epochSecond(){return this._seconds}nano(){return this._nanos}_withField(t,n){if(requireNonNull(t,"field"),t instanceof ChronoField){switch(t.checkValidValue(n),t){case ChronoField.MILLI_OF_SECOND:{const t=1e6*n;return t!==this._nanos?Instant._create(this._seconds,t):this}case ChronoField.MICRO_OF_SECOND:{const t=1e3*n;return t!==this._nanos?Instant._create(this._seconds,t):this}case ChronoField.NANO_OF_SECOND:return n!==this._nanos?Instant._create(this._seconds,n):this;case ChronoField.INSTANT_SECONDS:return n!==this._seconds?Instant._create(n,this._nanos):this}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.adjustInto(this,n)}truncatedTo(t){if(requireNonNull(t,"unit"),t===ChronoUnit.NANOS)return this;const n=t.duration();if(n.seconds()>LocalTime.SECONDS_PER_DAY)throw new DateTimeException("Unit is too large to be used for truncation");const e=n.toNanos();if(0!==MathUtil.intMod(LocalTime.NANOS_PER_DAY,e))throw new DateTimeException("Unit must divide into a standard day without remainder");const o=MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_DAY)*LocalTime.NANOS_PER_SECOND+this._nanos,i=MathUtil.intDiv(o,e)*e;return this.plusNanos(i-o)}_plusUnit(t,n){if(requireNonNull(t,"amountToAdd"),requireNonNull(n,"unit"),requireInstance(n,TemporalUnit),n instanceof ChronoUnit){switch(n){case ChronoUnit.NANOS:return this.plusNanos(t);case ChronoUnit.MICROS:return this.plusMicros(t);case ChronoUnit.MILLIS:return this.plusMillis(t);case ChronoUnit.SECONDS:return this.plusSeconds(t);case ChronoUnit.MINUTES:return this.plusSeconds(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_MINUTE));case ChronoUnit.HOURS:return this.plusSeconds(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_HOUR));case ChronoUnit.HALF_DAYS:return this.plusSeconds(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_DAY/2));case ChronoUnit.DAYS:return this.plusSeconds(MathUtil.safeMultiply(t,LocalTime.SECONDS_PER_DAY))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${n}`)}return n.addTo(this,t)}plusSeconds(t){return this._plus(t,0)}plusMillis(t){return this._plus(MathUtil.intDiv(t,1e3),1e6*MathUtil.intMod(t,1e3))}plusNanos(t){return this._plus(0,t)}plusMicros(t){return this._plus(MathUtil.intDiv(t,1e6),1e3*MathUtil.intMod(t,1e6))}_plus(t,n){if(0===t&&0===n)return this;let e=this._seconds+t;e+=MathUtil.intDiv(n,LocalTime.NANOS_PER_SECOND);const o=this._nanos+n%LocalTime.NANOS_PER_SECOND;return Instant.ofEpochSecond(e,o)}_minusUnit(t,n){return this._plusUnit(-1*t,n)}minusSeconds(t){return this.plusSeconds(-1*t)}minusMillis(t){return this.plusMillis(-1*t)}minusNanos(t){return this.plusNanos(-1*t)}minusMicros(t){return this.plusMicros(-1*t)}query(t){return requireNonNull(t,"query"),t===TemporalQueries.precision()?ChronoUnit.NANOS:t===TemporalQueries.localDate()||t===TemporalQueries.localTime()||t===TemporalQueries.chronology()||t===TemporalQueries.zoneId()||t===TemporalQueries.zone()||t===TemporalQueries.offset()?null:t.queryFrom(this)}adjustInto(t){return requireNonNull(t,"temporal"),t.with(ChronoField.INSTANT_SECONDS,this._seconds).with(ChronoField.NANO_OF_SECOND,this._nanos)}until(t,n){requireNonNull(t,"endExclusive"),requireNonNull(n,"unit");const e=Instant.from(t);if(n instanceof ChronoUnit){switch(n){case ChronoUnit.NANOS:return this._nanosUntil(e);case ChronoUnit.MICROS:return this._microsUntil(e);case ChronoUnit.MILLIS:return MathUtil.safeSubtract(e.toEpochMilli(),this.toEpochMilli());case ChronoUnit.SECONDS:return this._secondsUntil(e);case ChronoUnit.MINUTES:return MathUtil.intDiv(this._secondsUntil(e),LocalTime.SECONDS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(this._secondsUntil(e),LocalTime.SECONDS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(this._secondsUntil(e),12*LocalTime.SECONDS_PER_HOUR);case ChronoUnit.DAYS:return MathUtil.intDiv(this._secondsUntil(e),LocalTime.SECONDS_PER_DAY)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${n}`)}return n.between(this,e)}_microsUntil(t){const n=MathUtil.safeSubtract(t.epochSecond(),this.epochSecond()),e=MathUtil.safeMultiply(n,1e6);return MathUtil.safeAdd(e,MathUtil.intDiv(t.nano()-this.nano(),1e3))}_nanosUntil(t){const n=MathUtil.safeSubtract(t.epochSecond(),this.epochSecond()),e=MathUtil.safeMultiply(n,LocalTime.NANOS_PER_SECOND);return MathUtil.safeAdd(e,t.nano()-this.nano())}_secondsUntil(t){let n=MathUtil.safeSubtract(t.epochSecond(),this.epochSecond());const e=t.nano()-this.nano();return n>0&&e<0?n--:n<0&&e>0&&n++,n}atOffset(t){return OffsetDateTime.ofInstant(this,t)}atZone(t){return ZonedDateTime.ofInstant(this,t)}toEpochMilli(){return MathUtil.safeMultiply(this._seconds,1e3)+MathUtil.intDiv(this._nanos,1e6)}compareTo(t){requireNonNull(t,"otherInstant"),requireInstance(t,Instant,"otherInstant");const n=MathUtil.compareNumbers(this._seconds,t._seconds);return 0!==n?n:this._nanos-t._nanos}isAfter(t){return this.compareTo(t)>0}isBefore(t){return this.compareTo(t)<0}equals(t){return this===t||t instanceof Instant&&(this.epochSecond()===t.epochSecond()&&this.nano()===t.nano())}hashCode(){return MathUtil.hashCode(this._seconds,this._nanos)}toString(){return DateTimeFormatter.ISO_INSTANT.format(this)}toJSON(){return this.toString()}}export function _init(){Instant.MIN_SECONDS=-31619119219200,Instant.MAX_SECONDS=31494816403199,Instant.EPOCH=new Instant(0,0),Instant.MIN=Instant.ofEpochSecond(Instant.MIN_SECONDS,0),Instant.MAX=Instant.ofEpochSecond(Instant.MAX_SECONDS,999999999),Instant.FROM=createTemporalQuery("Instant.FROM",(t=>Instant.from(t)))}