/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull,requireInstance}from"./assert";import{MathUtil}from"./MathUtil";import{DateTimeException,UnsupportedTemporalTypeException,NullPointerException,IllegalArgumentException}from"./errors";import{IsoChronology}from"./chrono/IsoChronology";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{ChronoLocalDate}from"./chrono/ChronoLocalDate";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ValueRange}from"./temporal/ValueRange";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Clock}from"./Clock";import{DayOfWeek}from"./DayOfWeek";import{OffsetDateTime}from"./OffsetDateTime";import{OffsetTime}from"./OffsetTime";import{Month}from"./Month";import{Period}from"./Period";import{YearConstants}from"./YearConstants";import{LocalTime}from"./LocalTime";import{LocalDateTime}from"./LocalDateTime";import{Year}from"./Year";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{ZonedDateTime}from"./ZonedDateTime";const DAYS_PER_CYCLE=146097,DAYS_0000_TO_1970=719528;export class LocalDate extends ChronoLocalDate{static now(t){let e;return e=null==t?Clock.systemDefaultZone():t instanceof ZoneId?Clock.system(t):t,LocalDate.ofInstant(e.instant(),e.zone())}static ofInstant(t,e=ZoneId.systemDefault()){requireNonNull(t,"instant");const o=e.rules().offset(t),r=t.epochSecond()+o.totalSeconds(),i=MathUtil.floorDiv(r,LocalTime.SECONDS_PER_DAY);return LocalDate.ofEpochDay(i)}static of(t,e,o){return new LocalDate(t,e,o)}static ofYearDay(t,e){ChronoField.YEAR.checkValidValue(t);const o=IsoChronology.isLeapYear(t);366===e&&!1===o&&assert(!1,`Invalid date 'DayOfYear 366' as '${t}' is not a leap year`,DateTimeException);let r=Month.of(Math.floor((e-1)/31+1));e>r.firstDayOfYear(o)+r.length(o)-1&&(r=r.plus(1));const i=e-r.firstDayOfYear(o)+1;return new LocalDate(t,r.value(),i)}static ofEpochDay(t=0){let e,o,r,i,a;a=t+719528,a-=60,e=0,a<0&&(o=MathUtil.intDiv(a+1,146097)-1,e=400*o,a+=146097*-o),i=MathUtil.intDiv(400*a+591,146097),r=a-(365*i+MathUtil.intDiv(i,4)-MathUtil.intDiv(i,100)+MathUtil.intDiv(i,400)),r<0&&(i--,r=a-(365*i+MathUtil.intDiv(i,4)-MathUtil.intDiv(i,100)+MathUtil.intDiv(i,400))),i+=e;const n=r,s=MathUtil.intDiv(5*n+2,153),h=(s+2)%12+1,l=n-MathUtil.intDiv(306*s+5,10)+1;i+=MathUtil.intDiv(s,10);return new LocalDate(i,h,l)}static from(t){requireNonNull(t,"temporal");const e=t.query(TemporalQueries.localDate());if(null==e)throw new DateTimeException(`Unable to obtain LocalDate from TemporalAccessor: ${t}, type ${null!=t.constructor?t.constructor.name:""}`);return e}static parse(t,e=DateTimeFormatter.ISO_LOCAL_DATE){return assert(null!=e,"formatter",NullPointerException),e.parse(t,LocalDate.FROM)}static _resolvePreviousValid(t,e,o){switch(e){case 2:o=Math.min(o,IsoChronology.isLeapYear(t)?29:28);break;case 4:case 6:case 9:case 11:o=Math.min(o,30)}return LocalDate.of(t,e,o)}constructor(t,e,o){super(),requireNonNull(t,"year"),requireNonNull(e,"month"),requireNonNull(o,"dayOfMonth"),e instanceof Month&&(e=e.value()),this._year=MathUtil.safeToInt(t),this._month=MathUtil.safeToInt(e),this._day=MathUtil.safeToInt(o),LocalDate._validate(this._year,this._month,this._day)}static _validate(t,e,o){let r;if(ChronoField.YEAR.checkValidValue(t),ChronoField.MONTH_OF_YEAR.checkValidValue(e),ChronoField.DAY_OF_MONTH.checkValidValue(o),o>28){switch(r=31,e){case 2:r=IsoChronology.isLeapYear(t)?29:28;break;case 4:case 6:case 9:case 11:r=30}o>r&&assert(!1,29===o?`Invalid date 'February 29' as '${t}' is not a leap year`:`Invalid date '${t}' '${e}' '${o}'`,DateTimeException)}}isSupported(t){return super.isSupported(t)}range(t){if(t instanceof ChronoField){if(t.isDateBased()){switch(t){case ChronoField.DAY_OF_MONTH:return ValueRange.of(1,this.lengthOfMonth());case ChronoField.DAY_OF_YEAR:return ValueRange.of(1,this.lengthOfYear());case ChronoField.ALIGNED_WEEK_OF_MONTH:return ValueRange.of(1,this.month()===Month.FEBRUARY&&!1===this.isLeapYear()?4:5);case ChronoField.YEAR_OF_ERA:return this._year<=0?ValueRange.of(1,Year.MAX_VALUE+1):ValueRange.of(1,Year.MAX_VALUE)}return t.range()}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.rangeRefinedBy(this)}get(t){return this.getLong(t)}getLong(t){return assert(null!=t,"",NullPointerException),t instanceof ChronoField?this._get0(t):t.getFrom(this)}_get0(t){switch(t){case ChronoField.DAY_OF_WEEK:return this.dayOfWeek().value();case ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH:return MathUtil.intMod(this._day-1,7)+1;case ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR:return MathUtil.intMod(this.dayOfYear()-1,7)+1;case ChronoField.DAY_OF_MONTH:return this._day;case ChronoField.DAY_OF_YEAR:return this.dayOfYear();case ChronoField.EPOCH_DAY:return this.toEpochDay();case ChronoField.ALIGNED_WEEK_OF_MONTH:return MathUtil.intDiv(this._day-1,7)+1;case ChronoField.ALIGNED_WEEK_OF_YEAR:return MathUtil.intDiv(this.dayOfYear()-1,7)+1;case ChronoField.MONTH_OF_YEAR:return this._month;case ChronoField.PROLEPTIC_MONTH:return this._prolepticMonth();case ChronoField.YEAR_OF_ERA:return this._year>=1?this._year:1-this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year>=1?1:0}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}_prolepticMonth(){return 12*this._year+(this._month-1)}chronology(){return IsoChronology.INSTANCE}year(){return this._year}monthValue(){return this._month}month(){return Month.of(this._month)}dayOfMonth(){return this._day}dayOfYear(){return this.month().firstDayOfYear(this.isLeapYear())+this._day-1}dayOfWeek(){const t=MathUtil.floorMod(this.toEpochDay()+3,7);return DayOfWeek.of(t+1)}isLeapYear(){return IsoChronology.isLeapYear(this._year)}lengthOfMonth(){switch(this._month){case 2:return this.isLeapYear()?29:28;case 4:case 6:case 9:case 11:return 30;default:return 31}}lengthOfYear(){return this.isLeapYear()?366:365}_withAdjuster(t){return requireNonNull(t,"adjuster"),t instanceof LocalDate?t:super._withAdjuster(t)}_withField(t,e){if(assert(null!=t,"field",NullPointerException),t instanceof ChronoField){const o=t;switch(o.checkValidValue(e),o){case ChronoField.DAY_OF_WEEK:return this.plusDays(e-this.dayOfWeek().value());case ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH:return this.plusDays(e-this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH));case ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR:return this.plusDays(e-this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR));case ChronoField.DAY_OF_MONTH:return this.withDayOfMonth(e);case ChronoField.DAY_OF_YEAR:return this.withDayOfYear(e);case ChronoField.EPOCH_DAY:return LocalDate.ofEpochDay(e);case ChronoField.ALIGNED_WEEK_OF_MONTH:return this.plusWeeks(e-this.getLong(ChronoField.ALIGNED_WEEK_OF_MONTH));case ChronoField.ALIGNED_WEEK_OF_YEAR:return this.plusWeeks(e-this.getLong(ChronoField.ALIGNED_WEEK_OF_YEAR));case ChronoField.MONTH_OF_YEAR:return this.withMonth(e);case ChronoField.PROLEPTIC_MONTH:return this.plusMonths(e-this.getLong(ChronoField.PROLEPTIC_MONTH));case ChronoField.YEAR_OF_ERA:return this.withYear(this._year>=1?e:1-e);case ChronoField.YEAR:return this.withYear(e);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===e?this:this.withYear(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.adjustInto(this,e)}withYear(t){return this._year===t?this:(ChronoField.YEAR.checkValidValue(t),LocalDate._resolvePreviousValid(t,this._month,this._day))}withMonth(t){const e=t instanceof Month?t.value():t;return this._month===e?this:(ChronoField.MONTH_OF_YEAR.checkValidValue(e),LocalDate._resolvePreviousValid(this._year,e,this._day))}withDayOfMonth(t){return this._day===t?this:LocalDate.of(this._year,this._month,t)}withDayOfYear(t){return this.dayOfYear()===t?this:LocalDate.ofYearDay(this._year,t)}_plusUnit(t,e){if(requireNonNull(t,"amountToAdd"),requireNonNull(e,"unit"),e instanceof ChronoUnit){switch(e){case ChronoUnit.DAYS:return this.plusDays(t);case ChronoUnit.WEEKS:return this.plusWeeks(t);case ChronoUnit.MONTHS:return this.plusMonths(t);case ChronoUnit.YEARS:return this.plusYears(t);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(t,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(t,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(t,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),t))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}return e.addTo(this,t)}plusYears(t){if(0===t)return this;const e=ChronoField.YEAR.checkValidIntValue(this._year+t);return LocalDate._resolvePreviousValid(e,this._month,this._day)}plusMonths(t){if(0===t)return this;const e=12*this._year+(this._month-1)+t,o=ChronoField.YEAR.checkValidIntValue(MathUtil.floorDiv(e,12)),r=MathUtil.floorMod(e,12)+1;return LocalDate._resolvePreviousValid(o,r,this._day)}plusWeeks(t){return this.plusDays(MathUtil.safeMultiply(t,7))}plusDays(t){if(0===t)return this;const e=MathUtil.safeAdd(this.toEpochDay(),t);return LocalDate.ofEpochDay(e)}_minusUnit(t,e){return requireNonNull(t,"amountToSubtract"),requireNonNull(e,"unit"),this._plusUnit(-1*t,e)}minusYears(t){return this.plusYears(-1*t)}minusMonths(t){return this.plusMonths(-1*t)}minusWeeks(t){return this.plusWeeks(-1*t)}minusDays(t){return this.plusDays(-1*t)}query(t){return requireNonNull(t,"query"),t===TemporalQueries.localDate()?this:super.query(t)}adjustInto(t){return super.adjustInto(t)}until(t,e){return arguments.length<2?this.until1(t):this.until2(t,e)}until2(t,e){const o=LocalDate.from(t);if(e instanceof ChronoUnit){switch(e){case ChronoUnit.DAYS:return this.daysUntil(o);case ChronoUnit.WEEKS:return MathUtil.intDiv(this.daysUntil(o),7);case ChronoUnit.MONTHS:return this._monthsUntil(o);case ChronoUnit.YEARS:return MathUtil.intDiv(this._monthsUntil(o),12);case ChronoUnit.DECADES:return MathUtil.intDiv(this._monthsUntil(o),120);case ChronoUnit.CENTURIES:return MathUtil.intDiv(this._monthsUntil(o),1200);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(this._monthsUntil(o),12e3);case ChronoUnit.ERAS:return o.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}return e.between(this,o)}daysUntil(t){return t.toEpochDay()-this.toEpochDay()}_monthsUntil(t){const e=32*this._prolepticMonth()+this.dayOfMonth(),o=32*t._prolepticMonth()+t.dayOfMonth();return MathUtil.intDiv(o-e,32)}until1(t){const e=LocalDate.from(t);let o=e._prolepticMonth()-this._prolepticMonth(),r=e._day-this._day;if(o>0&&r<0){o--;const t=this.plusMonths(o);r=e.toEpochDay()-t.toEpochDay()}else o<0&&r>0&&(o++,r-=e.lengthOfMonth());const i=MathUtil.intDiv(o,12),a=MathUtil.intMod(o,12);return Period.of(i,a,r)}atTime(){return 1===arguments.length?this.atTime1.apply(this,arguments):this.atTime4.apply(this,arguments)}atTime1(t){if(requireNonNull(t,"time"),t instanceof LocalTime)return LocalDateTime.of(this,t);if(t instanceof OffsetTime)return this._atTimeOffsetTime(t);throw new IllegalArgumentException("time must be an instance of LocalTime or OffsetTime"+(t&&t.constructor&&t.constructor.name?`, but is ${t.constructor.name}`:""))}atTime4(t,e,o=0,r=0){return this.atTime1(LocalTime.of(t,e,o,r))}_atTimeOffsetTime(t){return OffsetDateTime.of(LocalDateTime.of(this,t.toLocalTime()),t.offset())}atStartOfDay(t){return null!=t?this._atStartOfDayWithZone(t):LocalDateTime.of(this,LocalTime.MIDNIGHT)}_atStartOfDayWithZone(t){requireNonNull(t,"zone");let e=this.atTime(LocalTime.MIDNIGHT);if(t instanceof ZoneOffset==!1){const o=t.rules().transition(e);null!=o&&o.isGap()&&(e=o.dateTimeAfter())}return ZonedDateTime.of(e,t)}toEpochDay(){const t=this._year,e=this._month;let o=0;return o+=365*t,t>=0?o+=MathUtil.intDiv(t+3,4)-MathUtil.intDiv(t+99,100)+MathUtil.intDiv(t+399,400):o-=MathUtil.intDiv(t,-4)-MathUtil.intDiv(t,-100)+MathUtil.intDiv(t,-400),o+=MathUtil.intDiv(367*e-362,12),o+=this.dayOfMonth()-1,e>2&&(o--,IsoChronology.isLeapYear(t)||o--),o-719528}compareTo(t){return requireNonNull(t,"other"),requireInstance(t,LocalDate,"other"),this._compareTo0(t)}_compareTo0(t){let e=this._year-t._year;return 0===e&&(e=this._month-t._month,0===e&&(e=this._day-t._day)),e}isAfter(t){return this.compareTo(t)>0}isBefore(t){return this.compareTo(t)<0}isEqual(t){return 0===this.compareTo(t)}equals(t){return this===t||t instanceof LocalDate&&0===this._compareTo0(t)}hashCode(){const t=this._year,e=this._month,o=this._day;return MathUtil.hash(4294965248&t^(t<<11)+(e<<6)+o)}toString(){let t,e,o;const r=this._year,i=this._month,a=this._day;return o=Math.abs(r)<1e3?r<0?`-${(""+(r-1e4)).slice(-4)}`:`${r+1e4}`.slice(-4):r>9999?`+${r}`:`${r}`,e=i<10?`-0${i}`:`-${i}`,t=a<10?`-0${a}`:`-${a}`,o+e+t}toJSON(){return this.toString()}format(t){return requireNonNull(t,"formatter"),requireInstance(t,DateTimeFormatter,"formatter"),super.format(t)}}export function _init(){LocalDate.MIN=LocalDate.of(YearConstants.MIN_VALUE,1,1),LocalDate.MAX=LocalDate.of(YearConstants.MAX_VALUE,12,31),LocalDate.EPOCH_0=LocalDate.ofEpochDay(0),LocalDate.FROM=createTemporalQuery("LocalDate.FROM",(t=>LocalDate.from(t)))}