/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{Instant}from"./Instant";import{LocalDate}from"./LocalDate";import{LocalTime}from"./LocalTime";import{OffsetDateTime}from"./OffsetDateTime";import{ZonedDateTime}from"./ZonedDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ChronoLocalDateTime}from"./chrono/ChronoLocalDateTime";export class LocalDateTime extends ChronoLocalDateTime{static now(t){return null==t?LocalDateTime._now(Clock.systemDefaultZone()):t instanceof Clock?LocalDateTime._now(t):LocalDateTime._now(Clock.system(t))}static _now(t){return requireNonNull(t,"clock"),LocalDateTime.ofInstant(t.instant(),t.zone())}static _ofEpochMillis(t,e){const i=MathUtil.floorDiv(t,1e3)+e.totalSeconds(),o=MathUtil.floorDiv(i,LocalTime.SECONDS_PER_DAY),a=MathUtil.floorMod(i,LocalTime.SECONDS_PER_DAY),r=1e6*MathUtil.floorMod(t,1e3),s=LocalDate.ofEpochDay(o),n=LocalTime.ofSecondOfDay(a,r);return new LocalDateTime(s,n)}static of(){return arguments.length<=2?LocalDateTime.ofDateAndTime.apply(this,arguments):LocalDateTime.ofNumbers.apply(this,arguments)}static ofNumbers(t,e,i,o=0,a=0,r=0,s=0){const n=LocalDate.of(t,e,i),l=LocalTime.of(o,a,r,s);return new LocalDateTime(n,l)}static ofDateAndTime(t,e){return requireNonNull(t,"date"),requireNonNull(e,"time"),new LocalDateTime(t,e)}static ofInstant(t,e=ZoneId.systemDefault()){requireNonNull(t,"instant"),requireInstance(t,Instant,"instant"),requireNonNull(e,"zone");const i=e.rules().offset(t);return LocalDateTime.ofEpochSecond(t.epochSecond(),t.nano(),i)}static ofEpochSecond(t=0,e=0,i){2===arguments.length&&e instanceof ZoneOffset&&(i=e,e=0),requireNonNull(i,"offset");const o=t+i.totalSeconds(),a=MathUtil.floorDiv(o,LocalTime.SECONDS_PER_DAY),r=MathUtil.floorMod(o,LocalTime.SECONDS_PER_DAY),s=LocalDate.ofEpochDay(a),n=LocalTime.ofSecondOfDay(r,e);return new LocalDateTime(s,n)}static from(t){if(requireNonNull(t,"temporal"),t instanceof LocalDateTime)return t;if(t instanceof ZonedDateTime)return t.toLocalDateTime();try{const e=LocalDate.from(t),i=LocalTime.from(t);return new LocalDateTime(e,i)}catch(e){throw new DateTimeException(`Unable to obtain LocalDateTime TemporalAccessor: ${t}, type ${null!=t.constructor?t.constructor.name:""}`)}}static parse(t,e=DateTimeFormatter.ISO_LOCAL_DATE_TIME){return requireNonNull(e,"formatter"),e.parse(t,LocalDateTime.FROM)}constructor(t,e){super(),requireInstance(t,LocalDate,"date"),requireInstance(e,LocalTime,"time"),this._date=t,this._time=e}_withDateTime(t,e){return this._date.equals(t)&&this._time.equals(e)?this:new LocalDateTime(t,e)}isSupported(t){return t instanceof ChronoField||t instanceof ChronoUnit?t.isDateBased()||t.isTimeBased():null!=t&&t.isSupportedBy(this)}range(t){return t instanceof ChronoField?t.isTimeBased()?this._time.range(t):this._date.range(t):t.rangeRefinedBy(this)}get(t){return t instanceof ChronoField?t.isTimeBased()?this._time.get(t):this._date.get(t):super.get(t)}getLong(t){return requireNonNull(t,"field"),t instanceof ChronoField?t.isTimeBased()?this._time.getLong(t):this._date.getLong(t):t.getFrom(this)}year(){return this._date.year()}monthValue(){return this._date.monthValue()}month(){return this._date.month()}dayOfMonth(){return this._date.dayOfMonth()}dayOfYear(){return this._date.dayOfYear()}dayOfWeek(){return this._date.dayOfWeek()}hour(){return this._time.hour()}minute(){return this._time.minute()}second(){return this._time.second()}nano(){return this._time.nano()}_withAdjuster(t){return requireNonNull(t,"adjuster"),t instanceof LocalDate?this._withDateTime(t,this._time):t instanceof LocalTime?this._withDateTime(this._date,t):t instanceof LocalDateTime?t:super._withAdjuster(t)}_withField(t,e){return requireNonNull(t,"field"),t instanceof ChronoField?t.isTimeBased()?this._withDateTime(this._date,this._time.with(t,e)):this._withDateTime(this._date.with(t,e),this._time):t.adjustInto(this,e)}withYear(t){return this._withDateTime(this._date.withYear(t),this._time)}withMonth(t){return this._withDateTime(this._date.withMonth(t),this._time)}withDayOfMonth(t){return this._withDateTime(this._date.withDayOfMonth(t),this._time)}withDayOfYear(t){return this._withDateTime(this._date.withDayOfYear(t),this._time)}withHour(t){const e=this._time.withHour(t);return this._withDateTime(this._date,e)}withMinute(t){const e=this._time.withMinute(t);return this._withDateTime(this._date,e)}withSecond(t){const e=this._time.withSecond(t);return this._withDateTime(this._date,e)}withNano(t){const e=this._time.withNano(t);return this._withDateTime(this._date,e)}truncatedTo(t){return this._withDateTime(this._date,this._time.truncatedTo(t))}_plusUnit(t,e){if(requireNonNull(e,"unit"),e instanceof ChronoUnit){switch(e){case ChronoUnit.NANOS:return this.plusNanos(t);case ChronoUnit.MICROS:return this.plusDays(MathUtil.intDiv(t,LocalTime.MICROS_PER_DAY)).plusNanos(1e3*MathUtil.intMod(t,LocalTime.MICROS_PER_DAY));case ChronoUnit.MILLIS:return this.plusDays(MathUtil.intDiv(t,LocalTime.MILLIS_PER_DAY)).plusNanos(1e6*MathUtil.intMod(t,LocalTime.MILLIS_PER_DAY));case ChronoUnit.SECONDS:return this.plusSeconds(t);case ChronoUnit.MINUTES:return this.plusMinutes(t);case ChronoUnit.HOURS:return this.plusHours(t);case ChronoUnit.HALF_DAYS:return this.plusDays(MathUtil.intDiv(t,256)).plusHours(12*MathUtil.intMod(t,256))}return this._withDateTime(this._date.plus(t,e),this._time)}return e.addTo(this,t)}plusYears(t){const e=this._date.plusYears(t);return this._withDateTime(e,this._time)}plusMonths(t){const e=this._date.plusMonths(t);return this._withDateTime(e,this._time)}plusWeeks(t){const e=this._date.plusWeeks(t);return this._withDateTime(e,this._time)}plusDays(t){const e=this._date.plusDays(t);return this._withDateTime(e,this._time)}plusHours(t){return this._plusWithOverflow(this._date,t,0,0,0,1)}plusMinutes(t){return this._plusWithOverflow(this._date,0,t,0,0,1)}plusSeconds(t){return this._plusWithOverflow(this._date,0,0,t,0,1)}plusNanos(t){return this._plusWithOverflow(this._date,0,0,0,t,1)}_minusUnit(t,e){return requireNonNull(e,"unit"),this._plusUnit(-1*t,e)}minusYears(t){return this.plusYears(-1*t)}minusMonths(t){return this.plusMonths(-1*t)}minusWeeks(t){return this.plusWeeks(-1*t)}minusDays(t){return this.plusDays(-1*t)}minusHours(t){return this._plusWithOverflow(this._date,t,0,0,0,-1)}minusMinutes(t){return this._plusWithOverflow(this._date,0,t,0,0,-1)}minusSeconds(t){return this._plusWithOverflow(this._date,0,0,t,0,-1)}minusNanos(t){return this._plusWithOverflow(this._date,0,0,0,t,-1)}_plusWithOverflow(t,e,i,o,a,r){if(0===e&&0===i&&0===o&&0===a)return this._withDateTime(t,this._time);let s=MathUtil.intDiv(a,LocalTime.NANOS_PER_DAY)+MathUtil.intDiv(o,LocalTime.SECONDS_PER_DAY)+MathUtil.intDiv(i,LocalTime.MINUTES_PER_DAY)+MathUtil.intDiv(e,LocalTime.HOURS_PER_DAY);s*=r;let n=MathUtil.intMod(a,LocalTime.NANOS_PER_DAY)+MathUtil.intMod(o,LocalTime.SECONDS_PER_DAY)*LocalTime.NANOS_PER_SECOND+MathUtil.intMod(i,LocalTime.MINUTES_PER_DAY)*LocalTime.NANOS_PER_MINUTE+MathUtil.intMod(e,LocalTime.HOURS_PER_DAY)*LocalTime.NANOS_PER_HOUR;const l=this._time.toNanoOfDay();n=n*r+l,s+=MathUtil.floorDiv(n,LocalTime.NANOS_PER_DAY);const h=MathUtil.floorMod(n,LocalTime.NANOS_PER_DAY),u=h===l?this._time:LocalTime.ofNanoOfDay(h);return this._withDateTime(t.plusDays(s),u)}query(t){return requireNonNull(t,"query"),t===TemporalQueries.localDate()?this.toLocalDate():super.query(t)}adjustInto(t){return super.adjustInto(t)}until(t,e){requireNonNull(t,"endExclusive"),requireNonNull(e,"unit");const i=LocalDateTime.from(t);if(e instanceof ChronoUnit){if(e.isTimeBased()){let t=this._date.daysUntil(i._date),o=i._time.toNanoOfDay()-this._time.toNanoOfDay();t>0&&o<0?(t--,o+=LocalTime.NANOS_PER_DAY):t<0&&o>0&&(t++,o-=LocalTime.NANOS_PER_DAY);let a=t;switch(e){case ChronoUnit.NANOS:return a=MathUtil.safeMultiply(a,LocalTime.NANOS_PER_DAY),MathUtil.safeAdd(a,o);case ChronoUnit.MICROS:return a=MathUtil.safeMultiply(a,LocalTime.MICROS_PER_DAY),MathUtil.safeAdd(a,MathUtil.intDiv(o,1e3));case ChronoUnit.MILLIS:return a=MathUtil.safeMultiply(a,LocalTime.MILLIS_PER_DAY),MathUtil.safeAdd(a,MathUtil.intDiv(o,1e6));case ChronoUnit.SECONDS:return a=MathUtil.safeMultiply(a,LocalTime.SECONDS_PER_DAY),MathUtil.safeAdd(a,MathUtil.intDiv(o,LocalTime.NANOS_PER_SECOND));case ChronoUnit.MINUTES:return a=MathUtil.safeMultiply(a,LocalTime.MINUTES_PER_DAY),MathUtil.safeAdd(a,MathUtil.intDiv(o,LocalTime.NANOS_PER_MINUTE));case ChronoUnit.HOURS:return a=MathUtil.safeMultiply(a,LocalTime.HOURS_PER_DAY),MathUtil.safeAdd(a,MathUtil.intDiv(o,LocalTime.NANOS_PER_HOUR));case ChronoUnit.HALF_DAYS:return a=MathUtil.safeMultiply(a,2),MathUtil.safeAdd(a,MathUtil.intDiv(o,12*LocalTime.NANOS_PER_HOUR))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}let t=i._date;const o=i._time;return t.isAfter(this._date)&&o.isBefore(this._time)?t=t.minusDays(1):t.isBefore(this._date)&&o.isAfter(this._time)&&(t=t.plusDays(1)),this._date.until(t,e)}return e.between(this,i)}atOffset(t){return OffsetDateTime.of(this,t)}atZone(t){return ZonedDateTime.of(this,t)}toLocalDate(){return this._date}toLocalTime(){return this._time}compareTo(t){return requireNonNull(t,"other"),requireInstance(t,LocalDateTime,"other"),this._compareTo0(t)}_compareTo0(t){let e=this._date.compareTo(t.toLocalDate());return 0===e&&(e=this._time.compareTo(t.toLocalTime())),e}isAfter(t){return this.compareTo(t)>0}isBefore(t){return this.compareTo(t)<0}isEqual(t){return 0===this.compareTo(t)}equals(t){return this===t||t instanceof LocalDateTime&&(this._date.equals(t._date)&&this._time.equals(t._time))}hashCode(){return this._date.hashCode()^this._time.hashCode()}toString(){return`${this._date.toString()}T${this._time.toString()}`}toJSON(){return this.toString()}format(t){return requireNonNull(t,"formatter"),t.format(this)}}export function _init(){LocalDateTime.MIN=LocalDateTime.of(LocalDate.MIN,LocalTime.MIN),LocalDateTime.MAX=LocalDateTime.of(LocalDate.MAX,LocalTime.MAX),LocalDateTime.FROM=createTemporalQuery("LocalDateTime.FROM",(t=>LocalDateTime.from(t)))}