/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{LocalDateTime}from"./LocalDateTime";import{ZoneId}from"./ZoneId";import{OffsetTime}from"./OffsetTime";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";export class LocalTime extends Temporal{static now(t){return null==t?LocalTime._now(Clock.systemDefaultZone()):t instanceof Clock?LocalTime._now(t):LocalTime._now(Clock.system(t))}static _now(t=Clock.systemDefaultZone()){return requireNonNull(t,"clock"),LocalTime.ofInstant(t.instant(),t.zone())}static ofInstant(t,e=ZoneId.systemDefault()){const i=e.rules().offset(t);let o=MathUtil.intMod(t.epochSecond(),LocalTime.SECONDS_PER_DAY);return o=MathUtil.intMod(o+i.totalSeconds(),LocalTime.SECONDS_PER_DAY),o<0&&(o+=LocalTime.SECONDS_PER_DAY),LocalTime.ofSecondOfDay(o,t.nano())}static of(t,e,i,o){return new LocalTime(t,e,i,o)}static ofSecondOfDay(t=0,e=0){ChronoField.SECOND_OF_DAY.checkValidValue(t),ChronoField.NANO_OF_SECOND.checkValidValue(e);const i=MathUtil.intDiv(t,LocalTime.SECONDS_PER_HOUR);t-=i*LocalTime.SECONDS_PER_HOUR;const o=MathUtil.intDiv(t,LocalTime.SECONDS_PER_MINUTE);return t-=o*LocalTime.SECONDS_PER_MINUTE,new LocalTime(i,o,t,e)}static ofNanoOfDay(t=0){ChronoField.NANO_OF_DAY.checkValidValue(t);const e=MathUtil.intDiv(t,LocalTime.NANOS_PER_HOUR);t-=e*LocalTime.NANOS_PER_HOUR;const i=MathUtil.intDiv(t,LocalTime.NANOS_PER_MINUTE);t-=i*LocalTime.NANOS_PER_MINUTE;const o=MathUtil.intDiv(t,LocalTime.NANOS_PER_SECOND);return t-=o*LocalTime.NANOS_PER_SECOND,new LocalTime(e,i,o,t)}static from(t){requireNonNull(t,"temporal");const e=t.query(TemporalQueries.localTime());if(null==e)throw new DateTimeException(`Unable to obtain LocalTime TemporalAccessor: ${t}, type ${null!=t.constructor?t.constructor.name:""}`);return e}static parse(t,e=DateTimeFormatter.ISO_LOCAL_TIME){return requireNonNull(e,"formatter"),e.parse(t,LocalTime.FROM)}constructor(t=0,e=0,i=0,o=0){super();const n=MathUtil.safeToInt(t),r=MathUtil.safeToInt(e),a=MathUtil.safeToInt(i),l=MathUtil.safeToInt(o);if(LocalTime._validate(n,r,a,l),0===r&&0===a&&0===l)return LocalTime.HOURS[n]||(this._hour=n,this._minute=r,this._second=a,this._nano=l,LocalTime.HOURS[n]=this),LocalTime.HOURS[n];this._hour=n,this._minute=r,this._second=a,this._nano=l}static _validate(t,e,i,o){ChronoField.HOUR_OF_DAY.checkValidValue(t),ChronoField.MINUTE_OF_HOUR.checkValidValue(e),ChronoField.SECOND_OF_MINUTE.checkValidValue(i),ChronoField.NANO_OF_SECOND.checkValidValue(o)}isSupported(t){return t instanceof ChronoField||t instanceof ChronoUnit?t.isTimeBased():null!=t&&t.isSupportedBy(this)}range(t){return requireNonNull(t),super.range(t)}get(t){return this.getLong(t)}getLong(t){return requireNonNull(t,"field"),t instanceof ChronoField?this._get0(t):t.getFrom(this)}_get0(t){switch(t){case ChronoField.NANO_OF_SECOND:return this._nano;case ChronoField.NANO_OF_DAY:return this.toNanoOfDay();case ChronoField.MICRO_OF_SECOND:return MathUtil.intDiv(this._nano,1e3);case ChronoField.MICRO_OF_DAY:return MathUtil.intDiv(this.toNanoOfDay(),1e3);case ChronoField.MILLI_OF_SECOND:return MathUtil.intDiv(this._nano,1e6);case ChronoField.MILLI_OF_DAY:return MathUtil.intDiv(this.toNanoOfDay(),1e6);case ChronoField.SECOND_OF_MINUTE:return this._second;case ChronoField.SECOND_OF_DAY:return this.toSecondOfDay();case ChronoField.MINUTE_OF_HOUR:return this._minute;case ChronoField.MINUTE_OF_DAY:return 60*this._hour+this._minute;case ChronoField.HOUR_OF_AMPM:return MathUtil.intMod(this._hour,12);case ChronoField.CLOCK_HOUR_OF_AMPM:{const t=MathUtil.intMod(this._hour,12);return t%12==0?12:t}case ChronoField.HOUR_OF_DAY:return this._hour;case ChronoField.CLOCK_HOUR_OF_DAY:return 0===this._hour?24:this._hour;case ChronoField.AMPM_OF_DAY:return MathUtil.intDiv(this._hour,12)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}hour(){return this._hour}minute(){return this._minute}second(){return this._second}nano(){return this._nano}_withAdjuster(t){return requireNonNull(t,"adjuster"),t instanceof LocalTime?t:super._withAdjuster(t)}_withField(t,e){if(requireNonNull(t,"field"),requireInstance(t,TemporalField,"field"),t instanceof ChronoField){switch(t.checkValidValue(e),t){case ChronoField.NANO_OF_SECOND:return this.withNano(e);case ChronoField.NANO_OF_DAY:return LocalTime.ofNanoOfDay(e);case ChronoField.MICRO_OF_SECOND:return this.withNano(1e3*e);case ChronoField.MICRO_OF_DAY:return LocalTime.ofNanoOfDay(1e3*e);case ChronoField.MILLI_OF_SECOND:return this.withNano(1e6*e);case ChronoField.MILLI_OF_DAY:return LocalTime.ofNanoOfDay(1e6*e);case ChronoField.SECOND_OF_MINUTE:return this.withSecond(e);case ChronoField.SECOND_OF_DAY:return this.plusSeconds(e-this.toSecondOfDay());case ChronoField.MINUTE_OF_HOUR:return this.withMinute(e);case ChronoField.MINUTE_OF_DAY:return this.plusMinutes(e-(60*this._hour+this._minute));case ChronoField.HOUR_OF_AMPM:return this.plusHours(e-MathUtil.intMod(this._hour,12));case ChronoField.CLOCK_HOUR_OF_AMPM:return this.plusHours((12===e?0:e)-MathUtil.intMod(this._hour,12));case ChronoField.HOUR_OF_DAY:return this.withHour(e);case ChronoField.CLOCK_HOUR_OF_DAY:return this.withHour(24===e?0:e);case ChronoField.AMPM_OF_DAY:return this.plusHours(12*(e-MathUtil.intDiv(this._hour,12)))}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.adjustInto(this,e)}withHour(t=0){return this._hour===t?this:new LocalTime(t,this._minute,this._second,this._nano)}withMinute(t=0){return this._minute===t?this:new LocalTime(this._hour,t,this._second,this._nano)}withSecond(t=0){return this._second===t?this:new LocalTime(this._hour,this._minute,t,this._nano)}withNano(t=0){return this._nano===t?this:new LocalTime(this._hour,this._minute,this._second,t)}truncatedTo(t){if(requireNonNull(t,"unit"),t===ChronoUnit.NANOS)return this;const e=t.duration();if(e.seconds()>LocalTime.SECONDS_PER_DAY)throw new DateTimeException("Unit is too large to be used for truncation");const i=e.toNanos();if(0!==MathUtil.intMod(LocalTime.NANOS_PER_DAY,i))throw new DateTimeException("Unit must divide into a standard day without remainder");const o=this.toNanoOfDay();return LocalTime.ofNanoOfDay(MathUtil.intDiv(o,i)*i)}_plusUnit(t,e){if(requireNonNull(e,"unit"),e instanceof ChronoUnit){switch(e){case ChronoUnit.NANOS:return this.plusNanos(t);case ChronoUnit.MICROS:return this.plusNanos(1e3*MathUtil.intMod(t,LocalTime.MICROS_PER_DAY));case ChronoUnit.MILLIS:return this.plusNanos(1e6*MathUtil.intMod(t,LocalTime.MILLIS_PER_DAY));case ChronoUnit.SECONDS:return this.plusSeconds(t);case ChronoUnit.MINUTES:return this.plusMinutes(t);case ChronoUnit.HOURS:return this.plusHours(t);case ChronoUnit.HALF_DAYS:return this.plusHours(12*MathUtil.intMod(t,2))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}return e.addTo(this,t)}plusHours(t){if(0===t)return this;const e=MathUtil.intMod(MathUtil.intMod(t,LocalTime.HOURS_PER_DAY)+this._hour+LocalTime.HOURS_PER_DAY,LocalTime.HOURS_PER_DAY);return new LocalTime(e,this._minute,this._second,this._nano)}plusMinutes(t){if(0===t)return this;const e=this._hour*LocalTime.MINUTES_PER_HOUR+this._minute,i=MathUtil.intMod(MathUtil.intMod(t,LocalTime.MINUTES_PER_DAY)+e+LocalTime.MINUTES_PER_DAY,LocalTime.MINUTES_PER_DAY);if(e===i)return this;const o=MathUtil.intDiv(i,LocalTime.MINUTES_PER_HOUR),n=MathUtil.intMod(i,LocalTime.MINUTES_PER_HOUR);return new LocalTime(o,n,this._second,this._nano)}plusSeconds(t){if(0===t)return this;const e=this._hour*LocalTime.SECONDS_PER_HOUR+this._minute*LocalTime.SECONDS_PER_MINUTE+this._second,i=MathUtil.intMod(MathUtil.intMod(t,LocalTime.SECONDS_PER_DAY)+e+LocalTime.SECONDS_PER_DAY,LocalTime.SECONDS_PER_DAY);if(e===i)return this;const o=MathUtil.intDiv(i,LocalTime.SECONDS_PER_HOUR),n=MathUtil.intMod(MathUtil.intDiv(i,LocalTime.SECONDS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR),r=MathUtil.intMod(i,LocalTime.SECONDS_PER_MINUTE);return new LocalTime(o,n,r,this._nano)}plusNanos(t){if(0===t)return this;const e=this.toNanoOfDay(),i=MathUtil.intMod(MathUtil.intMod(t,LocalTime.NANOS_PER_DAY)+e+LocalTime.NANOS_PER_DAY,LocalTime.NANOS_PER_DAY);if(e===i)return this;const o=MathUtil.intDiv(i,LocalTime.NANOS_PER_HOUR),n=MathUtil.intMod(MathUtil.intDiv(i,LocalTime.NANOS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR),r=MathUtil.intMod(MathUtil.intDiv(i,LocalTime.NANOS_PER_SECOND),LocalTime.SECONDS_PER_MINUTE),a=MathUtil.intMod(i,LocalTime.NANOS_PER_SECOND);return new LocalTime(o,n,r,a)}_minusUnit(t,e){return requireNonNull(e,"unit"),this._plusUnit(-1*t,e)}minusHours(t){return this.plusHours(-1*MathUtil.intMod(t,LocalTime.HOURS_PER_DAY))}minusMinutes(t){return this.plusMinutes(-1*MathUtil.intMod(t,LocalTime.MINUTES_PER_DAY))}minusSeconds(t){return this.plusSeconds(-1*MathUtil.intMod(t,LocalTime.SECONDS_PER_DAY))}minusNanos(t){return this.plusNanos(-1*MathUtil.intMod(t,LocalTime.NANOS_PER_DAY))}query(t){return requireNonNull(t,"query"),t===TemporalQueries.precision()?ChronoUnit.NANOS:t===TemporalQueries.localTime()?this:t===TemporalQueries.chronology()||t===TemporalQueries.zoneId()||t===TemporalQueries.zone()||t===TemporalQueries.offset()||t===TemporalQueries.localDate()?null:t.queryFrom(this)}adjustInto(t){return t.with(LocalTime.NANO_OF_DAY,this.toNanoOfDay())}until(t,e){requireNonNull(t,"endExclusive"),requireNonNull(e,"unit");const i=LocalTime.from(t);if(e instanceof ChronoUnit){const t=i.toNanoOfDay()-this.toNanoOfDay();switch(e){case ChronoUnit.NANOS:return t;case ChronoUnit.MICROS:return MathUtil.intDiv(t,1e3);case ChronoUnit.MILLIS:return MathUtil.intDiv(t,1e6);case ChronoUnit.SECONDS:return MathUtil.intDiv(t,LocalTime.NANOS_PER_SECOND);case ChronoUnit.MINUTES:return MathUtil.intDiv(t,LocalTime.NANOS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(t,LocalTime.NANOS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(t,12*LocalTime.NANOS_PER_HOUR)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}return e.between(this,i)}atDate(t){return LocalDateTime.of(t,this)}atOffset(t){return OffsetTime.of(this,t)}toSecondOfDay(){let t=this._hour*LocalTime.SECONDS_PER_HOUR;return t+=this._minute*LocalTime.SECONDS_PER_MINUTE,t+=this._second,t}toNanoOfDay(){let t=this._hour*LocalTime.NANOS_PER_HOUR;return t+=this._minute*LocalTime.NANOS_PER_MINUTE,t+=this._second*LocalTime.NANOS_PER_SECOND,t+=this._nano,t}compareTo(t){requireNonNull(t,"other"),requireInstance(t,LocalTime,"other");let e=MathUtil.compareNumbers(this._hour,t._hour);return 0===e&&(e=MathUtil.compareNumbers(this._minute,t._minute),0===e&&(e=MathUtil.compareNumbers(this._second,t._second),0===e&&(e=MathUtil.compareNumbers(this._nano,t._nano)))),e}isAfter(t){return this.compareTo(t)>0}isBefore(t){return this.compareTo(t)<0}equals(t){return this===t||t instanceof LocalTime&&(this._hour===t._hour&&this._minute===t._minute&&this._second===t._second&&this._nano===t._nano)}hashCode(){const t=this.toNanoOfDay();return MathUtil.hash(t)}toString(){let t="";const e=this._hour,i=this._minute,o=this._second,n=this._nano;return t+=e<10?"0":"",t+=e,t+=i<10?":0":":",t+=i,(o>0||n>0)&&(t+=o<10?":0":":",t+=o,n>0&&(t+=".",0===MathUtil.intMod(n,1e6)?t+=`${MathUtil.intDiv(n,1e6)+1e3}`.substring(1):0===MathUtil.intMod(n,1e3)?t+=`${MathUtil.intDiv(n,1e3)+1e6}`.substring(1):t+=`${n+1e9}`.substring(1))),t}toJSON(){return this.toString()}format(t){return requireNonNull(t,"formatter"),t.format(this)}}export function _init(){LocalTime.HOURS=[];for(let t=0;t<24;t++)LocalTime.of(t,0,0,0);LocalTime.MIN=LocalTime.HOURS[0],LocalTime.MAX=new LocalTime(23,59,59,999999999),LocalTime.MIDNIGHT=LocalTime.HOURS[0],LocalTime.NOON=LocalTime.HOURS[12],LocalTime.FROM=createTemporalQuery("LocalTime.FROM",(t=>LocalTime.from(t)))}LocalTime.HOURS_PER_DAY=24,LocalTime.MINUTES_PER_HOUR=60,LocalTime.MINUTES_PER_DAY=LocalTime.MINUTES_PER_HOUR*LocalTime.HOURS_PER_DAY,LocalTime.SECONDS_PER_MINUTE=60,LocalTime.SECONDS_PER_HOUR=LocalTime.SECONDS_PER_MINUTE*LocalTime.MINUTES_PER_HOUR,LocalTime.SECONDS_PER_DAY=LocalTime.SECONDS_PER_HOUR*LocalTime.HOURS_PER_DAY,LocalTime.MILLIS_PER_DAY=1e3*LocalTime.SECONDS_PER_DAY,LocalTime.MICROS_PER_DAY=1e6*LocalTime.SECONDS_PER_DAY,LocalTime.NANOS_PER_SECOND=1e9,LocalTime.NANOS_PER_MINUTE=LocalTime.NANOS_PER_SECOND*LocalTime.SECONDS_PER_MINUTE,LocalTime.NANOS_PER_HOUR=LocalTime.NANOS_PER_MINUTE*LocalTime.MINUTES_PER_HOUR,LocalTime.NANOS_PER_DAY=LocalTime.NANOS_PER_HOUR*LocalTime.HOURS_PER_DAY;