/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @license BSD-3-Clause (see LICENSE.md in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{TemporalAccessor}from"./temporal/TemporalAccessor";import{TemporalQuery,createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalQueries}from"./temporal/TemporalQueries";import{ValueRange}from"./temporal/ValueRange";import{Year}from"./Year";import{ZoneId}from"./ZoneId";export class MonthDay extends TemporalAccessor{static now(t){return 0===arguments.length?MonthDay.now0():1===arguments.length&&t instanceof ZoneId?MonthDay.nowZoneId(t):MonthDay.nowClock(t)}static now0(){return this.nowClock(Clock.systemDefaultZone())}static nowZoneId(t){return requireNonNull(t,"zone"),this.nowClock(Clock.system(t))}static nowClock(t){requireNonNull(t,"clock");const e=LocalDate.now(t);return MonthDay.of(e.month(),e.dayOfMonth())}static of(t,e){return 2===arguments.length&&t instanceof Month?MonthDay.ofMonthNumber(t,e):MonthDay.ofNumberNumber(t,e)}static ofMonthNumber(t,e){if(requireNonNull(t,"month"),ChronoField.DAY_OF_MONTH.checkValidValue(e),e>t.maxLength())throw new DateTimeException(`Illegal value for DayOfMonth field, value ${e} is not valid for month ${t.toString()}`);return new MonthDay(t.value(),e)}static ofNumberNumber(t,e){return requireNonNull(t,"month"),requireNonNull(e,"dayOfMonth"),MonthDay.of(Month.of(t),e)}static from(t){if(requireNonNull(t,"temporal"),requireInstance(t,TemporalAccessor,"temporal"),t instanceof MonthDay)return t;try{return MonthDay.of(t.get(ChronoField.MONTH_OF_YEAR),t.get(ChronoField.DAY_OF_MONTH))}catch(e){throw new DateTimeException(`Unable to obtain MonthDay from TemporalAccessor: ${t}, type ${t&&null!=t.constructor?t.constructor.name:""}`)}}static parse(t,e){return 1===arguments.length?MonthDay.parseString(t):MonthDay.parseStringFormatter(t,e)}static parseString(t){return MonthDay.parseStringFormatter(t,PARSER)}static parseStringFormatter(t,e){return requireNonNull(t,"text"),requireNonNull(e,"formatter"),requireInstance(e,DateTimeFormatter,"formatter"),e.parse(t,MonthDay.FROM)}constructor(t,e){super(),this._month=MathUtil.safeToInt(t),this._day=MathUtil.safeToInt(e)}monthValue(){return this._month}month(){return Month.of(this._month)}dayOfMonth(){return this._day}isSupported(t){return t instanceof ChronoField?t===ChronoField.MONTH_OF_YEAR||t===ChronoField.DAY_OF_MONTH:null!=t&&t.isSupportedBy(this)}range(t){return t===ChronoField.MONTH_OF_YEAR?t.range():t===ChronoField.DAY_OF_MONTH?ValueRange.of(1,this.month().minLength(),this.month().maxLength()):super.range(t)}get(t){return this.range(t).checkValidIntValue(this.getLong(t),t)}getLong(t){if(requireNonNull(t,"field"),t instanceof ChronoField){switch(t){case ChronoField.DAY_OF_MONTH:return this._day;case ChronoField.MONTH_OF_YEAR:return this._month}throw new UnsupportedTemporalTypeException(`Unsupported field: ${t}`)}return t.getFrom(this)}isValidYear(t){return!1==(29===this._day&&2===this._month&&!1===Year.isLeap(t))}withMonth(t){return this.with(Month.of(t))}with(t){if(requireNonNull(t,"month"),t.value()===this._month)return this;const e=Math.min(this._day,t.maxLength());return new MonthDay(t.value(),e)}withDayOfMonth(t){return t===this._day?this:MonthDay.of(this._month,t)}query(t){return requireNonNull(t,"query"),requireInstance(t,TemporalQuery,"query"),t===TemporalQueries.chronology()?IsoChronology.INSTANCE:super.query(t)}adjustInto(t){return requireNonNull(t,"temporal"),(t=t.with(ChronoField.MONTH_OF_YEAR,this._month)).with(ChronoField.DAY_OF_MONTH,Math.min(t.range(ChronoField.DAY_OF_MONTH).maximum(),this._day))}atYear(t){return LocalDate.of(t,this._month,this.isValidYear(t)?this._day:28)}compareTo(t){requireNonNull(t,"other"),requireInstance(t,MonthDay,"other");let e=this._month-t.monthValue();return 0===e&&(e=this._day-t.dayOfMonth()),e}isAfter(t){return requireNonNull(t,"other"),requireInstance(t,MonthDay,"other"),this.compareTo(t)>0}isBefore(t){return requireNonNull(t,"other"),requireInstance(t,MonthDay,"other"),this.compareTo(t)<0}equals(t){if(this===t)return!0;if(t instanceof MonthDay){const e=t;return this.monthValue()===e.monthValue()&&this.dayOfMonth()===e.dayOfMonth()}return!1}toString(){return`--${this._month<10?"0":""}${this._month}${this._day<10?"-0":"-"}${this._day}`}toJSON(){return this.toString()}format(t){return requireNonNull(t,"formatter"),requireInstance(t,DateTimeFormatter,"formatter"),t.format(this)}}let PARSER;export function _init(){PARSER=(new DateTimeFormatterBuilder).appendLiteral("--").appendValue(ChronoField.MONTH_OF_YEAR,2).appendLiteral("-").appendValue(ChronoField.DAY_OF_MONTH,2).toFormatter(),MonthDay.FROM=createTemporalQuery("MonthDay.FROM",(t=>MonthDay.from(t)))}