/**
 * @copyright (c) 2016-present, Philipp Thürwächter & Pattrick Hüper  & js-joda contributors
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Instant}from"./Instant";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDateTime}from"./LocalDateTime";import{LocalDate}from"./LocalDate";import{LocalTime}from"./LocalTime";import{MathUtil}from"./MathUtil";import{OffsetTime}from"./OffsetTime";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZonedDateTime}from"./ZonedDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{DateTimeException,IllegalArgumentException}from"./errors";import{createTemporalQuery}from"./temporal/TemporalQuery";import{requireInstance,requireNonNull}from"./assert";export class OffsetDateTime extends Temporal{static from(t){if(requireNonNull(t,"temporal"),t instanceof OffsetDateTime)return t;try{const e=ZoneOffset.from(t);try{const i=LocalDateTime.from(t);return OffsetDateTime.of(i,e)}catch(i){const o=Instant.from(t);return OffsetDateTime.ofInstant(o,e)}}catch(e){throw new DateTimeException(`Unable to obtain OffsetDateTime TemporalAccessor: ${t}, type ${null!=t.constructor?t.constructor.name:""}`)}}static now(t){if(0===arguments.length)return OffsetDateTime.now(Clock.systemDefaultZone());if(requireNonNull(t,"clockOrZone"),t instanceof ZoneId)return OffsetDateTime.now(Clock.system(t));if(t instanceof Clock){const e=t.instant();return OffsetDateTime.ofInstant(e,t.zone().rules().offset(e))}throw new IllegalArgumentException("clockOrZone must be an instance of ZoneId or Clock")}static of(){return arguments.length<=2?OffsetDateTime.ofDateTime.apply(this,arguments):3===arguments.length?OffsetDateTime.ofDateAndTime.apply(this,arguments):OffsetDateTime.ofNumbers.apply(this,arguments)}static ofDateTime(t,e){return new OffsetDateTime(t,e)}static ofDateAndTime(t,e,i){const o=LocalDateTime.of(t,e);return new OffsetDateTime(o,i)}static ofNumbers(t,e,i,o=0,s=0,r=0,n=0,a){const f=LocalDateTime.of(t,e,i,o,s,r,n);return new OffsetDateTime(f,a)}static ofInstant(t,e){requireNonNull(t,"instant"),requireNonNull(e,"zone");const i=e.rules().offset(t),o=LocalDateTime.ofEpochSecond(t.epochSecond(),t.nano(),i);return new OffsetDateTime(o,i)}static parse(t,e=DateTimeFormatter.ISO_OFFSET_DATE_TIME){return requireNonNull(e,"formatter"),e.parse(t,OffsetDateTime.FROM)}constructor(t,e){super(),requireNonNull(t,"dateTime"),requireInstance(t,LocalDateTime,"dateTime"),requireNonNull(e,"offset"),requireInstance(e,ZoneOffset,"offset"),this._dateTime=t,this._offset=e}adjustInto(t){return t.with(ChronoField.EPOCH_DAY,this.toLocalDate().toEpochDay()).with(ChronoField.NANO_OF_DAY,this.toLocalTime().toNanoOfDay()).with(ChronoField.OFFSET_SECONDS,this.offset().totalSeconds())}until(t,e){let i=OffsetDateTime.from(t);return e instanceof ChronoUnit?(i=i.withOffsetSameInstant(this._offset),this._dateTime.until(i._dateTime,e)):e.between(this,i)}atZoneSameInstant(t){return ZonedDateTime.ofInstant(this._dateTime,this._offset,t)}atZoneSimilarLocal(t){return ZonedDateTime.ofLocal(this._dateTime,t,this._offset)}query(t){return requireNonNull(t,"query"),t===TemporalQueries.chronology()?IsoChronology.INSTANCE:t===TemporalQueries.precision()?ChronoUnit.NANOS:t===TemporalQueries.offset()||t===TemporalQueries.zone()?this.offset():t===TemporalQueries.localDate()?this.toLocalDate():t===TemporalQueries.localTime()?this.toLocalTime():t===TemporalQueries.zoneId()?null:super.query(t)}get(t){if(t instanceof ChronoField){switch(t){case ChronoField.INSTANT_SECONDS:throw new DateTimeException(`Field too large for an int: ${t}`);case ChronoField.OFFSET_SECONDS:return this.offset().totalSeconds()}return this._dateTime.get(t)}return super.get(t)}getLong(t){if(t instanceof ChronoField){switch(t){case ChronoField.INSTANT_SECONDS:return this.toEpochSecond();case ChronoField.OFFSET_SECONDS:return this.offset().totalSeconds()}return this._dateTime.getLong(t)}return t.getFrom(this)}offset(){return this._offset}year(){return this._dateTime.year()}monthValue(){return this._dateTime.monthValue()}month(){return this._dateTime.month()}dayOfMonth(){return this._dateTime.dayOfMonth()}dayOfYear(){return this._dateTime.dayOfYear()}dayOfWeek(){return this._dateTime.dayOfWeek()}hour(){return this._dateTime.hour()}minute(){return this._dateTime.minute()}second(){return this._dateTime.second()}nano(){return this._dateTime.nano()}toLocalDateTime(){return this._dateTime}toLocalDate(){return this._dateTime.toLocalDate()}toLocalTime(){return this._dateTime.toLocalTime()}toOffsetTime(){return OffsetTime.of(this._dateTime.toLocalTime(),this._offset)}toZonedDateTime(){return ZonedDateTime.of(this._dateTime,this._offset)}toInstant(){return this._dateTime.toInstant(this._offset)}toEpochSecond(){return this._dateTime.toEpochSecond(this._offset)}isSupported(t){return t instanceof ChronoField||t instanceof ChronoUnit?t.isDateBased()||t.isTimeBased():null!=t&&t.isSupportedBy(this)}range(t){return t instanceof ChronoField?t===ChronoField.INSTANT_SECONDS||t===ChronoField.OFFSET_SECONDS?t.range():this._dateTime.range(t):t.rangeRefinedBy(this)}_withAdjuster(t){return requireNonNull(t),t instanceof LocalDate||t instanceof LocalTime||t instanceof LocalDateTime?this._withDateTimeOffset(this._dateTime.with(t),this._offset):t instanceof Instant?OffsetDateTime.ofInstant(t,this._offset):t instanceof ZoneOffset?this._withDateTimeOffset(this._dateTime,t):t instanceof OffsetDateTime?t:t.adjustInto(this)}_withField(t,e){if(requireNonNull(t),t instanceof ChronoField){const i=t;switch(i){case ChronoField.INSTANT_SECONDS:return OffsetDateTime.ofInstant(Instant.ofEpochSecond(e,this.nano()),this._offset);case ChronoField.OFFSET_SECONDS:return this._withDateTimeOffset(this._dateTime,ZoneOffset.ofTotalSeconds(i.checkValidIntValue(e)))}return this._withDateTimeOffset(this._dateTime.with(t,e),this._offset)}return t.adjustInto(this,e)}_withDateTimeOffset(t,e){return this._dateTime===t&&this._offset.equals(e)?this:new OffsetDateTime(t,e)}withYear(t){return this._withDateTimeOffset(this._dateTime.withYear(t),this._offset)}withMonth(t){return this._withDateTimeOffset(this._dateTime.withMonth(t),this._offset)}withDayOfMonth(t){return this._withDateTimeOffset(this._dateTime.withDayOfMonth(t),this._offset)}withDayOfYear(t){return this._withDateTimeOffset(this._dateTime.withDayOfYear(t),this._offset)}withHour(t){return this._withDateTimeOffset(this._dateTime.withHour(t),this._offset)}withMinute(t){return this._withDateTimeOffset(this._dateTime.withMinute(t),this._offset)}withSecond(t){return this._withDateTimeOffset(this._dateTime.withSecond(t),this._offset)}withNano(t){return this._withDateTimeOffset(this._dateTime.withNano(t),this._offset)}withOffsetSameLocal(t){return requireNonNull(t,"offset"),this._withDateTimeOffset(this._dateTime,t)}withOffsetSameInstant(t){if(requireNonNull(t,"offset"),t.equals(this._offset))return this;const e=t.totalSeconds()-this._offset.totalSeconds(),i=this._dateTime.plusSeconds(e);return new OffsetDateTime(i,t)}truncatedTo(t){return this._withDateTimeOffset(this._dateTime.truncatedTo(t),this._offset)}_plusAmount(t){return requireNonNull(t,"amount"),t.addTo(this)}_plusUnit(t,e){return e instanceof ChronoUnit?this._withDateTimeOffset(this._dateTime.plus(t,e),this._offset):e.addTo(this,t)}plusYears(t){return this._withDateTimeOffset(this._dateTime.plusYears(t),this._offset)}plusMonths(t){return this._withDateTimeOffset(this._dateTime.plusMonths(t),this._offset)}plusWeeks(t){return this._withDateTimeOffset(this._dateTime.plusWeeks(t),this._offset)}plusDays(t){return this._withDateTimeOffset(this._dateTime.plusDays(t),this._offset)}plusHours(t){return this._withDateTimeOffset(this._dateTime.plusHours(t),this._offset)}plusMinutes(t){return this._withDateTimeOffset(this._dateTime.plusMinutes(t),this._offset)}plusSeconds(t){return this._withDateTimeOffset(this._dateTime.plusSeconds(t),this._offset)}plusNanos(t){return this._withDateTimeOffset(this._dateTime.plusNanos(t),this._offset)}_minusAmount(t){return requireNonNull(t),t.subtractFrom(this)}_minusUnit(t,e){return this.plus(-1*t,e)}minusYears(t){return this._withDateTimeOffset(this._dateTime.minusYears(t),this._offset)}minusMonths(t){return this._withDateTimeOffset(this._dateTime.minusMonths(t),this._offset)}minusWeeks(t){return this._withDateTimeOffset(this._dateTime.minusWeeks(t),this._offset)}minusDays(t){return this._withDateTimeOffset(this._dateTime.minusDays(t),this._offset)}minusHours(t){return this._withDateTimeOffset(this._dateTime.minusHours(t),this._offset)}minusMinutes(t){return this._withDateTimeOffset(this._dateTime.minusMinutes(t),this._offset)}minusSeconds(t){return this._withDateTimeOffset(this._dateTime.minusSeconds(t),this._offset)}minusNanos(t){return this._withDateTimeOffset(this._dateTime.minusNanos(t),this._offset)}compareTo(t){if(requireNonNull(t,"other"),requireInstance(t,OffsetDateTime,"other"),this.offset().equals(t.offset()))return this.toLocalDateTime().compareTo(t.toLocalDateTime());let e=MathUtil.compareNumbers(this.toEpochSecond(),t.toEpochSecond());return 0===e&&(e=this.toLocalTime().nano()-t.toLocalTime().nano(),0===e&&(e=this.toLocalDateTime().compareTo(t.toLocalDateTime()))),e}isAfter(t){requireNonNull(t,"other");const e=this.toEpochSecond(),i=t.toEpochSecond();return e>i||e===i&&this.toLocalTime().nano()>t.toLocalTime().nano()}isBefore(t){requireNonNull(t,"other");const e=this.toEpochSecond(),i=t.toEpochSecond();return e<i||e===i&&this.toLocalTime().nano()<t.toLocalTime().nano()}isEqual(t){return requireNonNull(t,"other"),this.toEpochSecond()===t.toEpochSecond()&&this.toLocalTime().nano()===t.toLocalTime().nano()}equals(t){return this===t||t instanceof OffsetDateTime&&(this._dateTime.equals(t._dateTime)&&this._offset.equals(t._offset))}hashCode(){return this._dateTime.hashCode()^this._offset.hashCode()}toString(){return this._dateTime.toString()+this._offset.toString()}toJSON(){return this.toString()}format(t){return requireNonNull(t,"formatter"),t.format(this)}}export function _init(){OffsetDateTime.MIN=LocalDateTime.MIN.atOffset(ZoneOffset.MAX),OffsetDateTime.MAX=LocalDateTime.MAX.atOffset(ZoneOffset.MIN),OffsetDateTime.FROM=createTemporalQuery("OffsetDateTime.FROM",(t=>OffsetDateTime.from(t)))}