/**
 * @copyright (c) 2016-present, Philipp Thürwächter & Pattrick Hüper  & js-joda contributors
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{Clock}from"./Clock";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Instant,LocalTime}from"./js-joda";import{MathUtil}from"./MathUtil";import{OffsetDateTime}from"./OffsetDateTime";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{createTemporalQuery}from"./temporal/TemporalQuery";import{requireInstance,requireNonNull}from"./assert";export class OffsetTime extends Temporal{static from(t){if(requireNonNull(t,"temporal"),t instanceof OffsetTime)return t;if(t instanceof OffsetDateTime)return t.toOffsetTime();try{const e=LocalTime.from(t),i=ZoneOffset.from(t);return new OffsetTime(e,i)}catch(e){throw new DateTimeException(`Unable to obtain OffsetTime TemporalAccessor: ${t}, type ${null!=t.constructor?t.constructor.name:""}`)}}static now(t){return 0===arguments.length?OffsetTime._now(Clock.systemDefaultZone()):t instanceof Clock?OffsetTime._now(t):OffsetTime._now(Clock.system(t))}static _now(t){requireNonNull(t,"clock");const e=t.instant();return OffsetTime.ofInstant(e,t.zone().rules().offset(e))}static of(){return arguments.length<=2?OffsetTime.ofTimeAndOffset.apply(this,arguments):OffsetTime.ofNumbers.apply(this,arguments)}static ofNumbers(t,e,i,o,s){const r=LocalTime.of(t,e,i,o);return new OffsetTime(r,s)}static ofTimeAndOffset(t,e){return new OffsetTime(t,e)}static ofInstant(t,e){requireNonNull(t,"instant"),requireInstance(t,Instant,"instant"),requireNonNull(e,"zone"),requireInstance(e,ZoneId,"zone");const i=e.rules().offset(t);let o=t.epochSecond()%LocalTime.SECONDS_PER_DAY;o=(o+i.totalSeconds())%LocalTime.SECONDS_PER_DAY,o<0&&(o+=LocalTime.SECONDS_PER_DAY);const s=LocalTime.ofSecondOfDay(o,t.nano());return new OffsetTime(s,i)}static parse(t,e=DateTimeFormatter.ISO_OFFSET_TIME){return requireNonNull(e,"formatter"),e.parse(t,OffsetTime.FROM)}constructor(t,e){super(),requireNonNull(t,"time"),requireInstance(t,LocalTime,"time"),requireNonNull(e,"offset"),requireInstance(e,ZoneOffset,"offset"),this._time=t,this._offset=e}adjustInto(t){return t.with(ChronoField.NANO_OF_DAY,this._time.toNanoOfDay()).with(ChronoField.OFFSET_SECONDS,this.offset().totalSeconds())}atDate(t){return OffsetDateTime.of(t,this._time,this._offset)}format(t){return requireNonNull(t,"formatter"),t.format(this,OffsetTime.FROM)}get(t){return super.get(t)}getLong(t){return t instanceof ChronoField?t===ChronoField.OFFSET_SECONDS?this._offset.totalSeconds():this._time.getLong(t):t.getFrom(this)}hour(){return this._time.hour()}minute(){return this._time.minute()}second(){return this._time.second()}nano(){return this._time.nano()}offset(){return this._offset}isAfter(t){return requireNonNull(t,"other"),this._toEpochNano()>t._toEpochNano()}isBefore(t){return requireNonNull(t,"other"),this._toEpochNano()<t._toEpochNano()}isEqual(t){return requireNonNull(t,"other"),this._toEpochNano()===t._toEpochNano()}isSupported(t){return t instanceof ChronoField?t.isTimeBased()||t===ChronoField.OFFSET_SECONDS:t instanceof ChronoUnit?t.isTimeBased():null!=t&&t.isSupportedBy(this)}minusHours(t){return this._withLocalTimeOffset(this._time.minusHours(t),this._offset)}minusMinutes(t){return this._withLocalTimeOffset(this._time.minusMinutes(t),this._offset)}minusSeconds(t){return this._withLocalTimeOffset(this._time.minusSeconds(t),this._offset)}minusNanos(t){return this._withLocalTimeOffset(this._time.minusNanos(t),this._offset)}_minusAmount(t){return requireNonNull(t),t.subtractFrom(this)}_minusUnit(t,e){return this.plus(-1*t,e)}_plusAmount(t){return requireNonNull(t),t.addTo(this)}_plusUnit(t,e){return e instanceof ChronoUnit?this._withLocalTimeOffset(this._time.plus(t,e),this._offset):e.addTo(this,t)}plusHours(t){return this._withLocalTimeOffset(this._time.plusHours(t),this._offset)}plusMinutes(t){return this._withLocalTimeOffset(this._time.plusMinutes(t),this._offset)}plusSeconds(t){return this._withLocalTimeOffset(this._time.plusSeconds(t),this._offset)}plusNanos(t){return this._withLocalTimeOffset(this._time.plusNanos(t),this._offset)}query(t){return requireNonNull(t,"query"),t===TemporalQueries.precision()?ChronoUnit.NANOS:t===TemporalQueries.offset()||t===TemporalQueries.zone()?this.offset():t===TemporalQueries.localTime()?this._time:t===TemporalQueries.chronology()||t===TemporalQueries.localDate()||t===TemporalQueries.zoneId()?null:super.query(t)}range(t){return t instanceof ChronoField?t===ChronoField.OFFSET_SECONDS?t.range():this._time.range(t):t.rangeRefinedBy(this)}toLocalTime(){return this._time}truncatedTo(t){return this._withLocalTimeOffset(this._time.truncatedTo(t),this._offset)}until(t,e){requireNonNull(t,"endExclusive"),requireNonNull(e,"unit");const i=OffsetTime.from(t);if(e instanceof ChronoUnit){const t=i._toEpochNano()-this._toEpochNano();switch(e){case ChronoUnit.NANOS:return t;case ChronoUnit.MICROS:return MathUtil.intDiv(t,1e3);case ChronoUnit.MILLIS:return MathUtil.intDiv(t,1e6);case ChronoUnit.SECONDS:return MathUtil.intDiv(t,LocalTime.NANOS_PER_SECOND);case ChronoUnit.MINUTES:return MathUtil.intDiv(t,LocalTime.NANOS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(t,LocalTime.NANOS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(t,12*LocalTime.NANOS_PER_HOUR)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${e}`)}return e.between(this,i)}withHour(t){return this._withLocalTimeOffset(this._time.withHour(t),this._offset)}withMinute(t){return this._withLocalTimeOffset(this._time.withMinute(t),this._offset)}withSecond(t){return this._withLocalTimeOffset(this._time.withSecond(t),this._offset)}withNano(t){return this._withLocalTimeOffset(this._time.withNano(t),this._offset)}withOffsetSameInstant(t){if(requireNonNull(t,"offset"),t.equals(this._offset))return this;const e=t.totalSeconds()-this._offset.totalSeconds(),i=this._time.plusSeconds(e);return new OffsetTime(i,t)}withOffsetSameLocal(t){return null!=t&&t.equals(this._offset)?this:new OffsetTime(this._time,t)}_toEpochNano(){return this._time.toNanoOfDay()-this._offset.totalSeconds()*LocalTime.NANOS_PER_SECOND}_withAdjuster(t){return requireNonNull(t,"adjuster"),t instanceof LocalTime?this._withLocalTimeOffset(t,this._offset):t instanceof ZoneOffset?this._withLocalTimeOffset(this._time,t):t instanceof OffsetTime?t:t.adjustInto(this)}_withField(t,e){return requireNonNull(t,"field"),t instanceof ChronoField?t===ChronoField.OFFSET_SECONDS?this._withLocalTimeOffset(this._time,ZoneOffset.ofTotalSeconds(t.checkValidIntValue(e))):this._withLocalTimeOffset(this._time.with(t,e),this._offset):t.adjustInto(this,e)}_withLocalTimeOffset(t,e){return this._time===t&&this._offset.equals(e)?this:new OffsetTime(t,e)}compareTo(t){if(requireNonNull(t,"other"),requireInstance(t,OffsetTime,"other"),this._offset.equals(t._offset))return this._time.compareTo(t._time);const e=MathUtil.compareNumbers(this._toEpochNano(),t._toEpochNano());return 0===e?this._time.compareTo(t._time):e}equals(t){return this===t||t instanceof OffsetTime&&(this._time.equals(t._time)&&this._offset.equals(t._offset))}hashCode(){return this._time.hashCode()^this._offset.hashCode()}toString(){return this._time.toString()+this._offset.toString()}toJSON(){return this.toString()}}export function _init(){OffsetTime.MIN=OffsetTime.ofNumbers(0,0,0,0,ZoneOffset.MAX),OffsetTime.MAX=OffsetTime.ofNumbers(23,59,59,999999999,ZoneOffset.MIN),OffsetTime.FROM=createTemporalQuery("OffsetTime.FROM",(t=>OffsetTime.from(t)))}