/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException,ArithmeticException,DateTimeParseException}from"./errors";import{IsoChronology}from"./chrono/IsoChronology";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalAmount}from"./temporal/TemporalAmount";import{LocalDate}from"./LocalDate";const PATTERN=/([-+]?)P(?:([-+]?[0-9]+)Y)?(?:([-+]?[0-9]+)M)?(?:([-+]?[0-9]+)W)?(?:([-+]?[0-9]+)D)?/;export class Period extends TemporalAmount{constructor(t,s,e){super();const r=MathUtil.safeToInt(t),i=MathUtil.safeToInt(s),o=MathUtil.safeToInt(e);if(0===r&&0===i&&0===o)return Period.ZERO||(this._years=r,this._months=i,this._days=o,Period.ZERO=this),Period.ZERO;this._years=r,this._months=i,this._days=o}static ofYears(t){return Period.create(t,0,0)}static ofMonths(t){return Period.create(0,t,0)}static ofWeeks(t){return Period.create(0,0,MathUtil.safeMultiply(t,7))}static ofDays(t){return Period.create(0,0,t)}static of(t,s,e){return Period.create(t,s,e)}static from(t){if(t instanceof Period)return t;requireNonNull(t,"amount");let s=0,e=0,r=0;const i=t.units();for(let o=0;o<i.length;o++){const n=i[o],a=t.get(n);if(n===ChronoUnit.YEARS)s=MathUtil.safeToInt(a);else if(n===ChronoUnit.MONTHS)e=MathUtil.safeToInt(a);else{if(n!==ChronoUnit.DAYS)throw new DateTimeException(`Unit must be Years, Months or Days, but was ${n}`);r=MathUtil.safeToInt(a)}}return Period.create(s,e,r)}static between(t,s){return requireNonNull(t,"startDate"),requireNonNull(s,"endDate"),requireInstance(t,LocalDate,"startDate"),requireInstance(s,LocalDate,"endDate"),t.until(s)}static parse(t){requireNonNull(t,"text");try{return Period._parse(t)}catch(s){throw s instanceof ArithmeticException?new DateTimeParseException("Text cannot be parsed to a Period",t,0,s):s}}static _parse(t){const s=PATTERN.exec(t);if(null!=s){const e="-"===s[1]?-1:1,r=s[2],i=s[3],o=s[4],n=s[5];if(null!=r||null!=i||null!=o||null!=n){const s=Period._parseNumber(t,r,e),a=Period._parseNumber(t,i,e),h=Period._parseNumber(t,o,e);let u=Period._parseNumber(t,n,e);return u=MathUtil.safeAdd(u,MathUtil.safeMultiply(h,7)),Period.create(s,a,u)}}throw new DateTimeParseException("Text cannot be parsed to a Period",t,0)}static _parseNumber(t,s,e){if(null==s)return 0;const r=MathUtil.parseInt(s);return MathUtil.safeMultiply(r,e)}static create(t,s,e){return new Period(t,s,e)}units(){return[ChronoUnit.YEARS,ChronoUnit.MONTHS,ChronoUnit.DAYS]}chronology(){return IsoChronology.INSTANCE}get(t){if(t===ChronoUnit.YEARS)return this._years;if(t===ChronoUnit.MONTHS)return this._months;if(t===ChronoUnit.DAYS)return this._days;throw new UnsupportedTemporalTypeException(`Unsupported unit: ${t}`)}isZero(){return this===Period.ZERO}isNegative(){return this._years<0||this._months<0||this._days<0}years(){return this._years}months(){return this._months}days(){return this._days}withYears(t){return t===this._years?this:Period.create(t,this._months,this._days)}withMonths(t){return t===this._months?this:Period.create(this._years,t,this._days)}withDays(t){return t===this._days?this:Period.create(this._years,this._months,t)}plus(t){const s=Period.from(t);return Period.create(MathUtil.safeAdd(this._years,s._years),MathUtil.safeAdd(this._months,s._months),MathUtil.safeAdd(this._days,s._days))}plusYears(t){return 0===t?this:Period.create(MathUtil.safeToInt(MathUtil.safeAdd(this._years,t)),this._months,this._days)}plusMonths(t){return 0===t?this:Period.create(this._years,MathUtil.safeToInt(MathUtil.safeAdd(this._months,t)),this._days)}plusDays(t){return 0===t?this:Period.create(this._years,this._months,MathUtil.safeToInt(MathUtil.safeAdd(this._days,t)))}minus(t){const s=Period.from(t);return Period.create(MathUtil.safeSubtract(this._years,s._years),MathUtil.safeSubtract(this._months,s._months),MathUtil.safeSubtract(this._days,s._days))}minusYears(t){return this.plusYears(-1*t)}minusMonths(t){return this.plusMonths(-1*t)}minusDays(t){return this.plusDays(-1*t)}multipliedBy(t){return this===Period.ZERO||1===t?this:Period.create(MathUtil.safeMultiply(this._years,t),MathUtil.safeMultiply(this._months,t),MathUtil.safeMultiply(this._days,t))}negated(){return this.multipliedBy(-1)}normalized(){const t=this.toTotalMonths(),s=MathUtil.intDiv(t,12),e=MathUtil.intMod(t,12);return s===this._years&&e===this._months?this:Period.create(MathUtil.safeToInt(s),e,this._days)}toTotalMonths(){return 12*this._years+this._months}addTo(t){return requireNonNull(t,"temporal"),0!==this._years?t=0!==this._months?t.plus(this.toTotalMonths(),ChronoUnit.MONTHS):t.plus(this._years,ChronoUnit.YEARS):0!==this._months&&(t=t.plus(this._months,ChronoUnit.MONTHS)),0!==this._days&&(t=t.plus(this._days,ChronoUnit.DAYS)),t}subtractFrom(t){return requireNonNull(t,"temporal"),0!==this._years?t=0!==this._months?t.minus(this.toTotalMonths(),ChronoUnit.MONTHS):t.minus(this._years,ChronoUnit.YEARS):0!==this._months&&(t=t.minus(this._months,ChronoUnit.MONTHS)),0!==this._days&&(t=t.minus(this._days,ChronoUnit.DAYS)),t}equals(t){if(this===t)return!0;if(t instanceof Period){const s=t;return this._years===s._years&&this._months===s._months&&this._days===s._days}return!1}hashCode(){return MathUtil.hashCode(this._years,this._months,this._days)}toString(){if(this===Period.ZERO)return"P0D";{let t="P";return 0!==this._years&&(t+=`${this._years}Y`),0!==this._months&&(t+=`${this._months}M`),0!==this._days&&(t+=`${this._days}D`),t}}toJSON(){return this.toString()}}export function _init(){Period.ofDays(0)}