/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{requireNonNull,requireInstance}from"./assert";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{MonthDay}from"./MonthDay";import{SignStyle}from"./format/SignStyle";import{Temporal}from"./temporal/Temporal";import{TemporalAccessor}from"./temporal/TemporalAccessor";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalQuery,createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalUnit}from"./temporal/TemporalUnit";import{YearConstants}from"./YearConstants";import{YearMonth}from"./YearMonth";import{ZoneId}from"./ZoneId";export class Year extends Temporal{constructor(e){super(),this._year=MathUtil.safeToInt(e)}value(){return this._year}static now(e=void 0){return void 0===e?Year.now0():e instanceof ZoneId?Year.nowZoneId(e):Year.nowClock(e)}static now0(){return Year.nowClock(Clock.systemDefaultZone())}static nowZoneId(e){return requireNonNull(e,"zone"),requireInstance(e,ZoneId,"zone"),Year.nowClock(Clock.system(e))}static nowClock(e){requireNonNull(e,"clock"),requireInstance(e,Clock,"clock");const r=LocalDate.now(e);return Year.of(r.year())}static of(e){return requireNonNull(e,"isoYear"),ChronoField.YEAR.checkValidValue(e),new Year(e)}static from(e){if(requireNonNull(e,"temporal"),requireInstance(e,TemporalAccessor,"temporal"),e instanceof Year)return e;try{return Year.of(e.get(ChronoField.YEAR))}catch(r){throw new DateTimeException(`Unable to obtain Year from TemporalAccessor: ${e}, type ${e&&null!=e.constructor?e.constructor.name:""}`)}}static parse(e,r){return arguments.length<=1?Year.parseText(e):Year.parseTextFormatter(e,r)}static parseText(e){return requireNonNull(e,"text"),Year.parse(e,PARSER)}static parseTextFormatter(e,r=PARSER){return requireNonNull(e,"text"),requireNonNull(r,"formatter"),requireInstance(r,DateTimeFormatter,"formatter"),r.parse(e,Year.FROM)}static isLeap(e){return 0===MathUtil.intMod(e,4)&&(0!==MathUtil.intMod(e,100)||0===MathUtil.intMod(e,400))}isSupported(e){return 1===arguments.length&&e instanceof TemporalField?this.isSupportedField(e):this.isSupportedUnit(e)}isSupportedField(e){return e instanceof ChronoField?e===ChronoField.YEAR||e===ChronoField.YEAR_OF_ERA||e===ChronoField.ERA:null!=e&&e.isSupportedBy(this)}isSupportedUnit(e){return e instanceof ChronoUnit?e===ChronoUnit.YEARS||e===ChronoUnit.DECADES||e===ChronoUnit.CENTURIES||e===ChronoUnit.MILLENNIA||e===ChronoUnit.ERAS:null!=e&&e.isSupportedBy(this)}range(e){if(this.isSupported(e))return e.range();if(e instanceof ChronoField)throw new UnsupportedTemporalTypeException(`Unsupported field: ${e}`);return super.range(e)}get(e){return this.range(e).checkValidIntValue(this.getLong(e),e)}getLong(e){if(requireNonNull(e,"field"),e instanceof ChronoField){switch(e){case ChronoField.YEAR_OF_ERA:return this._year<1?1-this._year:this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year<1?0:1}throw new UnsupportedTemporalTypeException(`Unsupported field: ${e}`)}return e.getFrom(this)}isLeap(){return Year.isLeap(this._year)}_withField(e,r){if(requireNonNull(e,"field"),requireInstance(e,TemporalField,"field"),e instanceof ChronoField){switch(e.checkValidValue(r),e){case ChronoField.YEAR_OF_ERA:return Year.of(this._year<1?1-r:r);case ChronoField.YEAR:return Year.of(r);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===r?this:Year.of(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${e}`)}return e.adjustInto(this,r)}_plusUnit(e,r){if(requireNonNull(e,"amountToAdd"),requireNonNull(r,"unit"),requireInstance(r,TemporalUnit,"unit"),r instanceof ChronoUnit){switch(r){case ChronoUnit.YEARS:return this.plusYears(e);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(e,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(e,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(e,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),e))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${r}`)}return r.addTo(this,e)}plusYears(e){return 0===e?this:Year.of(ChronoField.YEAR.checkValidIntValue(MathUtil.safeAdd(this._year,e)))}minusYears(e){return e===MathUtil.MIN_SAFE_INTEGER?this.plusYears(MathUtil.MAX_SAFE_INTEGER).plusYears(1):this.plusYears(-e)}adjustInto(e){return requireNonNull(e,"temporal"),e.with(ChronoField.YEAR,this._year)}isValidMonthDay(e){return null!=e&&e.isValidYear(this._year)}length(){return this.isLeap()?366:365}atDay(e){return LocalDate.ofYearDay(this._year,e)}atMonth(e){return 1===arguments.length&&e instanceof Month?this.atMonthMonth(e):this.atMonthNumber(e)}atMonthMonth(e){return requireNonNull(e,"month"),requireInstance(e,Month,"month"),YearMonth.of(this._year,e)}atMonthNumber(e){return requireNonNull(e,"month"),YearMonth.of(this._year,e)}atMonthDay(e){return requireNonNull(e,"monthDay"),requireInstance(e,MonthDay,"monthDay"),e.atYear(this._year)}query(e){return requireNonNull(e,"query()"),requireInstance(e,TemporalQuery,"query()"),e===TemporalQueries.chronology()?IsoChronology.INSTANCE:e===TemporalQueries.precision()?ChronoUnit.YEARS:e===TemporalQueries.localDate()||e===TemporalQueries.localTime()||e===TemporalQueries.zone()||e===TemporalQueries.zoneId()||e===TemporalQueries.offset()?null:super.query(e)}compareTo(e){return requireNonNull(e,"other"),requireInstance(e,Year,"other"),this._year-e._year}isAfter(e){return requireNonNull(e,"other"),requireInstance(e,Year,"other"),this._year>e._year}isBefore(e){return requireNonNull(e,"other"),requireInstance(e,Year,"other"),this._year<e._year}format(e){return requireNonNull(e,"formatter"),requireInstance(e,DateTimeFormatter,"formatter"),e.format(this)}equals(e){return this===e||e instanceof Year&&this.value()===e.value()}toString(){return`${this._year}`}toJSON(){return this.toString()}until(e,r){const t=Year.from(e);if(r instanceof ChronoUnit){const e=t.value()-this.value();switch(r){case ChronoUnit.YEARS:return e;case ChronoUnit.DECADES:return MathUtil.intDiv(e,10);case ChronoUnit.CENTURIES:return MathUtil.intDiv(e,100);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(e,1e3);case ChronoUnit.ERAS:return t.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${r}`)}return r.between(this,t)}}let PARSER;export function _init(){Year.MIN_VALUE=YearConstants.MIN_VALUE,Year.MAX_VALUE=YearConstants.MAX_VALUE,PARSER=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).toFormatter(),Year.FROM=createTemporalQuery("Year.FROM",(e=>Year.from(e)))}