/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @license BSD-3-Clause (see LICENSE.md in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Clock}from"./Clock";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{SignStyle}from"./format/SignStyle";import{Temporal}from"./temporal/Temporal";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalQuery}from"./temporal/TemporalQuery";import{TemporalUnit}from"./temporal/TemporalUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ValueRange}from"./temporal/ValueRange";import{Year}from"./Year";import{ZoneId}from"./ZoneId";export class YearMonth extends Temporal{static now(e){return 0===arguments.length?YearMonth.now0():1===arguments.length&&e instanceof ZoneId?YearMonth.nowZoneId(e):YearMonth.nowClock(e)}static now0(){return YearMonth.nowClock(Clock.systemDefaultZone())}static nowZoneId(e){return YearMonth.nowClock(Clock.system(e))}static nowClock(e){const r=LocalDate.now(e);return YearMonth.of(r.year(),r.month())}static of(e,r){return 2===arguments.length&&r instanceof Month?YearMonth.ofNumberMonth(e,r):YearMonth.ofNumberNumber(e,r)}static ofNumberMonth(e,r){return requireNonNull(r,"month"),requireInstance(r,Month,"month"),YearMonth.ofNumberNumber(e,r.value())}static ofNumberNumber(e,r){return requireNonNull(e,"year"),requireNonNull(r,"month"),ChronoField.YEAR.checkValidValue(e),ChronoField.MONTH_OF_YEAR.checkValidValue(r),new YearMonth(e,r)}static from(e){if(requireNonNull(e,"temporal"),e instanceof YearMonth)return e;try{return YearMonth.of(e.get(ChronoField.YEAR),e.get(ChronoField.MONTH_OF_YEAR))}catch(r){throw new DateTimeException(`Unable to obtain YearMonth from TemporalAccessor: ${e}, type ${e&&null!=e.constructor?e.constructor.name:""}`)}}static parse(e,r){return 1===arguments.length?YearMonth.parseString(e):YearMonth.parseStringFormatter(e,r)}static parseString(e){return YearMonth.parseStringFormatter(e,PARSER)}static parseStringFormatter(e,r){return requireNonNull(r,"formatter"),r.parse(e,YearMonth.FROM)}constructor(e,r){super(),this._year=MathUtil.safeToInt(e),this._month=MathUtil.safeToInt(r)}isSupported(e){return 1===arguments.length&&e instanceof TemporalField?this.isSupportedField(e):this.isSupportedUnit(e)}isSupportedField(e){return e instanceof ChronoField?e===ChronoField.YEAR||e===ChronoField.MONTH_OF_YEAR||e===ChronoField.PROLEPTIC_MONTH||e===ChronoField.YEAR_OF_ERA||e===ChronoField.ERA:null!=e&&e.isSupportedBy(this)}isSupportedUnit(e){return e instanceof ChronoUnit?e===ChronoUnit.MONTHS||e===ChronoUnit.YEARS||e===ChronoUnit.DECADES||e===ChronoUnit.CENTURIES||e===ChronoUnit.MILLENNIA||e===ChronoUnit.ERAS:null!=e&&e.isSupportedBy(this)}range(e){return e===ChronoField.YEAR_OF_ERA?this.year()<=0?ValueRange.of(1,Year.MAX_VALUE+1):ValueRange.of(1,Year.MAX_VALUE):super.range(e)}get(e){return requireNonNull(e,"field"),requireInstance(e,TemporalField,"field"),this.range(e).checkValidIntValue(this.getLong(e),e)}getLong(e){if(requireNonNull(e,"field"),requireInstance(e,TemporalField,"field"),e instanceof ChronoField){switch(e){case ChronoField.MONTH_OF_YEAR:return this._month;case ChronoField.PROLEPTIC_MONTH:return this._getProlepticMonth();case ChronoField.YEAR_OF_ERA:return this._year<1?1-this._year:this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year<1?0:1}throw new UnsupportedTemporalTypeException(`Unsupported field: ${e}`)}return e.getFrom(this)}_getProlepticMonth(){return MathUtil.safeAdd(MathUtil.safeMultiply(this._year,12),this._month-1)}year(){return this._year}monthValue(){return this._month}month(){return Month.of(this._month)}isLeapYear(){return IsoChronology.isLeapYear(this._year)}isValidDay(e){return e>=1&&e<=this.lengthOfMonth()}lengthOfMonth(){return this.month().length(this.isLeapYear())}lengthOfYear(){return this.isLeapYear()?366:365}with(e,r){return 1===arguments.length?this._withAdjuster(e):this._withField(e,r)}_withField(e,r){if(requireNonNull(e,"field"),requireInstance(e,TemporalField,"field"),e instanceof ChronoField){const t=e;switch(t.checkValidValue(r),t){case ChronoField.MONTH_OF_YEAR:return this.withMonth(r);case ChronoField.PROLEPTIC_MONTH:return this.plusMonths(r-this.getLong(ChronoField.PROLEPTIC_MONTH));case ChronoField.YEAR_OF_ERA:return this.withYear(this._year<1?1-r:r);case ChronoField.YEAR:return this.withYear(r);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===r?this:this.withYear(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${e}`)}return e.adjustInto(this,r)}withYear(e){return ChronoField.YEAR.checkValidValue(e),new YearMonth(e,this._month)}withMonth(e){return ChronoField.MONTH_OF_YEAR.checkValidValue(e),new YearMonth(this._year,e)}_plusUnit(e,r){if(requireNonNull(r,"unit"),requireInstance(r,TemporalUnit,"unit"),r instanceof ChronoUnit){switch(r){case ChronoUnit.MONTHS:return this.plusMonths(e);case ChronoUnit.YEARS:return this.plusYears(e);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(e,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(e,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(e,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),e))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${r}`)}return r.addTo(this,e)}plusYears(e){if(0===e)return this;const r=ChronoField.YEAR.checkValidIntValue(this._year+e);return this.withYear(r)}plusMonths(e){if(0===e)return this;const r=12*this._year+(this._month-1)+e,t=ChronoField.YEAR.checkValidIntValue(MathUtil.floorDiv(r,12)),o=MathUtil.floorMod(r,12)+1;return new YearMonth(t,o)}minusYears(e){return e===MathUtil.MIN_SAFE_INTEGER?this.plusYears(MathUtil.MIN_SAFE_INTEGER).plusYears(1):this.plusYears(-e)}minusMonths(e){return e===MathUtil.MIN_SAFE_INTEGER?this.plusMonths(Math.MAX_SAFE_INTEGER).plusMonths(1):this.plusMonths(-e)}query(e){return requireNonNull(e,"query"),requireInstance(e,TemporalQuery,"query"),e===TemporalQueries.chronology()?IsoChronology.INSTANCE:e===TemporalQueries.precision()?ChronoUnit.MONTHS:e===TemporalQueries.localDate()||e===TemporalQueries.localTime()||e===TemporalQueries.zone()||e===TemporalQueries.zoneId()||e===TemporalQueries.offset()?null:super.query(e)}adjustInto(e){return requireNonNull(e,"temporal"),requireInstance(e,Temporal,"temporal"),e.with(ChronoField.PROLEPTIC_MONTH,this._getProlepticMonth())}until(e,r){requireNonNull(e,"endExclusive"),requireNonNull(r,"unit"),requireInstance(e,Temporal,"endExclusive"),requireInstance(r,TemporalUnit,"unit");const t=YearMonth.from(e);if(r instanceof ChronoUnit){const e=t._getProlepticMonth()-this._getProlepticMonth();switch(r){case ChronoUnit.MONTHS:return e;case ChronoUnit.YEARS:return MathUtil.intDiv(e,12);case ChronoUnit.DECADES:return MathUtil.intDiv(e,120);case ChronoUnit.CENTURIES:return MathUtil.intDiv(e,1200);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(e,12e3);case ChronoUnit.ERAS:return t.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${r}`)}return r.between(this,t)}atDay(e){return requireNonNull(e,"dayOfMonth"),LocalDate.of(this._year,this._month,e)}atEndOfMonth(){return LocalDate.of(this._year,this._month,this.lengthOfMonth())}compareTo(e){requireNonNull(e,"other"),requireInstance(e,YearMonth,"other");let r=this._year-e.year();return 0===r&&(r=this._month-e.monthValue()),r}isAfter(e){return this.compareTo(e)>0}isBefore(e){return this.compareTo(e)<0}equals(e){if(this===e)return!0;if(e instanceof YearMonth){const r=e;return this.year()===r.year()&&this.monthValue()===r.monthValue()}return!1}toString(){return PARSER.format(this)}toJSON(){return this.toString()}format(e){return requireNonNull(e,"formatter"),e.format(this)}}let PARSER;export function _init(){PARSER=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendLiteral("-").appendValue(ChronoField.MONTH_OF_YEAR,2).toFormatter(),YearMonth.FROM=createTemporalQuery("YearMonth.FROM",(e=>YearMonth.from(e)))}