/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"./assert";import{DateTimeException}from"./errors";import{MathUtil}from"./MathUtil";import{LocalTime}from"./LocalTime";import{ZoneId}from"./ZoneId";import{ChronoField}from"./temporal/ChronoField";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZoneRules}from"./zone/ZoneRules";const SECONDS_CACHE={},ID_CACHE={};export class ZoneOffset extends ZoneId{constructor(e){super(),ZoneOffset._validateTotalSeconds(e),this._totalSeconds=MathUtil.safeToInt(e),this._rules=ZoneRules.of(this),this._id=ZoneOffset._buildId(e)}totalSeconds(){return this._totalSeconds}id(){return this._id}static _buildId(e){if(0===e)return"Z";{const t=Math.abs(e),o=MathUtil.intDiv(t,LocalTime.SECONDS_PER_HOUR),n=MathUtil.intMod(MathUtil.intDiv(t,LocalTime.SECONDS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR);let s=`${e<0?"-":"+"}${o<10?"0":""}${o}${n<10?":0":":"}${n}`;const i=MathUtil.intMod(t,LocalTime.SECONDS_PER_MINUTE);return 0!==i&&(s+=(i<10?":0":":")+i),s}}static _validateTotalSeconds(e){if(Math.abs(e)>ZoneOffset.MAX_SECONDS)throw new DateTimeException("Zone offset not in valid range: -18:00 to +18:00")}static _validate(e,t,o){if(e<-18||e>18)throw new DateTimeException(`Zone offset hours not in valid range: value ${e} is not in the range -18 to 18`);if(e>0){if(t<0||o<0)throw new DateTimeException("Zone offset minutes and seconds must be positive because hours is positive")}else if(e<0){if(t>0||o>0)throw new DateTimeException("Zone offset minutes and seconds must be negative because hours is negative")}else if(t>0&&o<0||t<0&&o>0)throw new DateTimeException("Zone offset minutes and seconds must have the same sign");if(Math.abs(t)>59)throw new DateTimeException(`Zone offset minutes not in valid range: abs(value) ${Math.abs(t)} is not in the range 0 to 59`);if(Math.abs(o)>59)throw new DateTimeException(`Zone offset seconds not in valid range: abs(value) ${Math.abs(o)} is not in the range 0 to 59`);if(18===Math.abs(e)&&(Math.abs(t)>0||Math.abs(o)>0))throw new DateTimeException("Zone offset not in valid range: -18:00 to +18:00")}static of(e){requireNonNull(e,"offsetId");const t=ID_CACHE[e];if(null!=t)return t;let o,n,s;switch(e.length){case 2:e=`${e[0]}0${e[1]}`;case 3:o=ZoneOffset._parseNumber(e,1,!1),n=0,s=0;break;case 5:o=ZoneOffset._parseNumber(e,1,!1),n=ZoneOffset._parseNumber(e,3,!1),s=0;break;case 6:o=ZoneOffset._parseNumber(e,1,!1),n=ZoneOffset._parseNumber(e,4,!0),s=0;break;case 7:o=ZoneOffset._parseNumber(e,1,!1),n=ZoneOffset._parseNumber(e,3,!1),s=ZoneOffset._parseNumber(e,5,!1);break;case 9:o=ZoneOffset._parseNumber(e,1,!1),n=ZoneOffset._parseNumber(e,4,!0),s=ZoneOffset._parseNumber(e,7,!0);break;default:throw new DateTimeException(`Invalid ID for ZoneOffset, invalid format: ${e}`)}const i=e[0];if("+"!==i&&"-"!==i)throw new DateTimeException(`Invalid ID for ZoneOffset, plus/minus not found when expected: ${e}`);return"-"===i?ZoneOffset.ofHoursMinutesSeconds(-o,-n,-s):ZoneOffset.ofHoursMinutesSeconds(o,n,s)}static _parseNumber(e,t,o){if(o&&":"!==e[t-1])throw new DateTimeException(`Invalid ID for ZoneOffset, colon not found when expected: ${e}`);const n=e[t],s=e[t+1];if(n<"0"||n>"9"||s<"0"||s>"9")throw new DateTimeException(`Invalid ID for ZoneOffset, non numeric characters found: ${e}`);return 10*(n.charCodeAt(0)-48)+(s.charCodeAt(0)-48)}static ofHours(e){return ZoneOffset.ofHoursMinutesSeconds(e,0,0)}static ofHoursMinutes(e,t){return ZoneOffset.ofHoursMinutesSeconds(e,t,0)}static ofHoursMinutesSeconds(e,t,o){ZoneOffset._validate(e,t,o);const n=e*LocalTime.SECONDS_PER_HOUR+t*LocalTime.SECONDS_PER_MINUTE+o;return ZoneOffset.ofTotalSeconds(n)}static ofTotalMinutes(e){const t=e*LocalTime.SECONDS_PER_MINUTE;return ZoneOffset.ofTotalSeconds(t)}static ofTotalSeconds(e){if(e%(15*LocalTime.SECONDS_PER_MINUTE)==0){const t=e;let o=SECONDS_CACHE[t];return null==o&&(o=new ZoneOffset(e),SECONDS_CACHE[t]=o,ID_CACHE[o.id()]=o),o}return new ZoneOffset(e)}rules(){return this._rules}get(e){return this.getLong(e)}getLong(e){if(e===ChronoField.OFFSET_SECONDS)return this._totalSeconds;if(e instanceof ChronoField)throw new DateTimeException(`Unsupported field: ${e}`);return e.getFrom(this)}query(e){return requireNonNull(e,"query"),e===TemporalQueries.offset()||e===TemporalQueries.zone()?this:e===TemporalQueries.localDate()||e===TemporalQueries.localTime()||e===TemporalQueries.precision()||e===TemporalQueries.chronology()||e===TemporalQueries.zoneId()?null:e.queryFrom(this)}adjustInto(e){return e.with(ChronoField.OFFSET_SECONDS,this._totalSeconds)}compareTo(e){return requireNonNull(e,"other"),e._totalSeconds-this._totalSeconds}equals(e){return this===e||e instanceof ZoneOffset&&this._totalSeconds===e._totalSeconds}hashCode(){return this._totalSeconds}toString(){return this._id}}export function _init(){ZoneOffset.MAX_SECONDS=18*LocalTime.SECONDS_PER_HOUR,ZoneOffset.UTC=ZoneOffset.ofTotalSeconds(0),ZoneOffset.MIN=ZoneOffset.ofTotalSeconds(-ZoneOffset.MAX_SECONDS),ZoneOffset.MAX=ZoneOffset.ofTotalSeconds(ZoneOffset.MAX_SECONDS)}