/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"./assert";import{DateTimeException,IllegalArgumentException}from"./errors";import{MathUtil}from"./MathUtil";import{Clock}from"./Clock";import{Instant}from"./Instant";import{LocalDate}from"./LocalDate";import{LocalDateTime}from"./LocalDateTime";import{LocalTime}from"./LocalTime";import{OffsetDateTime}from"./OffsetDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{ChronoZonedDateTime}from"./chrono/ChronoZonedDateTime";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalQueries}from"./temporal/TemporalQueries";export class ZonedDateTime extends ChronoZonedDateTime{static now(e){let t;return t=e instanceof ZoneId?Clock.system(e):null==e?Clock.systemDefaultZone():e,ZonedDateTime.ofInstant(t.instant(),t.zone())}static of(){return arguments.length<=2?ZonedDateTime.of2.apply(this,arguments):3===arguments.length&&arguments[0]instanceof LocalDate?ZonedDateTime.of3.apply(this,arguments):ZonedDateTime.of8.apply(this,arguments)}static of3(e,t,o){return ZonedDateTime.of2(LocalDateTime.of(e,t),o)}static of2(e,t){return ZonedDateTime.ofLocal(e,t,null)}static of8(e,t,o,i,n,s,r,a){const l=LocalDateTime.of(e,t,o,i,n,s,r);return ZonedDateTime.ofLocal(l,a,null)}static ofLocal(e,t,o){if(requireNonNull(e,"localDateTime"),requireNonNull(t,"zone"),t instanceof ZoneOffset)return new ZonedDateTime(e,t,t);let i=null;const n=t.rules(),s=n.validOffsets(e);if(1===s.length)i=s[0];else if(0===s.length){const t=n.transition(e);e=e.plusSeconds(t.duration().seconds()),i=t.offsetAfter()}else i=null!=o&&s.some((e=>e.equals(o)))?o:requireNonNull(s[0],"offset");return new ZonedDateTime(e,i,t)}static ofInstant(){return 2===arguments.length?ZonedDateTime.ofInstant2.apply(this,arguments):ZonedDateTime.ofInstant3.apply(this,arguments)}static ofInstant2(e,t){return requireNonNull(e,"instant"),requireNonNull(t,"zone"),ZonedDateTime._create(e.epochSecond(),e.nano(),t)}static ofInstant3(e,t,o){return requireNonNull(e,"localDateTime"),requireNonNull(t,"offset"),requireNonNull(o,"zone"),ZonedDateTime._create(e.toEpochSecond(t),e.nano(),o)}static _create(e,t,o){const i=o.rules(),n=Instant.ofEpochSecond(e,t),s=i.offset(n),r=LocalDateTime.ofEpochSecond(e,t,s);return new ZonedDateTime(r,s,o)}static ofStrict(e,t,o){requireNonNull(e,"localDateTime"),requireNonNull(t,"offset"),requireNonNull(o,"zone");const i=o.rules();if(!1===i.isValidOffset(e,t)){const n=i.transition(e);if(null!=n&&n.isGap())throw new DateTimeException(`LocalDateTime ${e} does not exist in zone ${o} due to a gap in the local time-line, typically caused by daylight savings`);throw new DateTimeException(`ZoneOffset "${t}" is not valid for LocalDateTime "${e}" in zone "${o}"`)}return new ZonedDateTime(e,t,o)}static ofLenient(e,t,o){if(requireNonNull(e,"localDateTime"),requireNonNull(t,"offset"),requireNonNull(o,"zone"),o instanceof ZoneOffset&&!1===t.equals(o))throw new IllegalArgumentException("ZoneId must match ZoneOffset");return new ZonedDateTime(e,t,o)}static from(e){if(requireNonNull(e,"temporal"),e instanceof ZonedDateTime)return e;const t=ZoneId.from(e);if(e.isSupported(ChronoField.INSTANT_SECONDS)){const o=ZonedDateTime._from(e,t);if(null!=o)return o}const o=LocalDateTime.from(e);return ZonedDateTime.of2(o,t)}static _from(e,t){try{return ZonedDateTime.__from(e,t)}catch(e){if(!(e instanceof DateTimeException))throw e}}static __from(e,t){const o=e.getLong(ChronoField.INSTANT_SECONDS),i=e.get(ChronoField.NANO_OF_SECOND);return ZonedDateTime._create(o,i,t)}static parse(e,t=DateTimeFormatter.ISO_ZONED_DATE_TIME){return requireNonNull(t,"formatter"),t.parse(e,ZonedDateTime.FROM)}constructor(e,t,o){requireNonNull(e,"dateTime"),requireNonNull(t,"offset"),requireNonNull(o,"zone"),super(),this._dateTime=e,this._offset=t,this._zone=o}_resolveLocal(e){return requireNonNull(e,"newDateTime"),ZonedDateTime.ofLocal(e,this._zone,this._offset)}_resolveInstant(e){return ZonedDateTime.ofInstant3(e,this._offset,this._zone)}_resolveOffset(e){return!1===e.equals(this._offset)&&this._zone.rules().isValidOffset(this._dateTime,e)?new ZonedDateTime(this._dateTime,e,this._zone):this}isSupported(e){return e instanceof ChronoField||(e instanceof ChronoUnit?e.isDateBased()||e.isTimeBased():null!=e&&e.isSupportedBy(this))}range(e){return e instanceof ChronoField?e===ChronoField.INSTANT_SECONDS||e===ChronoField.OFFSET_SECONDS?e.range():this._dateTime.range(e):e.rangeRefinedBy(this)}get(e){return this.getLong(e)}getLong(e){if(e instanceof ChronoField){switch(e){case ChronoField.INSTANT_SECONDS:return this.toEpochSecond();case ChronoField.OFFSET_SECONDS:return this._offset.totalSeconds()}return this._dateTime.getLong(e)}return requireNonNull(e,"field"),e.getFrom(this)}offset(){return this._offset}withEarlierOffsetAtOverlap(){const e=this._zone.rules().transition(this._dateTime);if(null!=e&&e.isOverlap()){const t=e.offsetBefore();if(!1===t.equals(this._offset))return new ZonedDateTime(this._dateTime,t,this._zone)}return this}withLaterOffsetAtOverlap(){const e=this._zone.rules().transition(this.toLocalDateTime());if(null!=e){const t=e.offsetAfter();if(!1===t.equals(this._offset))return new ZonedDateTime(this._dateTime,t,this._zone)}return this}zone(){return this._zone}withZoneSameLocal(e){return requireNonNull(e,"zone"),this._zone.equals(e)?this:ZonedDateTime.ofLocal(this._dateTime,e,this._offset)}withZoneSameInstant(e){return requireNonNull(e,"zone"),this._zone.equals(e)?this:ZonedDateTime._create(this._dateTime.toEpochSecond(this._offset),this._dateTime.nano(),e)}withFixedOffsetZone(){return this._zone.equals(this._offset)?this:new ZonedDateTime(this._dateTime,this._offset,this._offset)}year(){return this._dateTime.year()}monthValue(){return this._dateTime.monthValue()}month(){return this._dateTime.month()}dayOfMonth(){return this._dateTime.dayOfMonth()}dayOfYear(){return this._dateTime.dayOfYear()}dayOfWeek(){return this._dateTime.dayOfWeek()}hour(){return this._dateTime.hour()}minute(){return this._dateTime.minute()}second(){return this._dateTime.second()}nano(){return this._dateTime.nano()}_withAdjuster(e){if(e instanceof LocalDate)return this._resolveLocal(LocalDateTime.of(e,this._dateTime.toLocalTime()));if(e instanceof LocalTime)return this._resolveLocal(LocalDateTime.of(this._dateTime.toLocalDate(),e));if(e instanceof LocalDateTime)return this._resolveLocal(e);if(e instanceof Instant){const t=e;return ZonedDateTime._create(t.epochSecond(),t.nano(),this._zone)}return e instanceof ZoneOffset?this._resolveOffset(e):super._withAdjuster(e)}_withField(e,t){if(e instanceof ChronoField){switch(e){case ChronoField.INSTANT_SECONDS:return ZonedDateTime._create(t,this.nano(),this._zone);case ChronoField.OFFSET_SECONDS:{const o=ZoneOffset.ofTotalSeconds(e.checkValidIntValue(t));return this._resolveOffset(o)}}return this._resolveLocal(this._dateTime.with(e,t))}return e.adjustInto(this,t)}withYear(e){return this._resolveLocal(this._dateTime.withYear(e))}withMonth(e){return this._resolveLocal(this._dateTime.withMonth(e))}withDayOfMonth(e){return this._resolveLocal(this._dateTime.withDayOfMonth(e))}withDayOfYear(e){return this._resolveLocal(this._dateTime.withDayOfYear(e))}withHour(e){return this._resolveLocal(this._dateTime.withHour(e))}withMinute(e){return this._resolveLocal(this._dateTime.withMinute(e))}withSecond(e){return this._resolveLocal(this._dateTime.withSecond(e))}withNano(e){return this._resolveLocal(this._dateTime.withNano(e))}truncatedTo(e){return this._resolveLocal(this._dateTime.truncatedTo(e))}_plusUnit(e,t){return t instanceof ChronoUnit?t.isDateBased()?this._resolveLocal(this._dateTime.plus(e,t)):this._resolveInstant(this._dateTime.plus(e,t)):(requireNonNull(t,"unit"),t.addTo(this,e))}plusYears(e){return this._resolveLocal(this._dateTime.plusYears(e))}plusMonths(e){return this._resolveLocal(this._dateTime.plusMonths(e))}plusWeeks(e){return this._resolveLocal(this._dateTime.plusWeeks(e))}plusDays(e){return this._resolveLocal(this._dateTime.plusDays(e))}plusHours(e){return this._resolveInstant(this._dateTime.plusHours(e))}plusMinutes(e){return this._resolveInstant(this._dateTime.plusMinutes(e))}plusSeconds(e){return this._resolveInstant(this._dateTime.plusSeconds(e))}plusNanos(e){return this._resolveInstant(this._dateTime.plusNanos(e))}_minusUnit(e,t){return this._plusUnit(-1*e,t)}minusYears(e){return this.plusYears(-1*e)}minusMonths(e){return this.plusMonths(-1*e)}minusWeeks(e){return this.plusWeeks(-1*e)}minusDays(e){return this.plusDays(-1*e)}minusHours(e){return this.plusHours(-1*e)}minusMinutes(e){return this.plusMinutes(-1*e)}minusSeconds(e){return this.plusSeconds(-1*e)}minusNanos(e){return this.plusNanos(-1*e)}query(e){return e===TemporalQueries.localDate()?this.toLocalDate():(requireNonNull(e,"query"),super.query(e))}until(e,t){let o=ZonedDateTime.from(e);if(t instanceof ChronoUnit){if(o=o.withZoneSameInstant(this._zone),t.isDateBased())return this._dateTime.until(o._dateTime,t);{const e=this._offset.totalSeconds()-o._offset.totalSeconds(),i=o._dateTime.plusSeconds(e);return this._dateTime.until(i,t)}}return t.between(this,o)}toLocalDateTime(){return this._dateTime}toLocalDate(){return this._dateTime.toLocalDate()}toLocalTime(){return this._dateTime.toLocalTime()}toOffsetDateTime(){return OffsetDateTime.of(this._dateTime,this._offset)}equals(e){return this===e||e instanceof ZonedDateTime&&(this._dateTime.equals(e._dateTime)&&this._offset.equals(e._offset)&&this._zone.equals(e._zone))}hashCode(){return MathUtil.hashCode(this._dateTime.hashCode(),this._offset.hashCode(),this._zone.hashCode())}toString(){let e=this._dateTime.toString()+this._offset.toString();return this._offset!==this._zone&&(e+=`[${this._zone.toString()}]`),e}toJSON(){return this.toString()}format(e){return super.format(e)}}export function _init(){ZonedDateTime.FROM=createTemporalQuery("ZonedDateTime.FROM",(e=>ZonedDateTime.from(e)))}