/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{Enum}from"../Enum";import{requireNonNull}from"../assert";import{DateTimeException}from"../errors";import{MathUtil}from"../MathUtil";import{DayOfWeek}from"../DayOfWeek";import{LocalDate}from"../LocalDate";import{Month}from"../Month";import{Year}from"../Year";import{ChronoField}from"../temporal/ChronoField";import{ResolverStyle}from"../format/ResolverStyle";import{TemporalAdjusters}from"../temporal/TemporalAdjusters";export class IsoChronology extends Enum{static isLeapYear(e){return 0==(3&e)&&(e%100!=0||e%400==0)}_updateResolveMap(e,o,t){requireNonNull(e,"fieldValues"),requireNonNull(o,"field");const l=e.get(o);if(null!=l&&l!==t)throw new DateTimeException(`Invalid state, field: ${o} ${l} conflicts with ${o} ${t}`);e.put(o,t)}resolveDate(e,o){if(e.containsKey(ChronoField.EPOCH_DAY))return LocalDate.ofEpochDay(e.remove(ChronoField.EPOCH_DAY));const t=e.remove(ChronoField.PROLEPTIC_MONTH);null!=t&&(o!==ResolverStyle.LENIENT&&ChronoField.PROLEPTIC_MONTH.checkValidValue(t),this._updateResolveMap(e,ChronoField.MONTH_OF_YEAR,MathUtil.floorMod(t,12)+1),this._updateResolveMap(e,ChronoField.YEAR,MathUtil.floorDiv(t,12)));const l=e.remove(ChronoField.YEAR_OF_ERA);if(null!=l){o!==ResolverStyle.LENIENT&&ChronoField.YEAR_OF_ERA.checkValidValue(l);const t=e.remove(ChronoField.ERA);if(null==t){const t=e.get(ChronoField.YEAR);o===ResolverStyle.STRICT?null!=t?this._updateResolveMap(e,ChronoField.YEAR,t>0?l:MathUtil.safeSubtract(1,l)):e.put(ChronoField.YEAR_OF_ERA,l):this._updateResolveMap(e,ChronoField.YEAR,null==t||t>0?l:MathUtil.safeSubtract(1,l))}else if(1===t)this._updateResolveMap(e,ChronoField.YEAR,l);else{if(0!==t)throw new DateTimeException(`Invalid value for era: ${t}`);this._updateResolveMap(e,ChronoField.YEAR,MathUtil.safeSubtract(1,l))}}else e.containsKey(ChronoField.ERA)&&ChronoField.ERA.checkValidValue(e.get(ChronoField.ERA));if(e.containsKey(ChronoField.YEAR)){if(e.containsKey(ChronoField.MONTH_OF_YEAR)&&e.containsKey(ChronoField.DAY_OF_MONTH)){const t=ChronoField.YEAR.checkValidIntValue(e.remove(ChronoField.YEAR)),l=e.remove(ChronoField.MONTH_OF_YEAR);let r=e.remove(ChronoField.DAY_OF_MONTH);if(o===ResolverStyle.LENIENT){const e=l-1,o=r-1;return LocalDate.of(t,1,1).plusMonths(e).plusDays(o)}return o===ResolverStyle.SMART?(ChronoField.DAY_OF_MONTH.checkValidValue(r),4===l||6===l||9===l||11===l?r=Math.min(r,30):2===l&&(r=Math.min(r,Month.FEBRUARY.length(Year.isLeap(t)))),LocalDate.of(t,l,r)):LocalDate.of(t,l,r)}if(e.containsKey(ChronoField.DAY_OF_YEAR)){const t=ChronoField.YEAR.checkValidIntValue(e.remove(ChronoField.YEAR));if(o===ResolverStyle.LENIENT){const o=MathUtil.safeSubtract(e.remove(ChronoField.DAY_OF_YEAR),1);return LocalDate.ofYearDay(t,1).plusDays(o)}const l=ChronoField.DAY_OF_YEAR.checkValidIntValue(e.remove(ChronoField.DAY_OF_YEAR));return LocalDate.ofYearDay(t,l)}if(e.containsKey(ChronoField.ALIGNED_WEEK_OF_YEAR)){if(e.containsKey(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR)){const t=ChronoField.YEAR.checkValidIntValue(e.remove(ChronoField.YEAR));if(o===ResolverStyle.LENIENT){const o=MathUtil.safeSubtract(e.remove(ChronoField.ALIGNED_WEEK_OF_YEAR),1),l=MathUtil.safeSubtract(e.remove(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR),1);return LocalDate.of(t,1,1).plusWeeks(o).plusDays(l)}const l=ChronoField.ALIGNED_WEEK_OF_YEAR.checkValidIntValue(e.remove(ChronoField.ALIGNED_WEEK_OF_YEAR)),r=ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR.checkValidIntValue(e.remove(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR)),i=LocalDate.of(t,1,1).plusDays(7*(l-1)+(r-1));if(o===ResolverStyle.STRICT&&i.get(ChronoField.YEAR)!==t)throw new DateTimeException("Strict mode rejected date parsed to a different year");return i}if(e.containsKey(ChronoField.DAY_OF_WEEK)){const t=ChronoField.YEAR.checkValidIntValue(e.remove(ChronoField.YEAR));if(o===ResolverStyle.LENIENT){const o=MathUtil.safeSubtract(e.remove(ChronoField.ALIGNED_WEEK_OF_YEAR),1),l=MathUtil.safeSubtract(e.remove(ChronoField.DAY_OF_WEEK),1);return LocalDate.of(t,1,1).plusWeeks(o).plusDays(l)}const l=ChronoField.ALIGNED_WEEK_OF_YEAR.checkValidIntValue(e.remove(ChronoField.ALIGNED_WEEK_OF_YEAR)),r=ChronoField.DAY_OF_WEEK.checkValidIntValue(e.remove(ChronoField.DAY_OF_WEEK)),i=LocalDate.of(t,1,1).plusWeeks(l-1).with(TemporalAdjusters.nextOrSame(DayOfWeek.of(r)));if(o===ResolverStyle.STRICT&&i.get(ChronoField.YEAR)!==t)throw new DateTimeException("Strict mode rejected date parsed to a different month");return i}}}return null}date(e){return LocalDate.from(e)}}export function _init(){IsoChronology.INSTANCE=new IsoChronology("IsoChronology")}