/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"../assert";import{DateTimeException}from"../errors";import{MathUtil}from"../MathUtil";import{EnumMap}from"./EnumMap";import{ResolverStyle}from"./ResolverStyle";import{IsoChronology}from"../chrono/IsoChronology";import{ChronoLocalDate}from"../chrono/ChronoLocalDate";import{ChronoField}from"../temporal/ChronoField";import{TemporalAccessor}from"../temporal/TemporalAccessor";import{TemporalQueries}from"../temporal/TemporalQueries";import{LocalTime}from"../LocalTime";import{LocalDate}from"../LocalDate";import{Period}from"../Period";import{ZoneOffset}from"../ZoneOffset";export class DateTimeBuilder extends TemporalAccessor{static create(e,i){const l=new DateTimeBuilder;return l._addFieldValue(e,i),l}constructor(){super(),this.fieldValues=new EnumMap,this.chrono=null,this.zone=null,this.date=null,this.time=null,this.leapSecond=!1,this.excessDays=null}getFieldValue0(e){return this.fieldValues.get(e)}_addFieldValue(e,i){requireNonNull(e,"field");const l=this.getFieldValue0(e);if(null!=l&&l!==i)throw new DateTimeException(`Conflict found: ${e} ${l} differs from ${e} ${i}: ${this}`);return this._putFieldValue0(e,i)}_putFieldValue0(e,i){return this.fieldValues.put(e,i),this}resolve(e,i){return null!=i&&this.fieldValues.retainAll(i),this._mergeDate(e),this._mergeTime(e),this._resolveTimeInferZeroes(e),null!=this.excessDays&&!1===this.excessDays.isZero()&&null!=this.date&&null!=this.time&&(this.date=this.date.plus(this.excessDays),this.excessDays=Period.ZERO),this._resolveInstant(),this}_mergeDate(e){this._checkDate(IsoChronology.INSTANCE.resolveDate(this.fieldValues,e))}_checkDate(e){if(null!=e){this._addObject(e);for(const i in this.fieldValues.keySet()){const l=ChronoField.byName(i);if(l&&void 0!==this.fieldValues.get(l)&&l.isDateBased()){let i;try{i=e.getLong(l)}catch(e){if(e instanceof DateTimeException)continue;throw e}const o=this.fieldValues.get(l);if(i!==o)throw new DateTimeException(`Conflict found: Field ${l} ${i} differs from ${l} ${o} derived from ${e}`)}}}}_mergeTime(e){if(this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_DAY)){const i=this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_DAY);e!==ResolverStyle.LENIENT&&(e===ResolverStyle.SMART&&0===i||ChronoField.CLOCK_HOUR_OF_DAY.checkValidValue(i)),this._addFieldValue(ChronoField.HOUR_OF_DAY,24===i?0:i)}if(this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_AMPM)){const i=this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_AMPM);e!==ResolverStyle.LENIENT&&(e===ResolverStyle.SMART&&0===i||ChronoField.CLOCK_HOUR_OF_AMPM.checkValidValue(i)),this._addFieldValue(ChronoField.HOUR_OF_AMPM,12===i?0:i)}if(e!==ResolverStyle.LENIENT&&(this.fieldValues.containsKey(ChronoField.AMPM_OF_DAY)&&ChronoField.AMPM_OF_DAY.checkValidValue(this.fieldValues.get(ChronoField.AMPM_OF_DAY)),this.fieldValues.containsKey(ChronoField.HOUR_OF_AMPM)&&ChronoField.HOUR_OF_AMPM.checkValidValue(this.fieldValues.get(ChronoField.HOUR_OF_AMPM))),this.fieldValues.containsKey(ChronoField.AMPM_OF_DAY)&&this.fieldValues.containsKey(ChronoField.HOUR_OF_AMPM)){const e=this.fieldValues.remove(ChronoField.AMPM_OF_DAY),i=this.fieldValues.remove(ChronoField.HOUR_OF_AMPM);this._addFieldValue(ChronoField.HOUR_OF_DAY,12*e+i)}if(this.fieldValues.containsKey(ChronoField.NANO_OF_DAY)){const i=this.fieldValues.remove(ChronoField.NANO_OF_DAY);e!==ResolverStyle.LENIENT&&ChronoField.NANO_OF_DAY.checkValidValue(i),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(i,1e9)),this._addFieldValue(ChronoField.NANO_OF_SECOND,MathUtil.intMod(i,1e9))}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_DAY)){const i=this.fieldValues.remove(ChronoField.MICRO_OF_DAY);e!==ResolverStyle.LENIENT&&ChronoField.MICRO_OF_DAY.checkValidValue(i),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(i,1e6)),this._addFieldValue(ChronoField.MICRO_OF_SECOND,MathUtil.intMod(i,1e6))}if(this.fieldValues.containsKey(ChronoField.MILLI_OF_DAY)){const i=this.fieldValues.remove(ChronoField.MILLI_OF_DAY);e!==ResolverStyle.LENIENT&&ChronoField.MILLI_OF_DAY.checkValidValue(i),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(i,1e3)),this._addFieldValue(ChronoField.MILLI_OF_SECOND,MathUtil.intMod(i,1e3))}if(this.fieldValues.containsKey(ChronoField.SECOND_OF_DAY)){const i=this.fieldValues.remove(ChronoField.SECOND_OF_DAY);e!==ResolverStyle.LENIENT&&ChronoField.SECOND_OF_DAY.checkValidValue(i),this._addFieldValue(ChronoField.HOUR_OF_DAY,MathUtil.intDiv(i,3600)),this._addFieldValue(ChronoField.MINUTE_OF_HOUR,MathUtil.intMod(MathUtil.intDiv(i,60),60)),this._addFieldValue(ChronoField.SECOND_OF_MINUTE,MathUtil.intMod(i,60))}if(this.fieldValues.containsKey(ChronoField.MINUTE_OF_DAY)){const i=this.fieldValues.remove(ChronoField.MINUTE_OF_DAY);e!==ResolverStyle.LENIENT&&ChronoField.MINUTE_OF_DAY.checkValidValue(i),this._addFieldValue(ChronoField.HOUR_OF_DAY,MathUtil.intDiv(i,60)),this._addFieldValue(ChronoField.MINUTE_OF_HOUR,MathUtil.intMod(i,60))}if(e!==ResolverStyle.LENIENT&&(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&ChronoField.MILLI_OF_SECOND.checkValidValue(this.fieldValues.get(ChronoField.MILLI_OF_SECOND)),this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)&&ChronoField.MICRO_OF_SECOND.checkValidValue(this.fieldValues.get(ChronoField.MICRO_OF_SECOND))),this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)){const e=this.fieldValues.remove(ChronoField.MILLI_OF_SECOND),i=this.fieldValues.get(ChronoField.MICRO_OF_SECOND);this._putFieldValue0(ChronoField.MICRO_OF_SECOND,1e3*e+MathUtil.intMod(i,1e3))}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)){const e=this.fieldValues.get(ChronoField.NANO_OF_SECOND);this._putFieldValue0(ChronoField.MICRO_OF_SECOND,MathUtil.intDiv(e,1e3)),this.fieldValues.remove(ChronoField.MICRO_OF_SECOND)}if(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)){const e=this.fieldValues.get(ChronoField.NANO_OF_SECOND);this._putFieldValue0(ChronoField.MILLI_OF_SECOND,MathUtil.intDiv(e,1e6)),this.fieldValues.remove(ChronoField.MILLI_OF_SECOND)}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)){const e=this.fieldValues.remove(ChronoField.MICRO_OF_SECOND);this._putFieldValue0(ChronoField.NANO_OF_SECOND,1e3*e)}else if(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)){const e=this.fieldValues.remove(ChronoField.MILLI_OF_SECOND);this._putFieldValue0(ChronoField.NANO_OF_SECOND,1e6*e)}}_resolveTimeInferZeroes(e){let i=this.fieldValues.get(ChronoField.HOUR_OF_DAY);const l=this.fieldValues.get(ChronoField.MINUTE_OF_HOUR),o=this.fieldValues.get(ChronoField.SECOND_OF_MINUTE);let t=this.fieldValues.get(ChronoField.NANO_OF_SECOND);if(null!=i&&(null!=l||null==o&&null==t)&&(null==l||null!=o||null==t)){if(e!==ResolverStyle.LENIENT){if(null!=i){e!==ResolverStyle.SMART||24!==i||null!=l&&0!==l||null!=o&&0!==o||null!=t&&0!==t||(i=0,this.excessDays=Period.ofDays(1));const s=ChronoField.HOUR_OF_DAY.checkValidIntValue(i);if(null!=l){const e=ChronoField.MINUTE_OF_HOUR.checkValidIntValue(l);if(null!=o){const i=ChronoField.SECOND_OF_MINUTE.checkValidIntValue(o);if(null!=t){const l=ChronoField.NANO_OF_SECOND.checkValidIntValue(t);this._addObject(LocalTime.of(s,e,i,l))}else this._addObject(LocalTime.of(s,e,i))}else null==t&&this._addObject(LocalTime.of(s,e))}else null==o&&null==t&&this._addObject(LocalTime.of(s,0))}}else if(null!=i){let e=i;if(null!=l)if(null!=o){null==t&&(t=0);let i=MathUtil.safeMultiply(e,36e11);i=MathUtil.safeAdd(i,MathUtil.safeMultiply(l,6e10)),i=MathUtil.safeAdd(i,MathUtil.safeMultiply(o,1e9)),i=MathUtil.safeAdd(i,t);const s=MathUtil.floorDiv(i,864e11),a=MathUtil.floorMod(i,864e11);this._addObject(LocalTime.ofNanoOfDay(a)),this.excessDays=Period.ofDays(s)}else{let i=MathUtil.safeMultiply(e,3600);i=MathUtil.safeAdd(i,MathUtil.safeMultiply(l,60));const o=MathUtil.floorDiv(i,86400),t=MathUtil.floorMod(i,86400);this._addObject(LocalTime.ofSecondOfDay(t)),this.excessDays=Period.ofDays(o)}else{const i=MathUtil.safeToInt(MathUtil.floorDiv(e,24));e=MathUtil.floorMod(e,24),this._addObject(LocalTime.of(e,0)),this.excessDays=Period.ofDays(i)}}this.fieldValues.remove(ChronoField.HOUR_OF_DAY),this.fieldValues.remove(ChronoField.MINUTE_OF_HOUR),this.fieldValues.remove(ChronoField.SECOND_OF_MINUTE),this.fieldValues.remove(ChronoField.NANO_OF_SECOND)}}_addObject(e){e instanceof ChronoLocalDate?this.date=e:e instanceof LocalTime&&(this.time=e)}_resolveInstant(){if(null!=this.date&&null!=this.time){const e=this.fieldValues.get(ChronoField.OFFSET_SECONDS);if(null!=e){const i=ZoneOffset.ofTotalSeconds(e),l=this.date.atTime(this.time).atZone(i).getLong(ChronoField.INSTANT_SECONDS);this.fieldValues.put(ChronoField.INSTANT_SECONDS,l)}else if(null!=this.zone){const e=this.date.atTime(this.time).atZone(this.zone).getLong(ChronoField.INSTANT_SECONDS);this.fieldValues.put(ChronoField.INSTANT_SECONDS,e)}}}build(e){return e.queryFrom(this)}isSupported(e){return null!=e&&(this.fieldValues.containsKey(e)&&void 0!==this.fieldValues.get(e)||null!=this.date&&this.date.isSupported(e)||null!=this.time&&this.time.isSupported(e))}getLong(e){requireNonNull(e,"field");const i=this.getFieldValue0(e);if(null==i){if(null!=this.date&&this.date.isSupported(e))return this.date.getLong(e);if(null!=this.time&&this.time.isSupported(e))return this.time.getLong(e);throw new DateTimeException(`Field not found: ${e}`)}return i}query(e){return e===TemporalQueries.zoneId()?this.zone:e===TemporalQueries.chronology()?this.chrono:e===TemporalQueries.localDate()?null!=this.date?LocalDate.from(this.date):null:e===TemporalQueries.localTime()?this.time:e===TemporalQueries.zone()||e===TemporalQueries.offset()?e.queryFrom(this):e===TemporalQueries.precision()?null:e.queryFrom(this)}}