/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull,requireInstance}from"../assert";import{IllegalArgumentException,IllegalStateException}from"../errors";import{MathUtil}from"../MathUtil";import{LocalDate}from"../LocalDate";import{LocalDateTime}from"../LocalDateTime";import{ZoneOffset}from"../ZoneOffset";import{ChronoLocalDate}from"../chrono/ChronoLocalDate";import{ChronoField}from"../temporal/ChronoField";import{IsoFields}from"../temporal/IsoFields";import{TemporalQueries}from"../temporal/TemporalQueries";import{DateTimeFormatter}from"./DateTimeFormatter";import{DecimalStyle}from"./DecimalStyle";import{SignStyle}from"./SignStyle";import{TextStyle}from"./TextStyle";import{ResolverStyle}from"./ResolverStyle";import{CharLiteralPrinterParser}from"./parser/CharLiteralPrinterParser";import{CompositePrinterParser}from"./parser/CompositePrinterParser";import{FractionPrinterParser}from"./parser/FractionPrinterParser";import{NumberPrinterParser,ReducedPrinterParser}from"./parser/NumberPrinterParser";import{OffsetIdPrinterParser}from"./parser/OffsetIdPrinterParser";import{PadPrinterParserDecorator}from"./parser/PadPrinterParserDecorator";import{SettingsParser}from"./parser/SettingsParser";import{StringLiteralPrinterParser}from"./parser/StringLiteralPrinterParser";import{ZoneIdPrinterParser}from"./parser/ZoneIdPrinterParser";const MAX_WIDTH=15;export class DateTimeFormatterBuilder{constructor(){this._active=this,this._parent=null,this._printerParsers=[],this._optional=!1,this._padNextWidth=0,this._padNextChar=null,this._valueParserIndex=-1}static _of(e,t){requireNonNull(e,"parent"),requireNonNull(t,"optional");const r=new DateTimeFormatterBuilder;return r._parent=e,r._optional=t,r}parseCaseSensitive(){return this._appendInternalPrinterParser(SettingsParser.SENSITIVE),this}parseCaseInsensitive(){return this._appendInternalPrinterParser(SettingsParser.INSENSITIVE),this}parseStrict(){return this._appendInternalPrinterParser(SettingsParser.STRICT),this}parseLenient(){return this._appendInternalPrinterParser(SettingsParser.LENIENT),this}parseDefaulting(e,t){return requireNonNull(e),this._appendInternal(new DefaultingParser(e,t)),this}appendValue(){return 1===arguments.length?this._appendValue1.apply(this,arguments):2===arguments.length?this._appendValue2.apply(this,arguments):this._appendValue4.apply(this,arguments)}_appendValue1(e){return requireNonNull(e),this._appendValuePrinterParser(new NumberPrinterParser(e,1,15,SignStyle.NORMAL)),this}_appendValue2(e,t){if(requireNonNull(e),t<1||t>15)throw new IllegalArgumentException(`The width must be from 1 to 15 inclusive but was ${t}`);const r=new NumberPrinterParser(e,t,t,SignStyle.NOT_NEGATIVE);return this._appendValuePrinterParser(r),this}_appendValue4(e,t,r,a){if(requireNonNull(e),requireNonNull(a),t===r&&a===SignStyle.NOT_NEGATIVE)return this._appendValue2(e,r);if(t<1||t>15)throw new IllegalArgumentException(`The minimum width must be from 1 to 15 inclusive but was ${t}`);if(r<1||r>15)throw new IllegalArgumentException(`The minimum width must be from 1 to 15 inclusive but was ${r}`);if(r<t)throw new IllegalArgumentException(`The maximum width must exceed or equal the minimum width but ${r} < ${t}`);const i=new NumberPrinterParser(e,t,r,a);return this._appendValuePrinterParser(i),this}appendValueReduced(){return 4===arguments.length&&arguments[3]instanceof ChronoLocalDate?this._appendValueReducedFieldWidthMaxWidthBaseDate.apply(this,arguments):this._appendValueReducedFieldWidthMaxWidthBaseValue.apply(this,arguments)}_appendValueReducedFieldWidthMaxWidthBaseValue(e,t,r,a){requireNonNull(e,"field");const i=new ReducedPrinterParser(e,t,r,a,null);return this._appendValuePrinterParser(i),this}_appendValueReducedFieldWidthMaxWidthBaseDate(e,t,r,a){requireNonNull(e,"field"),requireNonNull(a,"baseDate"),requireInstance(a,ChronoLocalDate,"baseDate");const i=new ReducedPrinterParser(e,t,r,0,a);return this._appendValuePrinterParser(i),this}_appendValuePrinterParser(e){if(assert(null!=e),this._active._valueParserIndex>=0&&this._active._printerParsers[this._active._valueParserIndex]instanceof NumberPrinterParser){const t=this._active._valueParserIndex;let r=this._active._printerParsers[t];e.minWidth()===e.maxWidth()&&e.signStyle()===SignStyle.NOT_NEGATIVE?(r=r.withSubsequentWidth(e.maxWidth()),this._appendInternal(e.withFixedWidth()),this._active._valueParserIndex=t):(r=r.withFixedWidth(),this._active._valueParserIndex=this._appendInternal(e)),this._active._printerParsers[t]=r}else this._active._valueParserIndex=this._appendInternal(e);return this}appendFraction(e,t,r,a){return this._appendInternal(new FractionPrinterParser(e,t,r,a)),this}appendInstant(e=-2){if(e<-2||e>9)throw new IllegalArgumentException(`Invalid fractional digits: ${e}`);return this._appendInternal(new InstantPrinterParser(e)),this}appendOffsetId(){return this._appendInternal(OffsetIdPrinterParser.INSTANCE_ID),this}appendOffset(e,t){return this._appendInternalPrinterParser(new OffsetIdPrinterParser(t,e)),this}appendZoneId(){return this._appendInternal(new ZoneIdPrinterParser(TemporalQueries.zoneId(),"ZoneId()")),this}appendPattern(e){return requireNonNull(e,"pattern"),this._parsePattern(e),this}appendZoneText(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendText(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendLocalizedOffset(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendWeekField(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}_parsePattern(e){const t={G:ChronoField.ERA,y:ChronoField.YEAR_OF_ERA,u:ChronoField.YEAR,Q:IsoFields.QUARTER_OF_YEAR,q:IsoFields.QUARTER_OF_YEAR,M:ChronoField.MONTH_OF_YEAR,L:ChronoField.MONTH_OF_YEAR,D:ChronoField.DAY_OF_YEAR,d:ChronoField.DAY_OF_MONTH,F:ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH,E:ChronoField.DAY_OF_WEEK,c:ChronoField.DAY_OF_WEEK,e:ChronoField.DAY_OF_WEEK,a:ChronoField.AMPM_OF_DAY,H:ChronoField.HOUR_OF_DAY,k:ChronoField.CLOCK_HOUR_OF_DAY,K:ChronoField.HOUR_OF_AMPM,h:ChronoField.CLOCK_HOUR_OF_AMPM,m:ChronoField.MINUTE_OF_HOUR,s:ChronoField.SECOND_OF_MINUTE,S:ChronoField.NANO_OF_SECOND,A:ChronoField.MILLI_OF_DAY,n:ChronoField.NANO_OF_SECOND,N:ChronoField.NANO_OF_DAY};for(let r=0;r<e.length;r++){let a=e.charAt(r);if(a>="A"&&a<="Z"||a>="a"&&a<="z"){let i=r++;for(;r<e.length&&e.charAt(r)===a;r++);let n=r-i;if("p"===a){let t=0;if(r<e.length&&(a=e.charAt(r),a>="A"&&a<="Z"||a>="a"&&a<="z")){for(t=n,i=r++;r<e.length&&e.charAt(r)===a;r++);n=r-i}if(0===t)throw new IllegalArgumentException(`Pad letter 'p' must be followed by valid pad pattern: ${e}`);this.padNext(t)}const s=t[a];if(null!=s)this._parseField(a,n,s);else if("z"===a){if(n>4)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);4===n?this.appendZoneText(TextStyle.FULL):this.appendZoneText(TextStyle.SHORT)}else if("V"===a){if(2!==n)throw new IllegalArgumentException(`Pattern letter count must be 2: ${a}`);this.appendZoneId()}else if("Z"===a)if(n<4)this.appendOffset("+HHMM","+0000");else if(4===n)this.appendLocalizedOffset(TextStyle.FULL);else{if(5!==n)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);this.appendOffset("+HH:MM:ss","Z")}else if("O"===a)if(1===n)this.appendLocalizedOffset(TextStyle.SHORT);else{if(4!==n)throw new IllegalArgumentException(`Pattern letter count must be 1 or 4: ${a}`);this.appendLocalizedOffset(TextStyle.FULL)}else if("X"===a){if(n>5)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);this.appendOffset(OffsetIdPrinterParser.PATTERNS[n+(1===n?0:1)],"Z")}else if("x"===a){if(n>5)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);const e=1===n?"+00":n%2==0?"+0000":"+00:00";this.appendOffset(OffsetIdPrinterParser.PATTERNS[n+(1===n?0:1)],e)}else if("W"===a){if(n>1)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);this.appendWeekField("W",n)}else if("w"===a){if(n>2)throw new IllegalArgumentException(`Too many pattern letters: ${a}`);this.appendWeekField("w",n)}else{if("Y"!==a)throw new IllegalArgumentException(`Unknown pattern letter: ${a}`);this.appendWeekField("Y",n)}r--}else if("'"===a){const t=r++;for(;r<e.length;r++)if("'"===e.charAt(r)){if(!(r+1<e.length&&"'"===e.charAt(r+1)))break;r++}if(r>=e.length)throw new IllegalArgumentException(`Pattern ends with an incomplete string literal: ${e}`);const a=e.substring(t+1,r);0===a.length?this.appendLiteral("'"):this.appendLiteral(a.replace("''","'"))}else if("["===a)this.optionalStart();else if("]"===a){if(null===this._active._parent)throw new IllegalArgumentException("Pattern invalid as it contains ] without previous [");this.optionalEnd()}else{if("{"===a||"}"===a||"#"===a)throw new IllegalArgumentException(`Pattern includes reserved character: '${a}'`);this.appendLiteral(a)}}}_parseField(e,t,r){switch(e){case"u":case"y":2===t?this.appendValueReduced(r,2,2,ReducedPrinterParser.BASE_DATE):t<4?this.appendValue(r,t,15,SignStyle.NORMAL):this.appendValue(r,t,15,SignStyle.EXCEEDS_PAD);break;case"M":case"Q":switch(t){case 1:this.appendValue(r);break;case 2:this.appendValue(r,2);break;case 3:this.appendText(r,TextStyle.SHORT);break;case 4:this.appendText(r,TextStyle.FULL);break;case 5:this.appendText(r,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${e}`)}break;case"L":case"q":switch(t){case 1:this.appendValue(r);break;case 2:this.appendValue(r,2);break;case 3:this.appendText(r,TextStyle.SHORT_STANDALONE);break;case 4:this.appendText(r,TextStyle.FULL_STANDALONE);break;case 5:this.appendText(r,TextStyle.NARROW_STANDALONE);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${e}`)}break;case"e":switch(t){case 1:case 2:this.appendWeekField("e",t);break;case 3:this.appendText(r,TextStyle.SHORT);break;case 4:this.appendText(r,TextStyle.FULL);break;case 5:this.appendText(r,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${e}`)}break;case"c":switch(t){case 1:this.appendWeekField("c",t);break;case 2:throw new IllegalArgumentException(`Invalid number of pattern letters: ${e}`);case 3:this.appendText(r,TextStyle.SHORT_STANDALONE);break;case 4:this.appendText(r,TextStyle.FULL_STANDALONE);break;case 5:this.appendText(r,TextStyle.NARROW_STANDALONE);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${e}`)}break;case"a":if(1!==t)throw new IllegalArgumentException(`Too many pattern letters: ${e}`);this.appendText(r,TextStyle.SHORT);break;case"E":case"G":switch(t){case 1:case 2:case 3:this.appendText(r,TextStyle.SHORT);break;case 4:this.appendText(r,TextStyle.FULL);break;case 5:this.appendText(r,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${e}`)}break;case"S":this.appendFraction(ChronoField.NANO_OF_SECOND,t,t,!1);break;case"F":if(1!==t)throw new IllegalArgumentException(`Too many pattern letters: ${e}`);this.appendValue(r);break;case"d":case"h":case"H":case"k":case"K":case"m":case"s":if(1===t)this.appendValue(r);else{if(2!==t)throw new IllegalArgumentException(`Too many pattern letters: ${e}`);this.appendValue(r,t)}break;case"D":if(1===t)this.appendValue(r);else{if(!(t<=3))throw new IllegalArgumentException(`Too many pattern letters: ${e}`);this.appendValue(r,t)}break;default:1===t?this.appendValue(r):this.appendValue(r,t)}}padNext(){return 1===arguments.length?this._padNext1.apply(this,arguments):this._padNext2.apply(this,arguments)}_padNext1(e){return this._padNext2(e," ")}_padNext2(e,t){if(e<1)throw new IllegalArgumentException(`The pad width must be at least one but was ${e}`);return this._active._padNextWidth=e,this._active._padNextChar=t,this._active._valueParserIndex=-1,this}optionalStart(){return this._active._valueParserIndex=-1,this._active=DateTimeFormatterBuilder._of(this._active,!0),this}optionalEnd(){if(null==this._active._parent)throw new IllegalStateException("Cannot call optionalEnd() as there was no previous call to optionalStart()");if(this._active._printerParsers.length>0){const e=new CompositePrinterParser(this._active._printerParsers,this._active._optional);this._active=this._active._parent,this._appendInternal(e)}else this._active=this._active._parent;return this}_appendInternal(e){return assert(null!=e),this._active._padNextWidth>0&&(null!=e&&(e=new PadPrinterParserDecorator(e,this._active._padNextWidth,this._active._padNextChar)),this._active._padNextWidth=0,this._active._padNextChar=0),this._active._printerParsers.push(e),this._active._valueParserIndex=-1,this._active._printerParsers.length-1}appendLiteral(e){return assert(null!=e),e.length>0&&(1===e.length?this._appendInternalPrinterParser(new CharLiteralPrinterParser(e.charAt(0))):this._appendInternalPrinterParser(new StringLiteralPrinterParser(e))),this}_appendInternalPrinterParser(e){return assert(null!=e),this._active._padNextWidth>0&&(null!=e&&(e=new PadPrinterParserDecorator(e,this._active._padNextWidth,this._active._padNextChar)),this._active._padNextWidth=0,this._active._padNextChar=0),this._active._printerParsers.push(e),this._active._valueParserIndex=-1,this._active._printerParsers.length-1}append(e){return requireNonNull(e,"formatter"),this._appendInternal(e._toPrinterParser(!1)),this}toFormatter(e=ResolverStyle.SMART){for(;null!=this._active._parent;)this.optionalEnd();const t=new CompositePrinterParser(this._printerParsers,!1);return new DateTimeFormatter(t,null,DecimalStyle.STANDARD,e,null,null,null)}}const SECONDS_PER_10000_YEARS=31556952e4,SECONDS_0000_TO_1970=62167219200;class InstantPrinterParser{constructor(e){this.fractionalDigits=e}print(e,t){const r=e.getValue(ChronoField.INSTANT_SECONDS);let a=0;if(e.temporal().isSupported(ChronoField.NANO_OF_SECOND)&&(a=e.temporal().getLong(ChronoField.NANO_OF_SECOND)),null==r)return!1;const i=r;let n=ChronoField.NANO_OF_SECOND.checkValidIntValue(a);if(i>=-62167219200){const e=i-31556952e4+62167219200,r=MathUtil.floorDiv(e,31556952e4)+1,a=MathUtil.floorMod(e,31556952e4),n=LocalDateTime.ofEpochSecond(a-62167219200,0,ZoneOffset.UTC);r>0&&t.append("+").append(r),t.append(n.toString()),0===n.second()&&t.append(":00")}else{const e=i+62167219200,r=MathUtil.intDiv(e,31556952e4),a=MathUtil.intMod(e,31556952e4),n=LocalDateTime.ofEpochSecond(a-62167219200,0,ZoneOffset.UTC),s=t.length();t.append(n.toString()),0===n.second()&&t.append(":00"),r<0&&(-1e4===n.year()?t.replace(s,s+2,""+(r-1)):0===a?t.insert(s,r):t.insert(s+1,Math.abs(r)))}if(-2===this.fractionalDigits)0!==n&&(t.append("."),0===MathUtil.intMod(n,1e6)?t.append(`${MathUtil.intDiv(n,1e6)+1e3}`.substring(1)):0===MathUtil.intMod(n,1e3)?t.append(`${MathUtil.intDiv(n,1e3)+1e6}`.substring(1)):t.append(`${n+1e9}`.substring(1)));else if(this.fractionalDigits>0||-1===this.fractionalDigits&&n>0){t.append(".");let e=1e8;for(let r=0;-1===this.fractionalDigits&&n>0||r<this.fractionalDigits;r++){const r=MathUtil.intDiv(n,e);t.append(r),n-=r*e,e=MathUtil.intDiv(e,10)}}return t.append("Z"),!0}parse(e,t,r){const a=e.copy(),i=this.fractionalDigits<0?0:this.fractionalDigits,n=this.fractionalDigits<0?9:this.fractionalDigits,s=(new DateTimeFormatterBuilder).append(DateTimeFormatter.ISO_LOCAL_DATE).appendLiteral("T").appendValue(ChronoField.HOUR_OF_DAY,2).appendLiteral(":").appendValue(ChronoField.MINUTE_OF_HOUR,2).appendLiteral(":").appendValue(ChronoField.SECOND_OF_MINUTE,2).appendFraction(ChronoField.NANO_OF_SECOND,i,n,!0).appendLiteral("Z").toFormatter()._toPrinterParser(!1).parse(a,t,r);if(s<0)return s;const l=a.getParsed(ChronoField.YEAR),o=a.getParsed(ChronoField.MONTH_OF_YEAR),p=a.getParsed(ChronoField.DAY_OF_MONTH);let d=a.getParsed(ChronoField.HOUR_OF_DAY);const h=a.getParsed(ChronoField.MINUTE_OF_HOUR),u=a.getParsed(ChronoField.SECOND_OF_MINUTE),c=a.getParsed(ChronoField.NANO_OF_SECOND);let _=null!=u?u:0;const P=null!=c?c:0,m=MathUtil.intMod(l,1e4);let f,g=0;24===d&&0===h&&0===_&&0===P?(d=0,g=1):23===d&&59===h&&60===_&&(e.setParsedLeapSecond(),_=59);try{f=LocalDateTime.of(m,o,p,d,h,_,0).plusDays(g).toEpochSecond(ZoneOffset.UTC),f+=MathUtil.safeMultiply(MathUtil.intDiv(l,1e4),31556952e4)}catch(e){return~r}let T=s;return T=e.setParsedField(ChronoField.INSTANT_SECONDS,f,r,T),e.setParsedField(ChronoField.NANO_OF_SECOND,P,r,T)}toString(){return"Instant()"}}class DefaultingParser{constructor(e,t){this._field=e,this._value=t}print(){return!0}parse(e,t,r){return null==e.getParsed(this._field)&&e.setParsedField(this._field,this._value,r,r),r}}export function _init(){ReducedPrinterParser.BASE_DATE=LocalDate.of(2e3,1,1),DateTimeFormatterBuilder.CompositePrinterParser=CompositePrinterParser,DateTimeFormatterBuilder.PadPrinterParserDecorator=PadPrinterParserDecorator,DateTimeFormatterBuilder.SettingsParser=SettingsParser,DateTimeFormatterBuilder.CharLiteralPrinterParser=StringLiteralPrinterParser,DateTimeFormatterBuilder.StringLiteralPrinterParser=StringLiteralPrinterParser,DateTimeFormatterBuilder.CharLiteralPrinterParser=CharLiteralPrinterParser,DateTimeFormatterBuilder.NumberPrinterParser=NumberPrinterParser,DateTimeFormatterBuilder.ReducedPrinterParser=ReducedPrinterParser,DateTimeFormatterBuilder.FractionPrinterParser=FractionPrinterParser,DateTimeFormatterBuilder.OffsetIdPrinterParser=OffsetIdPrinterParser,DateTimeFormatterBuilder.ZoneIdPrinterParser=ZoneIdPrinterParser}