/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert}from"../../assert";import{ArithmeticException,DateTimeException,IllegalArgumentException}from"../../errors";import{MathUtil}from"../../MathUtil";import{IsoChronology}from"../../chrono/IsoChronology";import{SignStyle}from"../SignStyle";const MAX_WIDTH=15,EXCEED_POINTS=[0,10,100,1e3,1e4,1e5,1e6,1e7,1e8,1e9];export class NumberPrinterParser{constructor(t,i,e,s,h=0){this._field=t,this._minWidth=i,this._maxWidth=e,this._signStyle=s,this._subsequentWidth=h}field(){return this._field}minWidth(){return this._minWidth}maxWidth(){return this._maxWidth}signStyle(){return this._signStyle}withFixedWidth(){return-1===this._subsequentWidth?this:new NumberPrinterParser(this._field,this._minWidth,this._maxWidth,this._signStyle,-1)}withSubsequentWidth(t){return new NumberPrinterParser(this._field,this._minWidth,this._maxWidth,this._signStyle,this._subsequentWidth+t)}_isFixedWidth(){return-1===this._subsequentWidth||this._subsequentWidth>0&&this._minWidth===this._maxWidth&&this._signStyle===SignStyle.NOT_NEGATIVE}print(t,i){const e=t.getValue(this._field);if(null==e)return!1;const s=this._getValue(t,e),h=t.symbols();let n=`${Math.abs(s)}`;if(n.length>this._maxWidth)throw new DateTimeException(`Field ${this._field} cannot be printed as the value ${s} exceeds the maximum print width of ${this._maxWidth}`);if(n=h.convertNumberToI18N(n),s>=0)switch(this._signStyle){case SignStyle.EXCEEDS_PAD:this._minWidth<15&&s>=EXCEED_POINTS[this._minWidth]&&i.append(h.positiveSign());break;case SignStyle.ALWAYS:i.append(h.positiveSign())}else switch(this._signStyle){case SignStyle.NORMAL:case SignStyle.EXCEEDS_PAD:case SignStyle.ALWAYS:i.append(h.negativeSign());break;case SignStyle.NOT_NEGATIVE:throw new DateTimeException(`Field ${this._field} cannot be printed as the value ${s} cannot be negative according to the SignStyle`)}for(let t=0;t<this._minWidth-n.length;t++)i.append(h.zeroDigit());return i.append(n),!0}parse(t,i,e){const s=i.length;if(e===s)return~e;assert(e>=0&&e<s);const h=i.charAt(e);let n=!1,r=!1;if(h===t.symbols().positiveSign()){if(!1===this._signStyle.parse(!0,t.isStrict(),this._minWidth===this._maxWidth))return~e;r=!0,e++}else if(h===t.symbols().negativeSign()){if(!1===this._signStyle.parse(!1,t.isStrict(),this._minWidth===this._maxWidth))return~e;n=!0,e++}else if(this._signStyle===SignStyle.ALWAYS&&t.isStrict())return~e;const a=t.isStrict()||this._isFixedWidth()?this._minWidth:1,l=e+a;if(l>s)return~e;let d=(t.isStrict()||this._isFixedWidth()?this._maxWidth:9)+Math.max(this._subsequentWidth,0),_=0,u=e;for(let h=0;h<2;h++){const n=Math.min(u+d,s);for(;u<n;){const s=i.charAt(u++),h=t.symbols().convertToDigit(s);if(h<0){if(u--,u<l)return~e;break}if(u-e>15)throw new ArithmeticException("number text exceeds length");_=10*_+h}if(!(this._subsequentWidth>0&&0===h))break;{const t=u-e;d=Math.max(a,t-this._subsequentWidth),u=e,_=0}}if(n){if(0===_&&t.isStrict())return~(e-1);0!==_&&(_=-_)}else if(this._signStyle===SignStyle.EXCEEDS_PAD&&t.isStrict()){const t=u-e;if(r){if(t<=this._minWidth)return~(e-1)}else if(t>this._minWidth)return~e}return this._setValue(t,_,e,u)}_getValue(t,i){return i}_setValue(t,i,e,s){return t.setParsedField(this._field,i,e,s)}toString(){return 1===this._minWidth&&15===this._maxWidth&&this._signStyle===SignStyle.NORMAL?`Value(${this._field})`:this._minWidth===this._maxWidth&&this._signStyle===SignStyle.NOT_NEGATIVE?`Value(${this._field},${this._minWidth})`:`Value(${this._field},${this._minWidth},${this._maxWidth},${this._signStyle})`}}export class ReducedPrinterParser extends NumberPrinterParser{constructor(t,i,e,s,h){if(super(t,i,e,SignStyle.NOT_NEGATIVE),i<1||i>10)throw new IllegalArgumentException(`The width must be from 1 to 10 inclusive but was ${i}`);if(e<1||e>10)throw new IllegalArgumentException(`The maxWidth must be from 1 to 10 inclusive but was ${e}`);if(e<i)throw new IllegalArgumentException("The maxWidth must be greater than the width");if(null===h){if(!1===t.range().isValidValue(s))throw new IllegalArgumentException("The base value must be within the range of the field");if(s+EXCEED_POINTS[i]>MathUtil.MAX_SAFE_INTEGER)throw new DateTimeException("Unable to add printer-parser as the range exceeds the capacity of an int")}this._baseValue=s,this._baseDate=h}_getValue(t,i){const e=Math.abs(i);let s=this._baseValue;if(null!==this._baseDate){t.temporal();s=IsoChronology.INSTANCE.date(this._baseDate).get(this._field)}return i>=s&&i<s+EXCEED_POINTS[this._minWidth]?e%EXCEED_POINTS[this._minWidth]:e%EXCEED_POINTS[this._maxWidth]}_setValue(t,i,e,s){let h=this._baseValue;if(null!=this._baseDate){h=t.getEffectiveChronology().date(this._baseDate).get(this._field)}if(s-e===this._minWidth&&i>=0){const t=EXCEED_POINTS[this._minWidth],e=h-h%t;(i=h>0?e+i:e-i)<h&&(i+=t)}return t.setParsedField(this._field,i,e,s)}withFixedWidth(){return-1===this._subsequentWidth?this:new ReducedPrinterParser(this._field,this._minWidth,this._maxWidth,this._baseValue,this._baseDate,-1)}withSubsequentWidth(t){return new ReducedPrinterParser(this._field,this._minWidth,this._maxWidth,this._baseValue,this._baseDate,this._subsequentWidth+t)}isFixedWidth(t){return!1!==t.isStrict()&&super.isFixedWidth(t)}toString(){return`ReducedValue(${this._field},${this._minWidth},${this._maxWidth},${null!=this._baseDate?this._baseDate:this._baseValue})`}}