/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{UnsupportedTemporalTypeException,IllegalStateException}from"../errors";import{DayOfWeek}from"../DayOfWeek";import{Duration}from"../Duration";import{MathUtil}from"../MathUtil";import{LocalDate}from"../LocalDate";import{ChronoField}from"./ChronoField";import{ChronoUnit}from"./ChronoUnit";import{TemporalField}from"./TemporalField";import{TemporalUnit}from"./TemporalUnit";import{ValueRange}from"./ValueRange";import{IsoChronology}from"../chrono/IsoChronology";import{ResolverStyle}from"../format/ResolverStyle";export const IsoFields={};const QUARTER_DAYS=[0,90,181,273,0,91,182,274];class Field extends TemporalField{isDateBased(){return!0}isTimeBased(){return!1}_isIso(){return!0}static _getWeekRangeByLocalDate(e){const t=Field._getWeekBasedYear(e);return ValueRange.of(1,Field._getWeekRangeByYear(t))}static _getWeekRangeByYear(e){const t=LocalDate.of(e,1,1);return t.dayOfWeek()===DayOfWeek.THURSDAY||t.dayOfWeek()===DayOfWeek.WEDNESDAY&&t.isLeapYear()?53:52}static _getWeek(e){const t=e.dayOfWeek().ordinal(),r=e.dayOfYear()-1,o=r+(3-t);let n=o-7*MathUtil.intDiv(o,7)-3;if(n<-3&&(n+=7),r<n)return Field._getWeekRangeByLocalDate(e.withDayOfYear(180).minusYears(1)).maximum();let i=MathUtil.intDiv(r-n,7)+1;return 53===i&&!1===(-3===n||-2===n&&e.isLeapYear())&&(i=1),i}static _getWeekBasedYear(e){let t=e.year(),r=e.dayOfYear();if(r<=3){r-e.dayOfWeek().ordinal()<-2&&t--}else if(r>=363){const o=e.dayOfWeek().ordinal();r=r-363-(e.isLeapYear()?1:0),r-o>=0&&t++}return t}displayName(){return this.toString()}resolve(){return null}name(){return this.toString()}}class DAY_OF_QUARTER_FIELD extends Field{toString(){return"DayOfQuarter"}baseUnit(){return ChronoUnit.DAYS}rangeUnit(){return QUARTER_YEARS}range(){return ValueRange.of(1,90,92)}isSupportedBy(e){return e.isSupported(ChronoField.DAY_OF_YEAR)&&e.isSupported(ChronoField.MONTH_OF_YEAR)&&e.isSupported(ChronoField.YEAR)&&this._isIso(e)}rangeRefinedBy(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");const t=e.getLong(QUARTER_OF_YEAR);if(1===t){const t=e.getLong(ChronoField.YEAR);return IsoChronology.isLeapYear(t)?ValueRange.of(1,91):ValueRange.of(1,90)}return 2===t?ValueRange.of(1,91):3===t||4===t?ValueRange.of(1,92):this.range()}getFrom(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");const t=e.get(ChronoField.DAY_OF_YEAR),r=e.get(ChronoField.MONTH_OF_YEAR),o=e.getLong(ChronoField.YEAR);return t-QUARTER_DAYS[MathUtil.intDiv(r-1,3)+(IsoChronology.isLeapYear(o)?4:0)]}adjustInto(e,t){const r=this.getFrom(e);return this.range().checkValidValue(t,this),e.with(ChronoField.DAY_OF_YEAR,e.getLong(ChronoField.DAY_OF_YEAR)+(t-r))}resolve(e,t,r){const o=e.get(ChronoField.YEAR),n=e.get(QUARTER_OF_YEAR);if(null==o||null==n)return null;const i=ChronoField.YEAR.checkValidIntValue(o),a=e.get(DAY_OF_QUARTER);let E;if(r===ResolverStyle.LENIENT){const e=n;E=LocalDate.of(i,1,1),E=E.plusMonths(MathUtil.safeMultiply(MathUtil.safeSubtract(e,1),3)),E=E.plusDays(MathUtil.safeSubtract(a,1))}else{const e=QUARTER_OF_YEAR.range().checkValidIntValue(n,QUARTER_OF_YEAR);if(r===ResolverStyle.STRICT){let t=92;1===e?t=IsoChronology.isLeapYear(i)?91:90:2===e&&(t=91),ValueRange.of(1,t).checkValidValue(a,this)}else this.range().checkValidValue(a,this);E=LocalDate.of(i,3*(e-1)+1,1).plusDays(a-1)}return e.remove(this),e.remove(ChronoField.YEAR),e.remove(QUARTER_OF_YEAR),E}}class QUARTER_OF_YEAR_FIELD extends Field{toString(){return"QuarterOfYear"}baseUnit(){return QUARTER_YEARS}rangeUnit(){return ChronoUnit.YEARS}range(){return ValueRange.of(1,4)}isSupportedBy(e){return e.isSupported(ChronoField.MONTH_OF_YEAR)&&this._isIso(e)}rangeRefinedBy(e){return this.range()}getFrom(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: QuarterOfYear");const t=e.getLong(ChronoField.MONTH_OF_YEAR);return MathUtil.intDiv(t+2,3)}adjustInto(e,t){const r=this.getFrom(e);return this.range().checkValidValue(t,this),e.with(ChronoField.MONTH_OF_YEAR,e.getLong(ChronoField.MONTH_OF_YEAR)+3*(t-r))}}class WEEK_OF_WEEK_BASED_YEAR_FIELD extends Field{toString(){return"WeekOfWeekBasedYear"}baseUnit(){return ChronoUnit.WEEKS}rangeUnit(){return WEEK_BASED_YEARS}range(){return ValueRange.of(1,52,53)}isSupportedBy(e){return e.isSupported(ChronoField.EPOCH_DAY)&&this._isIso(e)}rangeRefinedBy(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");return Field._getWeekRangeByLocalDate(LocalDate.from(e))}getFrom(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");return Field._getWeek(LocalDate.from(e))}adjustInto(e,t){return this.range().checkValidValue(t,this),e.plus(MathUtil.safeSubtract(t,this.getFrom(e)),ChronoUnit.WEEKS)}resolve(e,t,r){const o=e.get(WEEK_BASED_YEAR),n=e.get(ChronoField.DAY_OF_WEEK);if(null==o||null==n)return null;const i=WEEK_BASED_YEAR.range().checkValidIntValue(o,WEEK_BASED_YEAR),a=e.get(WEEK_OF_WEEK_BASED_YEAR);let E;if(r===ResolverStyle.LENIENT){let e=n,t=0;e>7?(t=MathUtil.intDiv(e-1,7),e=MathUtil.intMod(e-1,7)+1):e<1&&(t=MathUtil.intDiv(e,7)-1,e=MathUtil.intMod(e,7)+7),E=LocalDate.of(i,1,4).plusWeeks(a-1).plusWeeks(t).with(ChronoField.DAY_OF_WEEK,e)}else{const e=ChronoField.DAY_OF_WEEK.checkValidIntValue(n);if(r===ResolverStyle.STRICT){const e=LocalDate.of(i,1,4);Field._getWeekRangeByLocalDate(e).checkValidValue(a,this)}else this.range().checkValidValue(a,this);E=LocalDate.of(i,1,4).plusWeeks(a-1).with(ChronoField.DAY_OF_WEEK,e)}return e.remove(this),e.remove(WEEK_BASED_YEAR),e.remove(ChronoField.DAY_OF_WEEK),E}displayName(){return"Week"}}class WEEK_BASED_YEAR_FIELD extends Field{toString(){return"WeekBasedYear"}baseUnit(){return WEEK_BASED_YEARS}rangeUnit(){return ChronoUnit.FOREVER}range(){return ChronoField.YEAR.range()}isSupportedBy(e){return e.isSupported(ChronoField.EPOCH_DAY)&&this._isIso(e)}rangeRefinedBy(e){return ChronoField.YEAR.range()}getFrom(e){if(!1===e.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");return Field._getWeekBasedYear(LocalDate.from(e))}adjustInto(e,t){if(!1===this.isSupportedBy(e))throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");const r=this.range().checkValidIntValue(t,WEEK_BASED_YEAR),o=LocalDate.from(e),n=o.get(ChronoField.DAY_OF_WEEK);let i=Field._getWeek(o);53===i&&52===Field._getWeekRangeByYear(r)&&(i=52);let a=LocalDate.of(r,1,4);const E=n-a.get(ChronoField.DAY_OF_WEEK)+7*(i-1);return a=a.plusDays(E),e.with(a)}}class Unit extends TemporalUnit{constructor(e,t){super(),this._name=e,this._duration=t}duration(){return this._duration}isDurationEstimated(){return!0}isDateBased(){return!0}isTimeBased(){return!1}isSupportedBy(e){return e.isSupported(ChronoField.EPOCH_DAY)}addTo(e,t){switch(this){case WEEK_BASED_YEARS:{const r=MathUtil.safeAdd(e.get(WEEK_BASED_YEAR),t);return e.with(WEEK_BASED_YEAR,r)}case QUARTER_YEARS:return e.plus(MathUtil.intDiv(t,256),ChronoUnit.YEARS).plus(3*MathUtil.intMod(t,256),ChronoUnit.MONTHS);default:throw new IllegalStateException("Unreachable")}}between(e,t){switch(this){case WEEK_BASED_YEARS:return MathUtil.safeSubtract(t.getLong(WEEK_BASED_YEAR),e.getLong(WEEK_BASED_YEAR));case QUARTER_YEARS:return MathUtil.intDiv(e.until(t,ChronoUnit.MONTHS),3);default:throw new IllegalStateException("Unreachable")}}toString(){return this._name}}let DAY_OF_QUARTER=null,QUARTER_OF_YEAR=null,WEEK_OF_WEEK_BASED_YEAR=null,WEEK_BASED_YEAR=null,WEEK_BASED_YEARS=null,QUARTER_YEARS=null;export function _init(){DAY_OF_QUARTER=new DAY_OF_QUARTER_FIELD,QUARTER_OF_YEAR=new QUARTER_OF_YEAR_FIELD,WEEK_OF_WEEK_BASED_YEAR=new WEEK_OF_WEEK_BASED_YEAR_FIELD,WEEK_BASED_YEAR=new WEEK_BASED_YEAR_FIELD,WEEK_BASED_YEARS=new Unit("WeekBasedYears",Duration.ofSeconds(31556952)),QUARTER_YEARS=new Unit("QuarterYears",Duration.ofSeconds(7889238)),IsoFields.DAY_OF_QUARTER=DAY_OF_QUARTER,IsoFields.QUARTER_OF_YEAR=QUARTER_OF_YEAR,IsoFields.WEEK_OF_WEEK_BASED_YEAR=WEEK_OF_WEEK_BASED_YEAR,IsoFields.WEEK_BASED_YEAR=WEEK_BASED_YEAR,IsoFields.WEEK_BASED_YEARS=WEEK_BASED_YEARS,IsoFields.QUARTER_YEARS=QUARTER_YEARS,LocalDate.prototype.isoWeekOfWeekyear=function(){return this.get(IsoFields.WEEK_OF_WEEK_BASED_YEAR)},LocalDate.prototype.isoWeekyear=function(){return this.get(IsoFields.WEEK_BASED_YEAR)}}