/*!
 * Copyright (c) 2015-2020, Salesforce.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Salesforce.com nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
"use strict";const punycode=require("punycode/"),urlParse=require("url-parse"),pubsuffix=require("./pubsuffix-psl"),Store=require("./store").Store,MemoryCookieStore=require("./memstore").MemoryCookieStore,pathMatch=require("./pathMatch").pathMatch,validators=require("./validators.js"),VERSION=require("./version"),{fromCallback:fromCallback}=require("universalify"),{getCustomInspectSymbol:getCustomInspectSymbol}=require("./utilHelper"),COOKIE_OCTETS=/^[\x21\x23-\x2B\x2D-\x3A\x3C-\x5B\x5D-\x7E]+$/,CONTROL_CHARS=/[\x00-\x1F]/,TERMINATORS=["\n","\r","\0"],PATH_VALUE=/[\x20-\x3A\x3C-\x7E]+/,DATE_DELIM=/[\x09\x20-\x2F\x3B-\x40\x5B-\x60\x7B-\x7E]/,MONTH_TO_NUM={jan:0,feb:1,mar:2,apr:3,may:4,jun:5,jul:6,aug:7,sep:8,oct:9,nov:10,dec:11},MAX_TIME=2147483647e3,MIN_TIME=0,SAME_SITE_CONTEXT_VAL_ERR='Invalid sameSiteContext option for getCookies(); expected one of "strict", "lax", or "none"';function checkSameSiteContext(e){validators.validate(validators.isNonEmptyString(e),e);const t=String(e).toLowerCase();return"none"===t||"lax"===t||"strict"===t?t:null}const PrefixSecurityEnum=Object.freeze({SILENT:"silent",STRICT:"strict",DISABLED:"unsafe-disabled"}),IP_REGEX_LOWERCASE=/(?:^(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}$)|(?:^(?:(?:[a-f\d]{1,4}:){7}(?:[a-f\d]{1,4}|:)|(?:[a-f\d]{1,4}:){6}(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|:[a-f\d]{1,4}|:)|(?:[a-f\d]{1,4}:){5}(?::(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,2}|:)|(?:[a-f\d]{1,4}:){4}(?:(?::[a-f\d]{1,4}){0,1}:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,3}|:)|(?:[a-f\d]{1,4}:){3}(?:(?::[a-f\d]{1,4}){0,2}:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,4}|:)|(?:[a-f\d]{1,4}:){2}(?:(?::[a-f\d]{1,4}){0,3}:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,5}|:)|(?:[a-f\d]{1,4}:){1}(?:(?::[a-f\d]{1,4}){0,4}:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,6}|:)|(?::(?:(?::[a-f\d]{1,4}){0,5}:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)){3}|(?::[a-f\d]{1,4}){1,7}|:)))$)/,IP_V6_REGEX="\n\\[?(?:\n(?:[a-fA-F\\d]{1,4}:){7}(?:[a-fA-F\\d]{1,4}|:)|\n(?:[a-fA-F\\d]{1,4}:){6}(?:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|:[a-fA-F\\d]{1,4}|:)|\n(?:[a-fA-F\\d]{1,4}:){5}(?::(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,2}|:)|\n(?:[a-fA-F\\d]{1,4}:){4}(?:(?::[a-fA-F\\d]{1,4}){0,1}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,3}|:)|\n(?:[a-fA-F\\d]{1,4}:){3}(?:(?::[a-fA-F\\d]{1,4}){0,2}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,4}|:)|\n(?:[a-fA-F\\d]{1,4}:){2}(?:(?::[a-fA-F\\d]{1,4}){0,3}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,5}|:)|\n(?:[a-fA-F\\d]{1,4}:){1}(?:(?::[a-fA-F\\d]{1,4}){0,4}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,6}|:)|\n(?::(?:(?::[a-fA-F\\d]{1,4}){0,5}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,7}|:))\n)(?:%[0-9a-zA-Z]{1,})?\\]?\n".replace(/\s*\/\/.*$/gm,"").replace(/\n/g,"").trim(),IP_V6_REGEX_OBJECT=new RegExp(`^${IP_V6_REGEX}$`);function parseDigits(e,t,i,o){let r=0;for(;r<e.length;){const t=e.charCodeAt(r);if(t<=47||t>=58)break;r++}return r<t||r>i?null:o||r==e.length?parseInt(e.substr(0,r),10):null}function parseTime(e){const t=e.split(":"),i=[0,0,0];if(3!==t.length)return null;for(let e=0;e<3;e++){const o=2==e,r=parseDigits(t[e],1,2,o);if(null===r)return null;i[e]=r}return i}function parseMonth(e){e=String(e).substr(0,3).toLowerCase();const t=MONTH_TO_NUM[e];return t>=0?t:null}function parseDate(e){if(!e)return;const t=e.split(DATE_DELIM);if(!t)return;let i=null,o=null,r=null,n=null,s=null,a=null;for(let e=0;e<t.length;e++){const l=t[e].trim();if(!l.length)continue;let u;null===r&&(u=parseTime(l),u)?(i=u[0],o=u[1],r=u[2]):null!==n||(u=parseDigits(l,1,2,!0),null===u)?null!==s||(u=parseMonth(l),null===u)?null===a&&(u=parseDigits(l,2,4,!0),null!==u&&(a=u,a>=70&&a<=99?a+=1900:a>=0&&a<=69&&(a+=2e3))):s=u:n=u}return null===n||null===s||null===a||null===r||n<1||n>31||a<1601||i>23||o>59||r>59?void 0:new Date(Date.UTC(a,s,n,i,o,r))}function formatDate(e){return validators.validate(validators.isDate(e),e),e.toUTCString()}function canonicalDomain(e){return null==e?null:(e=e.trim().replace(/^\./,""),IP_V6_REGEX_OBJECT.test(e)&&(e=e.replace("[","").replace("]","")),punycode&&/[^\u0001-\u007f]/.test(e)&&(e=punycode.toASCII(e)),e.toLowerCase())}function domainMatch(e,t,i){if(null==e||null==t)return null;if(!1!==i&&(e=canonicalDomain(e),t=canonicalDomain(t)),e==t)return!0;const o=e.lastIndexOf(t);return!(o<=0)&&(e.length===t.length+o&&("."===e.substr(o-1,1)&&!IP_REGEX_LOWERCASE.test(e)))}function defaultPath(e){if(!e||"/"!==e.substr(0,1))return"/";if("/"===e)return e;const t=e.lastIndexOf("/");return 0===t?"/":e.slice(0,t)}function trimTerminator(e){if(validators.isEmptyString(e))return e;for(let t=0;t<TERMINATORS.length;t++){const i=e.indexOf(TERMINATORS[t]);-1!==i&&(e=e.substr(0,i))}return e}function parseCookiePair(e,t){e=trimTerminator(e),validators.validate(validators.isString(e),e);let i,o,r=e.indexOf("=");if(t)0===r&&(r=(e=e.substr(1)).indexOf("="));else if(r<=0)return;if(r<=0?(i="",o=e.trim()):(i=e.substr(0,r).trim(),o=e.substr(r+1).trim()),CONTROL_CHARS.test(i)||CONTROL_CHARS.test(o))return;const n=new Cookie;return n.key=i,n.value=o,n}function parse(e,t){if(t&&"object"==typeof t||(t={}),validators.isEmptyString(e)||!validators.isString(e))return null;const i=(e=e.trim()).indexOf(";"),o=parseCookiePair(-1===i?e:e.substr(0,i),!!t.loose);if(!o)return;if(-1===i)return o;const r=e.slice(i+1).trim();if(0===r.length)return o;const n=r.split(";");for(;n.length;){const e=n.shift().trim();if(0===e.length)continue;const t=e.indexOf("=");let i,r;switch(-1===t?(i=e,r=null):(i=e.substr(0,t),r=e.substr(t+1)),i=i.trim().toLowerCase(),r&&(r=r.trim()),i){case"expires":if(r){const e=parseDate(r);e&&(o.expires=e)}break;case"max-age":if(r&&/^-?[0-9]+$/.test(r)){const e=parseInt(r,10);o.setMaxAge(e)}break;case"domain":if(r){const e=r.trim().replace(/^\./,"");e&&(o.domain=e.toLowerCase())}break;case"path":o.path=r&&"/"===r[0]?r:null;break;case"secure":o.secure=!0;break;case"httponly":o.httpOnly=!0;break;case"samesite":switch(r?r.toLowerCase():""){case"strict":o.sameSite="strict";break;case"lax":o.sameSite="lax";break;case"none":o.sameSite="none";break;default:o.sameSite=void 0}break;default:o.extensions=o.extensions||[],o.extensions.push(e)}}return o}function isSecurePrefixConditionMet(e){return validators.validate(validators.isObject(e),e),!e.key.startsWith("__Secure-")||e.secure}function isHostPrefixConditionMet(e){return validators.validate(validators.isObject(e)),!e.key.startsWith("__Host-")||e.secure&&e.hostOnly&&null!=e.path&&"/"===e.path}function jsonParse(e){let t;try{t=JSON.parse(e)}catch(e){return e}return t}function fromJSON(e){if(!e||validators.isEmptyString(e))return null;let t;if("string"==typeof e){if(t=jsonParse(e),t instanceof Error)return null}else t=e;const i=new Cookie;for(let e=0;e<Cookie.serializableProperties.length;e++){const o=Cookie.serializableProperties[e];void 0!==t[o]&&t[o]!==cookieDefaults[o]&&("expires"===o||"creation"===o||"lastAccessed"===o?null===t[o]?i[o]=null:i[o]="Infinity"==t[o]?"Infinity":new Date(t[o]):i[o]=t[o])}return i}function cookieCompare(e,t){validators.validate(validators.isObject(e),e),validators.validate(validators.isObject(t),t);let i=0;const o=e.path?e.path.length:0;if(i=(t.path?t.path.length:0)-o,0!==i)return i;return i=(e.creation?e.creation.getTime():MAX_TIME)-(t.creation?t.creation.getTime():MAX_TIME),0!==i||(i=e.creationIndex-t.creationIndex),i}function permutePath(e){if(validators.validate(validators.isString(e)),"/"===e)return["/"];const t=[e];for(;e.length>1;){const i=e.lastIndexOf("/");if(0===i)break;e=e.substr(0,i),t.push(e)}return t.push("/"),t}function getCookieContext(e){if(e instanceof Object)return e;try{e=decodeURI(e)}catch(e){}return urlParse(e)}const cookieDefaults={key:"",value:"",expires:"Infinity",maxAge:null,domain:null,path:null,secure:!1,httpOnly:!1,extensions:null,hostOnly:null,pathIsDefault:null,creation:null,lastAccessed:null,sameSite:void 0};class Cookie{constructor(e={}){const t=getCustomInspectSymbol();t&&(this[t]=this.inspect),Object.assign(this,cookieDefaults,e),this.creation=this.creation||new Date,Object.defineProperty(this,"creationIndex",{configurable:!1,enumerable:!1,writable:!0,value:++Cookie.cookiesCreated})}inspect(){const e=Date.now(),t=null!=this.hostOnly?this.hostOnly:"?",i=this.creation?e-this.creation.getTime()+"ms":"?",o=this.lastAccessed?e-this.lastAccessed.getTime()+"ms":"?";return`Cookie="${this.toString()}; hostOnly=${t}; aAge=${o}; cAge=${i}"`}toJSON(){const e={};for(const t of Cookie.serializableProperties)this[t]!==cookieDefaults[t]&&("expires"===t||"creation"===t||"lastAccessed"===t?null===this[t]?e[t]=null:e[t]="Infinity"==this[t]?"Infinity":this[t].toISOString():"maxAge"===t?null!==this[t]&&(e[t]=this[t]==1/0||this[t]==-1/0?this[t].toString():this[t]):this[t]!==cookieDefaults[t]&&(e[t]=this[t]));return e}clone(){return fromJSON(this.toJSON())}validate(){if(!COOKIE_OCTETS.test(this.value))return!1;if(!(this.expires==1/0||this.expires instanceof Date||parseDate(this.expires)))return!1;if(null!=this.maxAge&&this.maxAge<=0)return!1;if(null!=this.path&&!PATH_VALUE.test(this.path))return!1;const e=this.cdomain();if(e){if(e.match(/\.$/))return!1;if(null==pubsuffix.getPublicSuffix(e))return!1}return!0}setExpires(e){e instanceof Date?this.expires=e:this.expires=parseDate(e)||"Infinity"}setMaxAge(e){this.maxAge=e===1/0||e===-1/0?e.toString():e}cookieString(){let e=this.value;return null==e&&(e=""),""===this.key?e:`${this.key}=${e}`}toString(){let e=this.cookieString();if(this.expires!=1/0&&(this.expires instanceof Date?e+=`; Expires=${formatDate(this.expires)}`:e+=`; Expires=${this.expires}`),null!=this.maxAge&&this.maxAge!=1/0&&(e+=`; Max-Age=${this.maxAge}`),this.domain&&!this.hostOnly&&(e+=`; Domain=${this.domain}`),this.path&&(e+=`; Path=${this.path}`),this.secure&&(e+="; Secure"),this.httpOnly&&(e+="; HttpOnly"),this.sameSite&&"none"!==this.sameSite){const t=Cookie.sameSiteCanonical[this.sameSite.toLowerCase()];e+=`; SameSite=${t||this.sameSite}`}return this.extensions&&this.extensions.forEach((t=>{e+=`; ${t}`})),e}TTL(e){if(null!=this.maxAge)return this.maxAge<=0?0:1e3*this.maxAge;let t=this.expires;return t!=1/0?(t instanceof Date||(t=parseDate(t)||1/0),t==1/0?1/0:t.getTime()-(e||Date.now())):1/0}expiryTime(e){if(null!=this.maxAge){const t=e||this.creation||new Date,i=this.maxAge<=0?-1/0:1e3*this.maxAge;return t.getTime()+i}return this.expires==1/0?1/0:this.expires.getTime()}expiryDate(e){const t=this.expiryTime(e);return t==1/0?new Date(MAX_TIME):t==-1/0?new Date(0):new Date(t)}isPersistent(){return null!=this.maxAge||this.expires!=1/0}canonicalizedDomain(){return null==this.domain?null:canonicalDomain(this.domain)}cdomain(){return this.canonicalizedDomain()}}function getNormalizedPrefixSecurity(e){if(null!=e){const t=e.toLowerCase();switch(t){case PrefixSecurityEnum.STRICT:case PrefixSecurityEnum.SILENT:case PrefixSecurityEnum.DISABLED:return t}}return PrefixSecurityEnum.SILENT}Cookie.cookiesCreated=0,Cookie.parse=parse,Cookie.fromJSON=fromJSON,Cookie.serializableProperties=Object.keys(cookieDefaults),Cookie.sameSiteLevel={strict:3,lax:2,none:1},Cookie.sameSiteCanonical={strict:"Strict",lax:"Lax"};class CookieJar{constructor(e,t={rejectPublicSuffixes:!0}){"boolean"==typeof t&&(t={rejectPublicSuffixes:t}),validators.validate(validators.isObject(t),t),this.rejectPublicSuffixes=t.rejectPublicSuffixes,this.enableLooseMode=!!t.looseMode,this.allowSpecialUseDomain="boolean"!=typeof t.allowSpecialUseDomain||t.allowSpecialUseDomain,this.store=e||new MemoryCookieStore,this.prefixSecurity=getNormalizedPrefixSecurity(t.prefixSecurity),this._cloneSync=syncWrap("clone"),this._importCookiesSync=syncWrap("_importCookies"),this.getCookiesSync=syncWrap("getCookies"),this.getCookieStringSync=syncWrap("getCookieString"),this.getSetCookieStringsSync=syncWrap("getSetCookieStrings"),this.removeAllCookiesSync=syncWrap("removeAllCookies"),this.setCookieSync=syncWrap("setCookie"),this.serializeSync=syncWrap("serialize")}setCookie(e,t,i,o){let r;if(validators.validate(validators.isUrlStringOrObject(t),o,i),validators.isFunction(t))return(o=t)(new Error("No URL was specified"));const n=getCookieContext(t);if(validators.isFunction(i)&&(o=i,i={}),validators.validate(validators.isFunction(o),o),!validators.isNonEmptyString(e)&&!validators.isObject(e)&&e instanceof String&&0==e.length)return o(null);const s=canonicalDomain(n.hostname),a=i.loose||this.enableLooseMode;let l=null;if(i.sameSiteContext&&(l=checkSameSiteContext(i.sameSiteContext),!l))return o(new Error(SAME_SITE_CONTEXT_VAL_ERR));if("string"==typeof e||e instanceof String){if(!(e=Cookie.parse(e,{loose:a})))return r=new Error("Cookie failed to parse"),o(i.ignoreError?null:r)}else if(!(e instanceof Cookie))return r=new Error("First argument to setCookie must be a Cookie object or string"),o(i.ignoreError?null:r);const u=i.now||new Date;if(this.rejectPublicSuffixes&&e.domain){if(null==pubsuffix.getPublicSuffix(e.cdomain(),{allowSpecialUseDomain:this.allowSpecialUseDomain,ignoreError:i.ignoreError})&&!IP_V6_REGEX_OBJECT.test(e.domain))return r=new Error("Cookie has domain set to a public suffix"),o(i.ignoreError?null:r)}if(e.domain){if(!domainMatch(s,e.cdomain(),!1))return r=new Error(`Cookie not in this host's domain. Cookie:${e.cdomain()} Request:${s}`),o(i.ignoreError?null:r);null==e.hostOnly&&(e.hostOnly=!1)}else e.hostOnly=!0,e.domain=s;if(e.path&&"/"===e.path[0]||(e.path=defaultPath(n.pathname),e.pathIsDefault=!0),!1===i.http&&e.httpOnly)return r=new Error("Cookie is HttpOnly and this isn't an HTTP API"),o(i.ignoreError?null:r);if("none"!==e.sameSite&&void 0!==e.sameSite&&l&&"none"===l)return r=new Error("Cookie is SameSite but this is a cross-origin request"),o(i.ignoreError?null:r);const c=this.prefixSecurity===PrefixSecurityEnum.SILENT;if(!(this.prefixSecurity===PrefixSecurityEnum.DISABLED)){let t,r=!1;if(isSecurePrefixConditionMet(e)?isHostPrefixConditionMet(e)||(r=!0,t="Cookie has __Host prefix but either Secure or HostOnly attribute is not set or Path is not '/'"):(r=!0,t="Cookie has __Secure prefix but Secure attribute is not set"),r)return o(i.ignoreError||c?null:new Error(t))}const d=this.store;d.updateCookie||(d.updateCookie=function(e,t,i){this.putCookie(t,i)}),d.findCookie(e.domain,e.path,e.key,(function(t,r){if(t)return o(t);const n=function(t){if(t)return o(t);o(null,e)};if(r){if(!1===i.http&&r.httpOnly)return t=new Error("old Cookie is HttpOnly and this isn't an HTTP API"),o(i.ignoreError?null:t);e.creation=r.creation,e.creationIndex=r.creationIndex,e.lastAccessed=u,d.updateCookie(r,e,n)}else e.creation=e.lastAccessed=u,d.putCookie(e,n)}))}getCookies(e,t,i){validators.validate(validators.isUrlStringOrObject(e),i,e);const o=getCookieContext(e);validators.isFunction(t)&&(i=t,t={}),validators.validate(validators.isObject(t),i,t),validators.validate(validators.isFunction(i),i);const r=canonicalDomain(o.hostname),n=o.pathname||"/";let s=t.secure;null!=s||!o.protocol||"https:"!=o.protocol&&"wss:"!=o.protocol||(s=!0);let a=0;if(t.sameSiteContext){const e=checkSameSiteContext(t.sameSiteContext);if(a=Cookie.sameSiteLevel[e],!a)return i(new Error(SAME_SITE_CONTEXT_VAL_ERR))}let l=t.http;null==l&&(l=!0);const u=t.now||Date.now(),c=!1!==t.expire,d=!!t.allPaths,f=this.store;function h(e){if(e.hostOnly){if(e.domain!=r)return!1}else if(!domainMatch(r,e.domain,!1))return!1;if(!d&&!pathMatch(n,e.path))return!1;if(e.secure&&!s)return!1;if(e.httpOnly&&!l)return!1;if(a){if(Cookie.sameSiteLevel[e.sameSite||"none"]>a)return!1}return!(c&&e.expiryTime()<=u)||(f.removeCookie(e.domain,e.path,e.key,(()=>{})),!1)}f.findCookies(r,d?null:n,this.allowSpecialUseDomain,((e,o)=>{if(e)return i(e);o=o.filter(h),!1!==t.sort&&(o=o.sort(cookieCompare));const r=new Date;for(const e of o)e.lastAccessed=r;i(null,o)}))}getCookieString(...e){const t=e.pop();validators.validate(validators.isFunction(t),t);e.push((function(e,i){e?t(e):t(null,i.sort(cookieCompare).map((e=>e.cookieString())).join("; "))})),this.getCookies.apply(this,e)}getSetCookieStrings(...e){const t=e.pop();validators.validate(validators.isFunction(t),t);e.push((function(e,i){e?t(e):t(null,i.map((e=>e.toString())))})),this.getCookies.apply(this,e)}serialize(e){validators.validate(validators.isFunction(e),e);let t=this.store.constructor.name;validators.isObject(t)&&(t=null);const i={version:`tough-cookie@${VERSION}`,storeType:t,rejectPublicSuffixes:!!this.rejectPublicSuffixes,enableLooseMode:!!this.enableLooseMode,allowSpecialUseDomain:!!this.allowSpecialUseDomain,prefixSecurity:getNormalizedPrefixSecurity(this.prefixSecurity),cookies:[]};if(!this.store.getAllCookies||"function"!=typeof this.store.getAllCookies)return e(new Error("store does not support getAllCookies and cannot be serialized"));this.store.getAllCookies(((t,o)=>t?e(t):(i.cookies=o.map((e=>(delete(e=e instanceof Cookie?e.toJSON():e).creationIndex,e))),e(null,i))))}toJSON(){return this.serializeSync()}_importCookies(e,t){let i=e.cookies;if(!i||!Array.isArray(i))return t(new Error("serialized jar has no cookies array"));i=i.slice();const o=e=>{if(e)return t(e);if(!i.length)return t(e,this);let r;try{r=fromJSON(i.shift())}catch(e){return t(e)}if(null===r)return o(null);this.store.putCookie(r,o)};o()}clone(e,t){1===arguments.length&&(t=e,e=null),this.serialize(((i,o)=>{if(i)return t(i);CookieJar.deserialize(o,e,t)}))}cloneSync(e){if(0===arguments.length)return this._cloneSync();if(!e.synchronous)throw new Error("CookieJar clone destination store is not synchronous; use async API instead.");return this._cloneSync(e)}removeAllCookies(e){validators.validate(validators.isFunction(e),e);const t=this.store;if("function"==typeof t.removeAllCookies&&t.removeAllCookies!==Store.prototype.removeAllCookies)return t.removeAllCookies(e);t.getAllCookies(((i,o)=>{if(i)return e(i);if(0===o.length)return e(null);let r=0;const n=[];function s(t){if(t&&n.push(t),r++,r===o.length)return e(n.length?n[0]:null)}o.forEach((e=>{t.removeCookie(e.domain,e.path,e.key,s)}))}))}static deserialize(e,t,i){let o;if(3!==arguments.length&&(i=t,t=null),validators.validate(validators.isFunction(i),i),"string"==typeof e){if(o=jsonParse(e),o instanceof Error)return i(o)}else o=e;const r=new CookieJar(t,{rejectPublicSuffixes:o.rejectPublicSuffixes,looseMode:o.enableLooseMode,allowSpecialUseDomain:o.allowSpecialUseDomain,prefixSecurity:o.prefixSecurity});r._importCookies(o,(e=>{if(e)return i(e);i(null,r)}))}static deserializeSync(e,t){const i="string"==typeof e?JSON.parse(e):e,o=new CookieJar(t,{rejectPublicSuffixes:i.rejectPublicSuffixes,looseMode:i.enableLooseMode});if(!o.store.synchronous)throw new Error("CookieJar store is not synchronous; use async API instead.");return o._importCookiesSync(i),o}}function syncWrap(e){return function(...t){if(!this.store.synchronous)throw new Error("CookieJar store is not synchronous; use async API instead.");let i,o;if(this[e](...t,((e,t)=>{i=e,o=t})),i)throw i;return o}}CookieJar.fromJSON=CookieJar.deserializeSync,["_importCookies","clone","getCookies","getCookieString","getSetCookieStrings","removeAllCookies","serialize","setCookie"].forEach((e=>{CookieJar.prototype[e]=fromCallback(CookieJar.prototype[e])})),CookieJar.deserialize=fromCallback(CookieJar.deserialize),exports.version=VERSION,exports.CookieJar=CookieJar,exports.Cookie=Cookie,exports.Store=Store,exports.MemoryCookieStore=MemoryCookieStore,exports.parseDate=parseDate,exports.formatDate=formatDate,exports.parse=parse,exports.fromJSON=fromJSON,exports.domainMatch=domainMatch,exports.defaultPath=defaultPath,exports.pathMatch=pathMatch,exports.getPublicSuffix=pubsuffix.getPublicSuffix,exports.cookieCompare=cookieCompare,exports.permuteDomain=require("./permuteDomain").permuteDomain,exports.permutePath=permutePath,exports.canonicalDomain=canonicalDomain,exports.PrefixSecurityEnum=PrefixSecurityEnum,exports.ParameterError=validators.ParameterError;